"""Top-level package for Pydantic QuantLib."""
from __future__ import annotations

__author__ = """George EC Burton"""
__email__ = "g.e.c.burton@gmail.com"
__version__ = "0.1.0"


# generated by datamodel-codegen:
#   filename:  schema.json
#   timestamp: 2021-01-03T20:14:28+00:00


from typing import Any, List, Literal, Optional, Union

from pydantic import Field, conint

from . import enums
from .core import BaseModel


class Period(BaseModel):
    resource_name: Optional[Literal["Period"]] = "Period"
    n: Optional[int] = None
    units: Optional[enums.TimeUnit] = None


class Date(BaseModel):
    resource_name: Optional[Literal["Date"]] = "Date"
    d: conint(ge=1, le=31)  # type: ignore
    m: conint(ge=1, le=12)  # type: ignore
    y: conint(ge=1900, le=2999)  # type: ignore


class EuropeanExercise(BaseModel):
    resource_name: Optional[Literal["EuropeanExercise"]] = "EuropeanExercise"
    date: Date


class AmericanExercise(BaseModel):
    resource_name: Optional[Literal["AmericanExercise"]] = "AmericanExercise"
    earliestDate: Date
    latestDate: Date
    payoffAtExpiry: Optional[bool] = None


class BermudanExercise(BaseModel):
    resource_name: Optional[Literal["BermudanExercise"]] = "BermudanExercise"
    dates: List[Date]
    payoffAtExpiry: Optional[bool] = None


class SwingExercise(BaseModel):
    resource_name: Optional[Literal["SwingExercise"]] = "SwingExercise"
    dates: List[Date]


class Array(BaseModel):
    resource_name: Optional[Literal["Array"]] = "Array"
    n: Optional[int] = None
    fill: Optional[float] = None


class Matrix(BaseModel):
    resource_name: Optional[Literal["Matrix"]] = "Matrix"
    rows: Optional[int] = None
    columns: Optional[int] = None
    fill: Optional[float] = None


class SVD(BaseModel):
    resource_name: Optional[Literal["SVD"]] = "SVD"
    value: Matrix


class BiCGstab(BaseModel):
    resource_name: Optional[Literal["BiCGstab"]] = "BiCGstab"


class GMRES(BaseModel):
    resource_name: Optional[Literal["GMRES"]] = "GMRES"


class Quote(BaseModel):
    resource_name: Optional[Literal["Quote"]] = "Quote"


class QuoteHandle(BaseModel):
    resource_name: Optional[Literal["QuoteHandle"]] = "QuoteHandle"
    value: Optional[QUOTE] = None


class SimpleQuote(BaseModel):
    resource_name: Optional[Literal["SimpleQuote"]] = "SimpleQuote"
    value: float


class DerivedQuote(BaseModel):
    resource_name: Optional[Literal["DerivedQuote"]] = "DerivedQuote"


class CompositeQuote(BaseModel):
    resource_name: Optional[Literal["CompositeQuote"]] = "CompositeQuote"


class Calendar(BaseModel):
    resource_name: Optional[Literal["Calendar"]] = "Calendar"


class Argentina(BaseModel):
    resource_name: Optional[Literal["Argentina"]] = "Argentina"
    m: Optional[enums.ArgentinaMarket] = None


class Australia(BaseModel):
    resource_name: Optional[Literal["Australia"]] = "Australia"


class Brazil(BaseModel):
    resource_name: Optional[Literal["Brazil"]] = "Brazil"
    m: Optional[enums.BrazilMarket] = None


class Canada(BaseModel):
    resource_name: Optional[Literal["Canada"]] = "Canada"
    m: Optional[enums.CanadaMarket] = None


class China(BaseModel):
    resource_name: Optional[Literal["China"]] = "China"
    m: Optional[enums.ChinaMarket] = None


class CzechRepublic(BaseModel):
    resource_name: Optional[Literal["CzechRepublic"]] = "CzechRepublic"
    m: Optional[enums.CzechMarket] = None


class Denmark(BaseModel):
    resource_name: Optional[Literal["Denmark"]] = "Denmark"


class Finland(BaseModel):
    resource_name: Optional[Literal["Finland"]] = "Finland"


class France(BaseModel):
    resource_name: Optional[Literal["France"]] = "France"
    m: Optional[enums.FranceMarket] = None


class Germany(BaseModel):
    resource_name: Optional[Literal["Germany"]] = "Germany"
    m: Optional[enums.GermanyMarket] = None


class HongKong(BaseModel):
    resource_name: Optional[Literal["HongKong"]] = "HongKong"
    m: Optional[enums.HongKongMarket] = None


class Hungary(BaseModel):
    resource_name: Optional[Literal["Hungary"]] = "Hungary"


class Iceland(BaseModel):
    resource_name: Optional[Literal["Iceland"]] = "Iceland"
    m: Optional[enums.IcelandMarket] = None


class India(BaseModel):
    resource_name: Optional[Literal["India"]] = "India"
    m: Optional[enums.IndiaMarket] = None


class Indonesia(BaseModel):
    resource_name: Optional[Literal["Indonesia"]] = "Indonesia"
    m: Optional[enums.IndonesiaMarket] = None


class Israel(BaseModel):
    resource_name: Optional[Literal["Israel"]] = "Israel"
    m: Optional[enums.IsraelMarket] = None


class Italy(BaseModel):
    resource_name: Optional[Literal["Italy"]] = "Italy"
    m: Optional[enums.ItalyMarket] = None


class Japan(BaseModel):
    resource_name: Optional[Literal["Japan"]] = "Japan"


class Mexico(BaseModel):
    resource_name: Optional[Literal["Mexico"]] = "Mexico"
    m: Optional[enums.MexicoMarket] = None


class NewZealand(BaseModel):
    resource_name: Optional[Literal["NewZealand"]] = "NewZealand"


class Norway(BaseModel):
    resource_name: Optional[Literal["Norway"]] = "Norway"


class Poland(BaseModel):
    resource_name: Optional[Literal["Poland"]] = "Poland"


class Russia(BaseModel):
    resource_name: Optional[Literal["Russia"]] = "Russia"
    m: Optional[enums.RussiaMarket] = None


class Romania(BaseModel):
    resource_name: Optional[Literal["Romania"]] = "Romania"


class SaudiArabia(BaseModel):
    resource_name: Optional[Literal["SaudiArabia"]] = "SaudiArabia"
    m: Optional[enums.SaudiArabiaMarket] = None


class Singapore(BaseModel):
    resource_name: Optional[Literal["Singapore"]] = "Singapore"
    m: Optional[enums.SingaporeMarket] = None


class Slovakia(BaseModel):
    resource_name: Optional[Literal["Slovakia"]] = "Slovakia"
    m: Optional[enums.SlovakiaMarket] = None


class SouthAfrica(BaseModel):
    resource_name: Optional[Literal["SouthAfrica"]] = "SouthAfrica"


class SouthKorea(BaseModel):
    resource_name: Optional[Literal["SouthKorea"]] = "SouthKorea"
    m: Optional[enums.SouthKoreaMarket] = None


class Sweden(BaseModel):
    resource_name: Optional[Literal["Sweden"]] = "Sweden"


class Switzerland(BaseModel):
    resource_name: Optional[Literal["Switzerland"]] = "Switzerland"


class Taiwan(BaseModel):
    resource_name: Optional[Literal["Taiwan"]] = "Taiwan"
    m: Optional[enums.TaiwanMarket] = None


class TARGET(BaseModel):
    resource_name: Optional[Literal["TARGET"]] = "TARGET"


class Thailand(BaseModel):
    resource_name: Optional[Literal["Thailand"]] = "Thailand"


class Turkey(BaseModel):
    resource_name: Optional[Literal["Turkey"]] = "Turkey"


class Ukraine(BaseModel):
    resource_name: Optional[Literal["Ukraine"]] = "Ukraine"
    m: Optional[enums.UkraineMarket] = None


class UnitedKingdom(BaseModel):
    resource_name: Optional[Literal["UnitedKingdom"]] = "UnitedKingdom"
    m: Optional[enums.UnitedKingdomMarket] = None


class UnitedStates(BaseModel):
    resource_name: Optional[Literal["UnitedStates"]] = "UnitedStates"
    m: Optional[enums.UnitedStatesMarket] = None


class NullCalendar(BaseModel):
    resource_name: Optional[Literal["NullCalendar"]] = "NullCalendar"


class WeekendsOnly(BaseModel):
    resource_name: Optional[Literal["WeekendsOnly"]] = "WeekendsOnly"


class BespokeCalendar(BaseModel):
    resource_name: Optional[Literal["BespokeCalendar"]] = "BespokeCalendar"
    name: str


class DayCounter(BaseModel):
    resource_name: Optional[Literal["DayCounter"]] = "DayCounter"


class Actual360(BaseModel):
    resource_name: Optional[Literal["Actual360"]] = "Actual360"
    includeLastDay: Optional[bool] = None


class OneDayCounter(BaseModel):
    resource_name: Optional[Literal["OneDayCounter"]] = "OneDayCounter"


class SimpleDayCounter(BaseModel):
    resource_name: Optional[Literal["SimpleDayCounter"]] = "SimpleDayCounter"


class Business252(BaseModel):
    resource_name: Optional[Literal["Business252"]] = "Business252"
    c: Optional[CALENDAR] = None


class InterestRate(BaseModel):
    resource_name: Optional[Literal["InterestRate"]] = "InterestRate"
    r: Optional[float] = None
    dc: Optional[DAYCOUNTER] = None
    comp: Optional[enums.Compounding] = None
    freq: Optional[float] = None


class Rounding(BaseModel):
    resource_name: Optional[Literal["Rounding"]] = "Rounding"


class UpRounding(BaseModel):
    resource_name: Optional[Literal["UpRounding"]] = "UpRounding"
    precision: int
    digit: Optional[int] = None


class DownRounding(BaseModel):
    resource_name: Optional[Literal["DownRounding"]] = "DownRounding"
    precision: int
    digit: Optional[int] = None


class ClosestRounding(BaseModel):
    resource_name: Optional[Literal["ClosestRounding"]] = "ClosestRounding"
    precision: int
    digit: Optional[int] = None


class CeilingTruncation(BaseModel):
    resource_name: Optional[Literal["CeilingTruncation"]] = "CeilingTruncation"
    precision: int
    digit: Optional[int] = None


class FloorTruncation(BaseModel):
    resource_name: Optional[Literal["FloorTruncation"]] = "FloorTruncation"
    precision: int
    digit: Optional[int] = None


class Currency(BaseModel):
    resource_name: Optional[Literal["Currency"]] = "Currency"


class ARSCurrency(BaseModel):
    resource_name: Optional[Literal["ARSCurrency"]] = "ARSCurrency"


class ATSCurrency(BaseModel):
    resource_name: Optional[Literal["ATSCurrency"]] = "ATSCurrency"


class AUDCurrency(BaseModel):
    resource_name: Optional[Literal["AUDCurrency"]] = "AUDCurrency"


class BDTCurrency(BaseModel):
    resource_name: Optional[Literal["BDTCurrency"]] = "BDTCurrency"


class BEFCurrency(BaseModel):
    resource_name: Optional[Literal["BEFCurrency"]] = "BEFCurrency"


class BGLCurrency(BaseModel):
    resource_name: Optional[Literal["BGLCurrency"]] = "BGLCurrency"


class BRLCurrency(BaseModel):
    resource_name: Optional[Literal["BRLCurrency"]] = "BRLCurrency"


class BYRCurrency(BaseModel):
    resource_name: Optional[Literal["BYRCurrency"]] = "BYRCurrency"


class CADCurrency(BaseModel):
    resource_name: Optional[Literal["CADCurrency"]] = "CADCurrency"


class CHFCurrency(BaseModel):
    resource_name: Optional[Literal["CHFCurrency"]] = "CHFCurrency"


class CLPCurrency(BaseModel):
    resource_name: Optional[Literal["CLPCurrency"]] = "CLPCurrency"


class CNYCurrency(BaseModel):
    resource_name: Optional[Literal["CNYCurrency"]] = "CNYCurrency"


class COPCurrency(BaseModel):
    resource_name: Optional[Literal["COPCurrency"]] = "COPCurrency"


class CYPCurrency(BaseModel):
    resource_name: Optional[Literal["CYPCurrency"]] = "CYPCurrency"


class CZKCurrency(BaseModel):
    resource_name: Optional[Literal["CZKCurrency"]] = "CZKCurrency"


class DEMCurrency(BaseModel):
    resource_name: Optional[Literal["DEMCurrency"]] = "DEMCurrency"


class DKKCurrency(BaseModel):
    resource_name: Optional[Literal["DKKCurrency"]] = "DKKCurrency"


class EEKCurrency(BaseModel):
    resource_name: Optional[Literal["EEKCurrency"]] = "EEKCurrency"


class ESPCurrency(BaseModel):
    resource_name: Optional[Literal["ESPCurrency"]] = "ESPCurrency"


class EURCurrency(BaseModel):
    resource_name: Optional[Literal["EURCurrency"]] = "EURCurrency"


class FIMCurrency(BaseModel):
    resource_name: Optional[Literal["FIMCurrency"]] = "FIMCurrency"


class FRFCurrency(BaseModel):
    resource_name: Optional[Literal["FRFCurrency"]] = "FRFCurrency"


class GBPCurrency(BaseModel):
    resource_name: Optional[Literal["GBPCurrency"]] = "GBPCurrency"


class GRDCurrency(BaseModel):
    resource_name: Optional[Literal["GRDCurrency"]] = "GRDCurrency"


class HKDCurrency(BaseModel):
    resource_name: Optional[Literal["HKDCurrency"]] = "HKDCurrency"


class HUFCurrency(BaseModel):
    resource_name: Optional[Literal["HUFCurrency"]] = "HUFCurrency"


class IEPCurrency(BaseModel):
    resource_name: Optional[Literal["IEPCurrency"]] = "IEPCurrency"


class IDRCurrency(BaseModel):
    resource_name: Optional[Literal["IDRCurrency"]] = "IDRCurrency"


class ILSCurrency(BaseModel):
    resource_name: Optional[Literal["ILSCurrency"]] = "ILSCurrency"


class INRCurrency(BaseModel):
    resource_name: Optional[Literal["INRCurrency"]] = "INRCurrency"


class IQDCurrency(BaseModel):
    resource_name: Optional[Literal["IQDCurrency"]] = "IQDCurrency"


class IRRCurrency(BaseModel):
    resource_name: Optional[Literal["IRRCurrency"]] = "IRRCurrency"


class ISKCurrency(BaseModel):
    resource_name: Optional[Literal["ISKCurrency"]] = "ISKCurrency"


class ITLCurrency(BaseModel):
    resource_name: Optional[Literal["ITLCurrency"]] = "ITLCurrency"


class JPYCurrency(BaseModel):
    resource_name: Optional[Literal["JPYCurrency"]] = "JPYCurrency"


class KRWCurrency(BaseModel):
    resource_name: Optional[Literal["KRWCurrency"]] = "KRWCurrency"


class KWDCurrency(BaseModel):
    resource_name: Optional[Literal["KWDCurrency"]] = "KWDCurrency"


class LTLCurrency(BaseModel):
    resource_name: Optional[Literal["LTLCurrency"]] = "LTLCurrency"


class LUFCurrency(BaseModel):
    resource_name: Optional[Literal["LUFCurrency"]] = "LUFCurrency"


class LVLCurrency(BaseModel):
    resource_name: Optional[Literal["LVLCurrency"]] = "LVLCurrency"


class MTLCurrency(BaseModel):
    resource_name: Optional[Literal["MTLCurrency"]] = "MTLCurrency"


class MXNCurrency(BaseModel):
    resource_name: Optional[Literal["MXNCurrency"]] = "MXNCurrency"


class MYRCurrency(BaseModel):
    resource_name: Optional[Literal["MYRCurrency"]] = "MYRCurrency"


class NLGCurrency(BaseModel):
    resource_name: Optional[Literal["NLGCurrency"]] = "NLGCurrency"


class NOKCurrency(BaseModel):
    resource_name: Optional[Literal["NOKCurrency"]] = "NOKCurrency"


class NPRCurrency(BaseModel):
    resource_name: Optional[Literal["NPRCurrency"]] = "NPRCurrency"


class NZDCurrency(BaseModel):
    resource_name: Optional[Literal["NZDCurrency"]] = "NZDCurrency"


class PEHCurrency(BaseModel):
    resource_name: Optional[Literal["PEHCurrency"]] = "PEHCurrency"


class PEICurrency(BaseModel):
    resource_name: Optional[Literal["PEICurrency"]] = "PEICurrency"


class PENCurrency(BaseModel):
    resource_name: Optional[Literal["PENCurrency"]] = "PENCurrency"


class PKRCurrency(BaseModel):
    resource_name: Optional[Literal["PKRCurrency"]] = "PKRCurrency"


class PLNCurrency(BaseModel):
    resource_name: Optional[Literal["PLNCurrency"]] = "PLNCurrency"


class PTECurrency(BaseModel):
    resource_name: Optional[Literal["PTECurrency"]] = "PTECurrency"


class ROLCurrency(BaseModel):
    resource_name: Optional[Literal["ROLCurrency"]] = "ROLCurrency"


class RONCurrency(BaseModel):
    resource_name: Optional[Literal["RONCurrency"]] = "RONCurrency"


class RUBCurrency(BaseModel):
    resource_name: Optional[Literal["RUBCurrency"]] = "RUBCurrency"


class SARCurrency(BaseModel):
    resource_name: Optional[Literal["SARCurrency"]] = "SARCurrency"


class SEKCurrency(BaseModel):
    resource_name: Optional[Literal["SEKCurrency"]] = "SEKCurrency"


class SGDCurrency(BaseModel):
    resource_name: Optional[Literal["SGDCurrency"]] = "SGDCurrency"


class SITCurrency(BaseModel):
    resource_name: Optional[Literal["SITCurrency"]] = "SITCurrency"


class SKKCurrency(BaseModel):
    resource_name: Optional[Literal["SKKCurrency"]] = "SKKCurrency"


class THBCurrency(BaseModel):
    resource_name: Optional[Literal["THBCurrency"]] = "THBCurrency"


class TRLCurrency(BaseModel):
    resource_name: Optional[Literal["TRLCurrency"]] = "TRLCurrency"


class TRYCurrency(BaseModel):
    resource_name: Optional[Literal["TRYCurrency"]] = "TRYCurrency"


class TTDCurrency(BaseModel):
    resource_name: Optional[Literal["TTDCurrency"]] = "TTDCurrency"


class TWDCurrency(BaseModel):
    resource_name: Optional[Literal["TWDCurrency"]] = "TWDCurrency"


class USDCurrency(BaseModel):
    resource_name: Optional[Literal["USDCurrency"]] = "USDCurrency"


class VEBCurrency(BaseModel):
    resource_name: Optional[Literal["VEBCurrency"]] = "VEBCurrency"


class VNDCurrency(BaseModel):
    resource_name: Optional[Literal["VNDCurrency"]] = "VNDCurrency"


class ZARCurrency(BaseModel):
    resource_name: Optional[Literal["ZARCurrency"]] = "ZARCurrency"


class SafeLinearInterpolation(BaseModel):
    resource_name: Optional[
        Literal["SafeLinearInterpolation"]
    ] = "SafeLinearInterpolation"
    x: Array
    y: Array


class SafeLogLinearInterpolation(BaseModel):
    resource_name: Optional[
        Literal["SafeLogLinearInterpolation"]
    ] = "SafeLogLinearInterpolation"
    x: Array
    y: Array


class SafeBackwardFlatInterpolation(BaseModel):
    resource_name: Optional[
        Literal["SafeBackwardFlatInterpolation"]
    ] = "SafeBackwardFlatInterpolation"
    x: Array
    y: Array


class SafeForwardFlatInterpolation(BaseModel):
    resource_name: Optional[
        Literal["SafeForwardFlatInterpolation"]
    ] = "SafeForwardFlatInterpolation"
    x: Array
    y: Array


class SafeCubicNaturalSpline(BaseModel):
    resource_name: Optional[
        Literal["SafeCubicNaturalSpline"]
    ] = "SafeCubicNaturalSpline"
    x: Array
    y: Array


class SafeLogCubicNaturalSpline(BaseModel):
    resource_name: Optional[
        Literal["SafeLogCubicNaturalSpline"]
    ] = "SafeLogCubicNaturalSpline"
    x: Array
    y: Array


class SafeMonotonicCubicNaturalSpline(BaseModel):
    resource_name: Optional[
        Literal["SafeMonotonicCubicNaturalSpline"]
    ] = "SafeMonotonicCubicNaturalSpline"
    x: Array
    y: Array


class SafeMonotonicLogCubicNaturalSpline(BaseModel):
    resource_name: Optional[
        Literal["SafeMonotonicLogCubicNaturalSpline"]
    ] = "SafeMonotonicLogCubicNaturalSpline"
    x: Array
    y: Array


class SafeKrugerCubic(BaseModel):
    resource_name: Optional[Literal["SafeKrugerCubic"]] = "SafeKrugerCubic"
    x: Array
    y: Array


class SafeKrugerLogCubic(BaseModel):
    resource_name: Optional[Literal["SafeKrugerLogCubic"]] = "SafeKrugerLogCubic"
    x: Array
    y: Array


class SafeFritschButlandCubic(BaseModel):
    resource_name: Optional[
        Literal["SafeFritschButlandCubic"]
    ] = "SafeFritschButlandCubic"
    x: Array
    y: Array


class SafeFritschButlandLogCubic(BaseModel):
    resource_name: Optional[
        Literal["SafeFritschButlandLogCubic"]
    ] = "SafeFritschButlandLogCubic"
    x: Array
    y: Array


class SafeParabolic(BaseModel):
    resource_name: Optional[Literal["SafeParabolic"]] = "SafeParabolic"
    x: Array
    y: Array


class SafeLogParabolic(BaseModel):
    resource_name: Optional[Literal["SafeLogParabolic"]] = "SafeLogParabolic"
    x: Array
    y: Array


class SafeMonotonicParabolic(BaseModel):
    resource_name: Optional[
        Literal["SafeMonotonicParabolic"]
    ] = "SafeMonotonicParabolic"
    x: Array
    y: Array


class SafeMonotonicLogParabolic(BaseModel):
    resource_name: Optional[
        Literal["SafeMonotonicLogParabolic"]
    ] = "SafeMonotonicLogParabolic"
    x: Array
    y: Array


class SafeBilinearInterpolation(BaseModel):
    resource_name: Optional[
        Literal["SafeBilinearInterpolation"]
    ] = "SafeBilinearInterpolation"
    x: Array
    y: Array
    m: Matrix


class SafeBicubicSpline(BaseModel):
    resource_name: Optional[Literal["SafeBicubicSpline"]] = "SafeBicubicSpline"
    x: Array
    y: Array
    m: Matrix


class BackwardFlat(BaseModel):
    resource_name: Optional[Literal["BackwardFlat"]] = "BackwardFlat"


class ForwardFlat(BaseModel):
    resource_name: Optional[Literal["ForwardFlat"]] = "ForwardFlat"


class Linear(BaseModel):
    resource_name: Optional[Literal["Linear"]] = "Linear"


class LogLinear(BaseModel):
    resource_name: Optional[Literal["LogLinear"]] = "LogLinear"


class Cubic(BaseModel):
    resource_name: Optional[Literal["Cubic"]] = "Cubic"


class MonotonicCubic(BaseModel):
    resource_name: Optional[Literal["MonotonicCubic"]] = "MonotonicCubic"


class DefaultLogCubic(BaseModel):
    resource_name: Optional[Literal["DefaultLogCubic"]] = "DefaultLogCubic"


class MonotonicLogCubic(BaseModel):
    resource_name: Optional[Literal["MonotonicLogCubic"]] = "MonotonicLogCubic"


class SplineCubic(BaseModel):
    resource_name: Optional[Literal["SplineCubic"]] = "SplineCubic"


class Kruger(BaseModel):
    resource_name: Optional[Literal["Kruger"]] = "Kruger"


class KrugerLog(BaseModel):
    resource_name: Optional[Literal["KrugerLog"]] = "KrugerLog"


class ConvexMonotone(BaseModel):
    resource_name: Optional[Literal["ConvexMonotone"]] = "ConvexMonotone"
    quadraticity: Optional[float] = None
    monotonicity: Optional[float] = None
    forcePositive: Optional[bool] = None


class RichardsonExtrapolation(BaseModel):
    resource_name: Optional[
        Literal["RichardsonExtrapolation"]
    ] = "RichardsonExtrapolation"


class SafeConvexMonotoneInterpolation(BaseModel):
    resource_name: Optional[
        Literal["SafeConvexMonotoneInterpolation"]
    ] = "SafeConvexMonotoneInterpolation"
    x: Array
    y: Array
    quadraticity: Optional[float] = None
    monotonicity: Optional[float] = None
    forcePositive: Optional[bool] = None


class TermStructure(BaseModel):
    resource_name: Optional[Literal["TermStructure"]] = "TermStructure"


class YieldTermStructure(BaseModel):
    resource_name: Optional[Literal["YieldTermStructure"]] = "YieldTermStructure"


class YieldTermStructureHandle(BaseModel):
    resource_name: Optional[
        Literal["YieldTermStructureHandle"]
    ] = "YieldTermStructureHandle"
    value: Optional[YIELDTERMSTRUCTURE] = None


class ImpliedTermStructure(BaseModel):
    resource_name: Optional[Literal["ImpliedTermStructure"]] = "ImpliedTermStructure"
    curveHandle: YieldTermStructureHandle
    referenceDate: Date


class ZeroSpreadedTermStructure(BaseModel):
    resource_name: Optional[
        Literal["ZeroSpreadedTermStructure"]
    ] = "ZeroSpreadedTermStructure"
    curveHandle: YieldTermStructureHandle
    spreadHandle: QuoteHandle
    comp: Optional[enums.Compounding] = None
    freq: Optional[float] = None
    dc: Optional[DAYCOUNTER] = None


class ForwardSpreadedTermStructure(BaseModel):
    resource_name: Optional[
        Literal["ForwardSpreadedTermStructure"]
    ] = "ForwardSpreadedTermStructure"
    curveHandle: YieldTermStructureHandle
    spreadHandle: QuoteHandle


class FlatForward(BaseModel):
    resource_name: Optional[Literal["FlatForward"]] = "FlatForward"
    referenceDate: Date
    forward: float
    dayCounter: DAYCOUNTER
    compounding: Optional[enums.Compounding] = None
    frequency: Optional[float] = None


class IntervalPrice(BaseModel):
    resource_name: Optional[Literal["IntervalPrice"]] = "IntervalPrice"
    Close = 1
    High = 2
    Low = 3
    Open = 0


class IndexManager(BaseModel):
    resource_name: Optional[Literal["IndexManager"]] = "IndexManager"


class Index(BaseModel):
    resource_name: Optional[Literal["Index"]] = "Index"


class InterestRateIndex(BaseModel):
    resource_name: Optional[Literal["InterestRateIndex"]] = "InterestRateIndex"


class IborIndex(BaseModel):
    resource_name: Optional[Literal["IborIndex"]] = "IborIndex"
    familyName: str
    tenor: Period
    settlementDays: int
    currency: CURRENCY
    calendar: CALENDAR
    convention: enums.BusinessDayConvention
    endOfMonth: bool
    dayCounter: DAYCOUNTER
    h: Optional[YieldTermStructureHandle] = None


class OvernightIndex(BaseModel):
    resource_name: Optional[Literal["OvernightIndex"]] = "OvernightIndex"
    familyName: str
    settlementDays: int
    currency: CURRENCY
    calendar: CALENDAR
    dayCounter: DAYCOUNTER
    h: Optional[YieldTermStructureHandle] = None


class Libor(BaseModel):
    resource_name: Optional[Literal["Libor"]] = "Libor"
    familyName: str
    tenor: Period
    settlementDays: float
    currency: CURRENCY
    financialCenterCalendar: CALENDAR
    dayCounter: DAYCOUNTER
    h: Optional[YieldTermStructureHandle] = None


class DailyTenorLibor(BaseModel):
    resource_name: Optional[Literal["DailyTenorLibor"]] = "DailyTenorLibor"
    familyName: str
    settlementDays: float
    currency: CURRENCY
    financialCenterCalendar: CALENDAR
    dayCounter: DAYCOUNTER
    h: Optional[YieldTermStructureHandle] = None


class SwapIndex(BaseModel):
    resource_name: Optional[Literal["SwapIndex"]] = "SwapIndex"
    familyName: str
    tenor: Period
    settlementDays: int
    currency: CURRENCY
    calendar: CALENDAR
    fixedLegTenor: Period
    fixedLegConvention: enums.BusinessDayConvention
    fixedLegDayCounter: DAYCOUNTER
    iborIndex: IBORINDEX
    discountCurve: Optional[YieldTermStructureHandle] = None


class SwapSpreadIndex(BaseModel):
    resource_name: Optional[Literal["SwapSpreadIndex"]] = "SwapSpreadIndex"
    familyName: str
    swapIndex1: SWAPINDEX
    swapIndex2: SWAPINDEX
    gearing1: Optional[float] = None
    gearing2: Optional[float] = None


class AUDLibor(BaseModel):
    resource_name: Optional[Literal["AUDLibor"]] = "AUDLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class CADLibor(BaseModel):
    resource_name: Optional[Literal["CADLibor"]] = "CADLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class CADLiborON(BaseModel):
    resource_name: Optional[Literal["CADLiborON"]] = "CADLiborON"
    h: Optional[YieldTermStructureHandle] = None


class Cdor(BaseModel):
    resource_name: Optional[Literal["Cdor"]] = "Cdor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class CHFLibor(BaseModel):
    resource_name: Optional[Literal["CHFLibor"]] = "CHFLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class DKKLibor(BaseModel):
    resource_name: Optional[Literal["DKKLibor"]] = "DKKLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Bbsw(BaseModel):
    resource_name: Optional[Literal["Bbsw"]] = "Bbsw"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Bbsw1M(BaseModel):
    resource_name: Optional[Literal["Bbsw1M"]] = "Bbsw1M"
    h: Optional[YieldTermStructureHandle] = None


class Bbsw2M(BaseModel):
    resource_name: Optional[Literal["Bbsw2M"]] = "Bbsw2M"
    h: Optional[YieldTermStructureHandle] = None


class Bbsw3M(BaseModel):
    resource_name: Optional[Literal["Bbsw3M"]] = "Bbsw3M"
    h: Optional[YieldTermStructureHandle] = None


class Bbsw4M(BaseModel):
    resource_name: Optional[Literal["Bbsw4M"]] = "Bbsw4M"
    h: Optional[YieldTermStructureHandle] = None


class Bbsw5M(BaseModel):
    resource_name: Optional[Literal["Bbsw5M"]] = "Bbsw5M"
    h: Optional[YieldTermStructureHandle] = None


class Bbsw6M(BaseModel):
    resource_name: Optional[Literal["Bbsw6M"]] = "Bbsw6M"
    h: Optional[YieldTermStructureHandle] = None


class Bkbm(BaseModel):
    resource_name: Optional[Literal["Bkbm"]] = "Bkbm"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Bkbm1M(BaseModel):
    resource_name: Optional[Literal["Bkbm1M"]] = "Bkbm1M"
    h: Optional[YieldTermStructureHandle] = None


class Bkbm2M(BaseModel):
    resource_name: Optional[Literal["Bkbm2M"]] = "Bkbm2M"
    h: Optional[YieldTermStructureHandle] = None


class Bkbm3M(BaseModel):
    resource_name: Optional[Literal["Bkbm3M"]] = "Bkbm3M"
    h: Optional[YieldTermStructureHandle] = None


class Bkbm4M(BaseModel):
    resource_name: Optional[Literal["Bkbm4M"]] = "Bkbm4M"
    h: Optional[YieldTermStructureHandle] = None


class Bkbm5M(BaseModel):
    resource_name: Optional[Literal["Bkbm5M"]] = "Bkbm5M"
    h: Optional[YieldTermStructureHandle] = None


class Bkbm6M(BaseModel):
    resource_name: Optional[Literal["Bkbm6M"]] = "Bkbm6M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor(BaseModel):
    resource_name: Optional[Literal["Euribor"]] = "Euribor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class EuriborSW(BaseModel):
    resource_name: Optional[Literal["EuriborSW"]] = "EuriborSW"
    h: Optional[YieldTermStructureHandle] = None


class Euribor2W(BaseModel):
    resource_name: Optional[Literal["Euribor2W"]] = "Euribor2W"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3W(BaseModel):
    resource_name: Optional[Literal["Euribor3W"]] = "Euribor3W"
    h: Optional[YieldTermStructureHandle] = None


class Euribor1M(BaseModel):
    resource_name: Optional[Literal["Euribor1M"]] = "Euribor1M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor2M(BaseModel):
    resource_name: Optional[Literal["Euribor2M"]] = "Euribor2M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3M(BaseModel):
    resource_name: Optional[Literal["Euribor3M"]] = "Euribor3M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor4M(BaseModel):
    resource_name: Optional[Literal["Euribor4M"]] = "Euribor4M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor5M(BaseModel):
    resource_name: Optional[Literal["Euribor5M"]] = "Euribor5M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor6M(BaseModel):
    resource_name: Optional[Literal["Euribor6M"]] = "Euribor6M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor7M(BaseModel):
    resource_name: Optional[Literal["Euribor7M"]] = "Euribor7M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor8M(BaseModel):
    resource_name: Optional[Literal["Euribor8M"]] = "Euribor8M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor9M(BaseModel):
    resource_name: Optional[Literal["Euribor9M"]] = "Euribor9M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor10M(BaseModel):
    resource_name: Optional[Literal["Euribor10M"]] = "Euribor10M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor11M(BaseModel):
    resource_name: Optional[Literal["Euribor11M"]] = "Euribor11M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor1Y(BaseModel):
    resource_name: Optional[Literal["Euribor1Y"]] = "Euribor1Y"
    h: Optional[YieldTermStructureHandle] = None


class Euribor365(BaseModel):
    resource_name: Optional[Literal["Euribor365"]] = "Euribor365"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Euribor365SW(BaseModel):
    resource_name: Optional[Literal["Euribor365SW"]] = "Euribor365SW"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3652W(BaseModel):
    resource_name: Optional[Literal["Euribor3652W"]] = "Euribor3652W"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3653W(BaseModel):
    resource_name: Optional[Literal["Euribor3653W"]] = "Euribor3653W"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3651M(BaseModel):
    resource_name: Optional[Literal["Euribor3651M"]] = "Euribor3651M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3652M(BaseModel):
    resource_name: Optional[Literal["Euribor3652M"]] = "Euribor3652M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3653M(BaseModel):
    resource_name: Optional[Literal["Euribor3653M"]] = "Euribor3653M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3654M(BaseModel):
    resource_name: Optional[Literal["Euribor3654M"]] = "Euribor3654M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3655M(BaseModel):
    resource_name: Optional[Literal["Euribor3655M"]] = "Euribor3655M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3656M(BaseModel):
    resource_name: Optional[Literal["Euribor3656M"]] = "Euribor3656M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3657M(BaseModel):
    resource_name: Optional[Literal["Euribor3657M"]] = "Euribor3657M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3658M(BaseModel):
    resource_name: Optional[Literal["Euribor3658M"]] = "Euribor3658M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3659M(BaseModel):
    resource_name: Optional[Literal["Euribor3659M"]] = "Euribor3659M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor36510M(BaseModel):
    resource_name: Optional[Literal["Euribor36510M"]] = "Euribor36510M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor36511M(BaseModel):
    resource_name: Optional[Literal["Euribor36511M"]] = "Euribor36511M"
    h: Optional[YieldTermStructureHandle] = None


class Euribor3651Y(BaseModel):
    resource_name: Optional[Literal["Euribor3651Y"]] = "Euribor3651Y"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor(BaseModel):
    resource_name: Optional[Literal["EURLibor"]] = "EURLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class EURLiborSW(BaseModel):
    resource_name: Optional[Literal["EURLiborSW"]] = "EURLiborSW"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor2W(BaseModel):
    resource_name: Optional[Literal["EURLibor2W"]] = "EURLibor2W"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor1M(BaseModel):
    resource_name: Optional[Literal["EURLibor1M"]] = "EURLibor1M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor2M(BaseModel):
    resource_name: Optional[Literal["EURLibor2M"]] = "EURLibor2M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor3M(BaseModel):
    resource_name: Optional[Literal["EURLibor3M"]] = "EURLibor3M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor4M(BaseModel):
    resource_name: Optional[Literal["EURLibor4M"]] = "EURLibor4M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor5M(BaseModel):
    resource_name: Optional[Literal["EURLibor5M"]] = "EURLibor5M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor6M(BaseModel):
    resource_name: Optional[Literal["EURLibor6M"]] = "EURLibor6M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor7M(BaseModel):
    resource_name: Optional[Literal["EURLibor7M"]] = "EURLibor7M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor8M(BaseModel):
    resource_name: Optional[Literal["EURLibor8M"]] = "EURLibor8M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor9M(BaseModel):
    resource_name: Optional[Literal["EURLibor9M"]] = "EURLibor9M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor10M(BaseModel):
    resource_name: Optional[Literal["EURLibor10M"]] = "EURLibor10M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor11M(BaseModel):
    resource_name: Optional[Literal["EURLibor11M"]] = "EURLibor11M"
    h: Optional[YieldTermStructureHandle] = None


class EURLibor1Y(BaseModel):
    resource_name: Optional[Literal["EURLibor1Y"]] = "EURLibor1Y"
    h: Optional[YieldTermStructureHandle] = None


class GBPLibor(BaseModel):
    resource_name: Optional[Literal["GBPLibor"]] = "GBPLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class GBPLiborON(BaseModel):
    resource_name: Optional[Literal["GBPLiborON"]] = "GBPLiborON"
    h: Optional[YieldTermStructureHandle] = None


class Jibar(BaseModel):
    resource_name: Optional[Literal["Jibar"]] = "Jibar"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class JPYLibor(BaseModel):
    resource_name: Optional[Literal["JPYLibor"]] = "JPYLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Mosprime(BaseModel):
    resource_name: Optional[Literal["Mosprime"]] = "Mosprime"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class NZDLibor(BaseModel):
    resource_name: Optional[Literal["NZDLibor"]] = "NZDLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Pribor(BaseModel):
    resource_name: Optional[Literal["Pribor"]] = "Pribor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Robor(BaseModel):
    resource_name: Optional[Literal["Robor"]] = "Robor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class SEKLibor(BaseModel):
    resource_name: Optional[Literal["SEKLibor"]] = "SEKLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Shibor(BaseModel):
    resource_name: Optional[Literal["Shibor"]] = "Shibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Tibor(BaseModel):
    resource_name: Optional[Literal["Tibor"]] = "Tibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class THBFIX(BaseModel):
    resource_name: Optional[Literal["THBFIX"]] = "THBFIX"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class TRLibor(BaseModel):
    resource_name: Optional[Literal["TRLibor"]] = "TRLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class USDLibor(BaseModel):
    resource_name: Optional[Literal["USDLibor"]] = "USDLibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class USDLiborON(BaseModel):
    resource_name: Optional[Literal["USDLiborON"]] = "USDLiborON"
    h: Optional[YieldTermStructureHandle] = None


class Wibor(BaseModel):
    resource_name: Optional[Literal["Wibor"]] = "Wibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Zibor(BaseModel):
    resource_name: Optional[Literal["Zibor"]] = "Zibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Aonia(BaseModel):
    resource_name: Optional[Literal["Aonia"]] = "Aonia"
    h: Optional[YieldTermStructureHandle] = None


class Eonia(BaseModel):
    resource_name: Optional[Literal["Eonia"]] = "Eonia"
    h: Optional[YieldTermStructureHandle] = None


class Sonia(BaseModel):
    resource_name: Optional[Literal["Sonia"]] = "Sonia"
    h: Optional[YieldTermStructureHandle] = None


class FedFunds(BaseModel):
    resource_name: Optional[Literal["FedFunds"]] = "FedFunds"
    h: Optional[YieldTermStructureHandle] = None


class Nzocr(BaseModel):
    resource_name: Optional[Literal["Nzocr"]] = "Nzocr"
    h: Optional[YieldTermStructureHandle] = None


class Sofr(BaseModel):
    resource_name: Optional[Literal["Sofr"]] = "Sofr"
    h: Optional[YieldTermStructureHandle] = None


class EuriborSwapIsdaFixA(BaseModel):
    resource_name: Optional[Literal["EuriborSwapIsdaFixA"]] = "EuriborSwapIsdaFixA"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class EuriborSwapIsdaFixB(BaseModel):
    resource_name: Optional[Literal["EuriborSwapIsdaFixB"]] = "EuriborSwapIsdaFixB"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class EuriborSwapIfrFix(BaseModel):
    resource_name: Optional[Literal["EuriborSwapIfrFix"]] = "EuriborSwapIfrFix"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class EurLiborSwapIsdaFixA(BaseModel):
    resource_name: Optional[Literal["EurLiborSwapIsdaFixA"]] = "EurLiborSwapIsdaFixA"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class EurLiborSwapIsdaFixB(BaseModel):
    resource_name: Optional[Literal["EurLiborSwapIsdaFixB"]] = "EurLiborSwapIsdaFixB"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class EurLiborSwapIfrFix(BaseModel):
    resource_name: Optional[Literal["EurLiborSwapIfrFix"]] = "EurLiborSwapIfrFix"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class ChfLiborSwapIsdaFix(BaseModel):
    resource_name: Optional[Literal["ChfLiborSwapIsdaFix"]] = "ChfLiborSwapIsdaFix"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class GbpLiborSwapIsdaFix(BaseModel):
    resource_name: Optional[Literal["GbpLiborSwapIsdaFix"]] = "GbpLiborSwapIsdaFix"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class JpyLiborSwapIsdaFixAm(BaseModel):
    resource_name: Optional[Literal["JpyLiborSwapIsdaFixAm"]] = "JpyLiborSwapIsdaFixAm"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class JpyLiborSwapIsdaFixPm(BaseModel):
    resource_name: Optional[Literal["JpyLiborSwapIsdaFixPm"]] = "JpyLiborSwapIsdaFixPm"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class UsdLiborSwapIsdaFixAm(BaseModel):
    resource_name: Optional[Literal["UsdLiborSwapIsdaFixAm"]] = "UsdLiborSwapIsdaFixAm"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class UsdLiborSwapIsdaFixPm(BaseModel):
    resource_name: Optional[Literal["UsdLiborSwapIsdaFixPm"]] = "UsdLiborSwapIsdaFixPm"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class Bibor(BaseModel):
    resource_name: Optional[Literal["Bibor"]] = "Bibor"
    tenor: Period
    h: Optional[YieldTermStructureHandle] = None


class BiborSW(BaseModel):
    resource_name: Optional[Literal["BiborSW"]] = "BiborSW"
    h: Optional[YieldTermStructureHandle] = None


class Bibor1M(BaseModel):
    resource_name: Optional[Literal["Bibor1M"]] = "Bibor1M"
    h: Optional[YieldTermStructureHandle] = None


class Bibor2M(BaseModel):
    resource_name: Optional[Literal["Bibor2M"]] = "Bibor2M"
    h: Optional[YieldTermStructureHandle] = None


class Bibor3M(BaseModel):
    resource_name: Optional[Literal["Bibor3M"]] = "Bibor3M"
    h: Optional[YieldTermStructureHandle] = None


class Bibor6M(BaseModel):
    resource_name: Optional[Literal["Bibor6M"]] = "Bibor6M"
    h: Optional[YieldTermStructureHandle] = None


class Bibor9M(BaseModel):
    resource_name: Optional[Literal["Bibor9M"]] = "Bibor9M"
    h: Optional[YieldTermStructureHandle] = None


class Bibor1Y(BaseModel):
    resource_name: Optional[Literal["Bibor1Y"]] = "Bibor1Y"
    h: Optional[YieldTermStructureHandle] = None


class Brent(BaseModel):
    resource_name: Optional[Literal["Brent"]] = "Brent"


class Bisection(BaseModel):
    resource_name: Optional[Literal["Bisection"]] = "Bisection"


class FalsePosition(BaseModel):
    resource_name: Optional[Literal["FalsePosition"]] = "FalsePosition"


class Ridder(BaseModel):
    resource_name: Optional[Literal["Ridder"]] = "Ridder"


class Secant(BaseModel):
    resource_name: Optional[Literal["Secant"]] = "Secant"


class Newton(BaseModel):
    resource_name: Optional[Literal["Newton"]] = "Newton"


class NewtonSafe(BaseModel):
    resource_name: Optional[Literal["NewtonSafe"]] = "NewtonSafe"


class Constraint(BaseModel):
    resource_name: Optional[Literal["Constraint"]] = "Constraint"


class BoundaryConstraint(BaseModel):
    resource_name: Optional[Literal["BoundaryConstraint"]] = "BoundaryConstraint"
    lower: float
    upper: float


class NoConstraint(BaseModel):
    resource_name: Optional[Literal["NoConstraint"]] = "NoConstraint"


class PositiveConstraint(BaseModel):
    resource_name: Optional[Literal["PositiveConstraint"]] = "PositiveConstraint"


class CompositeConstraint(BaseModel):
    resource_name: Optional[Literal["CompositeConstraint"]] = "CompositeConstraint"
    c1: CONSTRAINT
    c2: CONSTRAINT


class NonhomogeneousBoundaryConstraint(BaseModel):
    resource_name: Optional[
        Literal["NonhomogeneousBoundaryConstraint"]
    ] = "NonhomogeneousBoundaryConstraint"
    l: Array
    u: Array


class EndCriteria(BaseModel):
    resource_name: Optional[Literal["EndCriteria"]] = "EndCriteria"
    maxIteration: int
    maxStationaryStateIterations: int
    rootEpsilon: float
    functionEpsilon: float
    gradientNormEpsilon: float


class OptimizationMethod(BaseModel):
    resource_name: Optional[Literal["OptimizationMethod"]] = "OptimizationMethod"


class ConjugateGradient(BaseModel):
    resource_name: Optional[Literal["ConjugateGradient"]] = "ConjugateGradient"


class Simplex(BaseModel):
    resource_name: Optional[Literal["Simplex"]] = "Simplex"
    lambda_: float = Field(..., alias="lambda")


class SteepestDescent(BaseModel):
    resource_name: Optional[Literal["SteepestDescent"]] = "SteepestDescent"


class BFGS(BaseModel):
    resource_name: Optional[Literal["BFGS"]] = "BFGS"


class LevenbergMarquardt(BaseModel):
    resource_name: Optional[Literal["LevenbergMarquardt"]] = "LevenbergMarquardt"
    epsfcn: Optional[float] = None
    xtol: Optional[float] = None
    gtol: Optional[float] = None
    useCostFunctionsJacobian: Optional[bool] = None


class DifferentialEvolution(BaseModel):
    resource_name: Optional[Literal["DifferentialEvolution"]] = "DifferentialEvolution"


class SamplerGaussian(BaseModel):
    resource_name: Optional[Literal["SamplerGaussian"]] = "SamplerGaussian"
    seed: Optional[int] = None


class SamplerLogNormal(BaseModel):
    resource_name: Optional[Literal["SamplerLogNormal"]] = "SamplerLogNormal"
    seed: Optional[int] = None


class SamplerMirrorGaussian(BaseModel):
    resource_name: Optional[Literal["SamplerMirrorGaussian"]] = "SamplerMirrorGaussian"
    lower: Array
    upper: Array
    seed: Optional[int] = None


class ProbabilityBoltzmannDownhill(BaseModel):
    resource_name: Optional[
        Literal["ProbabilityBoltzmannDownhill"]
    ] = "ProbabilityBoltzmannDownhill"
    seed: Optional[int] = None


class TemperatureExponential(BaseModel):
    resource_name: Optional[
        Literal["TemperatureExponential"]
    ] = "TemperatureExponential"
    initialTemp: float
    dimension: int
    power: Optional[float] = None


class ReannealingTrivial(BaseModel):
    resource_name: Optional[Literal["ReannealingTrivial"]] = "ReannealingTrivial"


class Optimizer(BaseModel):
    resource_name: Optional[Literal["Optimizer"]] = "Optimizer"


class VolatilityTermStructure(BaseModel):
    resource_name: Optional[
        Literal["VolatilityTermStructure"]
    ] = "VolatilityTermStructure"


class BlackVolTermStructure(BaseModel):
    resource_name: Optional[Literal["BlackVolTermStructure"]] = "BlackVolTermStructure"


class BlackVolTermStructureHandle(BaseModel):
    resource_name: Optional[
        Literal["BlackVolTermStructureHandle"]
    ] = "BlackVolTermStructureHandle"
    value: Optional[BLACKVOLTERMSTRUCTURE] = None


class LocalVolTermStructure(BaseModel):
    resource_name: Optional[Literal["LocalVolTermStructure"]] = "LocalVolTermStructure"


class LocalVolTermStructureHandle(BaseModel):
    resource_name: Optional[
        Literal["LocalVolTermStructureHandle"]
    ] = "LocalVolTermStructureHandle"
    value: Optional[LOCALVOLTERMSTRUCTURE] = None


class OptionletVolatilityStructure(BaseModel):
    resource_name: Optional[
        Literal["OptionletVolatilityStructure"]
    ] = "OptionletVolatilityStructure"


class OptionletVolatilityStructureHandle(BaseModel):
    resource_name: Optional[
        Literal["OptionletVolatilityStructureHandle"]
    ] = "OptionletVolatilityStructureHandle"
    value: Optional[OPTIONLETVOLATILITYSTRUCTURE] = None


class SwaptionVolatilityStructure(BaseModel):
    resource_name: Optional[
        Literal["SwaptionVolatilityStructure"]
    ] = "SwaptionVolatilityStructure"


class SwaptionVolatilityStructureHandle(BaseModel):
    resource_name: Optional[
        Literal["SwaptionVolatilityStructureHandle"]
    ] = "SwaptionVolatilityStructureHandle"
    value: Optional[SWAPTIONVOLATILITYSTRUCTURE] = None


class BlackConstantVol(BaseModel):
    resource_name: Optional[Literal["BlackConstantVol"]] = "BlackConstantVol"
    referenceDate: Date
    c: CALENDAR
    volatility: float
    dayCounter: DAYCOUNTER


class BlackVarianceCurve(BaseModel):
    resource_name: Optional[Literal["BlackVarianceCurve"]] = "BlackVarianceCurve"
    referenceDate: Date
    dates: List[Date]
    volatilities: List[float]
    dayCounter: DAYCOUNTER
    forceMonotoneVariance: Optional[bool] = None


class BlackVarianceSurface(BaseModel):
    resource_name: Optional[Literal["BlackVarianceSurface"]] = "BlackVarianceSurface"
    ConstantExtrapolation = 0
    InterpolatorDefaultExtrapolation = 1


class LocalConstantVol(BaseModel):
    resource_name: Optional[Literal["LocalConstantVol"]] = "LocalConstantVol"
    referenceDate: Date
    volatility: float
    dayCounter: DAYCOUNTER


class LocalVolSurface(BaseModel):
    resource_name: Optional[Literal["LocalVolSurface"]] = "LocalVolSurface"
    blackTS: BlackVolTermStructureHandle
    riskFreeTS: YieldTermStructureHandle
    dividendTS: YieldTermStructureHandle
    underlying: QuoteHandle


class SwaptionVolatilityDiscrete(BaseModel):
    resource_name: Optional[
        Literal["SwaptionVolatilityDiscrete"]
    ] = "SwaptionVolatilityDiscrete"


class SwaptionVolCube1(BaseModel):
    resource_name: Optional[Literal["SwaptionVolCube1"]] = "SwaptionVolCube1"
    atmVolStructure: SwaptionVolatilityStructureHandle
    optionTenors: List[Period]
    swapTenors: List[Period]
    strikeSpreads: List[float]
    volSpreads: List[List[QuoteHandle]]
    swapIndex: SWAPINDEX
    shortSwapIndex: SWAPINDEX
    vegaWeightedSmileFit: bool
    parametersGuess: List[List[QuoteHandle]]
    isParameterFixed: List[bool]
    isAtmCalibrated: bool
    endCriteria: Optional[EndCriteria] = None
    maxErrorTolerance: Optional[float] = None
    optMethod: Optional[OPTIMIZATIONMETHOD] = None
    errorAccept: Optional[float] = None
    useMaxError: Optional[bool] = None
    maxGuesses: Optional[int] = None
    backwardFlat: Optional[bool] = None
    cutoffStrike: Optional[float] = None


class SwaptionVolCube2(BaseModel):
    resource_name: Optional[Literal["SwaptionVolCube2"]] = "SwaptionVolCube2"
    atmVolStructure: SwaptionVolatilityStructureHandle
    optionTenors: List[Period]
    swapTenors: List[Period]
    strikeSpreads: List[float]
    volSpreads: List[List[QuoteHandle]]
    swapIndex: SWAPINDEX
    shortSwapIndex: SWAPINDEX
    vegaWeightedSmileFit: bool


class SmileSection(BaseModel):
    resource_name: Optional[Literal["SmileSection"]] = "SmileSection"


class SabrSmileSection(BaseModel):
    resource_name: Optional[Literal["SabrSmileSection"]] = "SabrSmileSection"
    timeToExpiry: float
    forward: float
    sabrParameters: List[float]
    shift: Optional[float] = None


class KahaleSmileSection(BaseModel):
    resource_name: Optional[Literal["KahaleSmileSection"]] = "KahaleSmileSection"
    source: SMILESECTION
    atm: Optional[float] = None
    interpolate: Optional[bool] = None
    exponentialExtrapolation: Optional[bool] = None
    deleteArbitragePoints: Optional[bool] = None
    moneynessGrid: Optional[List[float]] = None
    gap: Optional[float] = None
    forcedLeftIndex: Optional[int] = None
    forcedRightIndex: Optional[int] = None


class ZabrShortMaturityLognormal(BaseModel):
    resource_name: Optional[
        Literal["ZabrShortMaturityLognormal"]
    ] = "ZabrShortMaturityLognormal"


class ZabrShortMaturityNormal(BaseModel):
    resource_name: Optional[
        Literal["ZabrShortMaturityNormal"]
    ] = "ZabrShortMaturityNormal"


class ZabrLocalVolatility(BaseModel):
    resource_name: Optional[Literal["ZabrLocalVolatility"]] = "ZabrLocalVolatility"


class ZabrFullFd(BaseModel):
    resource_name: Optional[Literal["ZabrFullFd"]] = "ZabrFullFd"


class NoArbSabrSmileSection(BaseModel):
    resource_name: Optional[Literal["NoArbSabrSmileSection"]] = "NoArbSabrSmileSection"
    d: Date
    forward: float
    sabrParameters: List[float]
    dc: Optional[DAYCOUNTER] = None
    shift: Optional[float] = None


class NoArbSabrInterpolatedSmileSection(BaseModel):
    resource_name: Optional[
        Literal["NoArbSabrInterpolatedSmileSection"]
    ] = "NoArbSabrInterpolatedSmileSection"
    optionDate: Date
    forward: float
    strikes: List[float]
    hasFloatingStrikes: bool
    atmVolatility: float
    vols: List[float]
    alpha: float
    beta: float
    nu: float
    rho: float
    isAlphaFixed: Optional[bool] = None
    isBetaFixed: Optional[bool] = None
    isNuFixed: Optional[bool] = None
    isRhoFixed: Optional[bool] = None
    vegaWeighted: Optional[bool] = None
    endCriteria: Optional[EndCriteria] = None
    method: Optional[OPTIMIZATIONMETHOD] = None
    dc: Optional[DAYCOUNTER] = None


class StochasticProcess(BaseModel):
    resource_name: Optional[Literal["StochasticProcess"]] = "StochasticProcess"


class StochasticProcess1D(BaseModel):
    resource_name: Optional[Literal["StochasticProcess1D"]] = "StochasticProcess1D"


class GeneralizedBlackScholesProcess(BaseModel):
    resource_name: Optional[
        Literal["GeneralizedBlackScholesProcess"]
    ] = "GeneralizedBlackScholesProcess"
    s0: QuoteHandle
    dividendTS: YieldTermStructureHandle
    riskFreeTS: YieldTermStructureHandle
    volTS: BlackVolTermStructureHandle


class BlackScholesProcess(BaseModel):
    resource_name: Optional[Literal["BlackScholesProcess"]] = "BlackScholesProcess"
    s0: QuoteHandle
    riskFreeTS: YieldTermStructureHandle
    volTS: BlackVolTermStructureHandle


class BlackScholesMertonProcess(BaseModel):
    resource_name: Optional[
        Literal["BlackScholesMertonProcess"]
    ] = "BlackScholesMertonProcess"
    s0: QuoteHandle
    dividendTS: YieldTermStructureHandle
    riskFreeTS: YieldTermStructureHandle
    volTS: BlackVolTermStructureHandle


class BlackProcess(BaseModel):
    resource_name: Optional[Literal["BlackProcess"]] = "BlackProcess"
    s0: QuoteHandle
    riskFreeTS: YieldTermStructureHandle
    volTS: BlackVolTermStructureHandle


class GarmanKohlagenProcess(BaseModel):
    resource_name: Optional[Literal["GarmanKohlagenProcess"]] = "GarmanKohlagenProcess"
    s0: QuoteHandle
    foreignRiskFreeTS: YieldTermStructureHandle
    domesticRiskFreeTS: YieldTermStructureHandle
    volTS: BlackVolTermStructureHandle


class Merton76Process(BaseModel):
    resource_name: Optional[Literal["Merton76Process"]] = "Merton76Process"
    stateVariable: QuoteHandle
    dividendTS: YieldTermStructureHandle
    riskFreeTS: YieldTermStructureHandle
    volTS: BlackVolTermStructureHandle
    jumpIntensity: QuoteHandle
    meanLogJump: QuoteHandle
    jumpVolatility: QuoteHandle


class StochasticProcessArray(BaseModel):
    resource_name: Optional[
        Literal["StochasticProcessArray"]
    ] = "StochasticProcessArray"
    array: List[STOCHASTICPROCESS1D]
    correlation: Matrix


class GeometricBrownianMotionProcess(BaseModel):
    resource_name: Optional[
        Literal["GeometricBrownianMotionProcess"]
    ] = "GeometricBrownianMotionProcess"
    initialValue: float
    mu: float
    sigma: float


class VarianceGammaProcess(BaseModel):
    resource_name: Optional[Literal["VarianceGammaProcess"]] = "VarianceGammaProcess"
    s0: QuoteHandle
    dividendYield: YieldTermStructureHandle
    riskFreeRate: YieldTermStructureHandle
    sigma: float
    nu: float
    theta: float


class HestonProcess(BaseModel):
    resource_name: Optional[Literal["HestonProcess"]] = "HestonProcess"
    riskFreeTS: YieldTermStructureHandle
    dividendTS: YieldTermStructureHandle
    s0: QuoteHandle
    v0: float
    kappa: float
    theta: float
    sigma: float
    rho: float


class BatesProcess(BaseModel):
    resource_name: Optional[Literal["BatesProcess"]] = "BatesProcess"
    riskFreeRate: YieldTermStructureHandle
    dividendYield: YieldTermStructureHandle
    s0: QuoteHandle
    v0: float
    kappa: float
    theta: float
    sigma: float
    rho: float
    lambda_: float = Field(..., alias="lambda")
    nu: float
    delta: float


class HullWhiteProcess(BaseModel):
    resource_name: Optional[Literal["HullWhiteProcess"]] = "HullWhiteProcess"
    riskFreeTS: YieldTermStructureHandle
    a: float
    sigma: float


class HullWhiteForwardProcess(BaseModel):
    resource_name: Optional[
        Literal["HullWhiteForwardProcess"]
    ] = "HullWhiteForwardProcess"
    riskFreeTS: YieldTermStructureHandle
    a: float
    sigma: float


class G2Process(BaseModel):
    resource_name: Optional[Literal["G2Process"]] = "G2Process"
    a: float
    sigma: float
    b: float
    eta: float
    rho: float


class G2ForwardProcess(BaseModel):
    resource_name: Optional[Literal["G2ForwardProcess"]] = "G2ForwardProcess"
    a: float
    sigma: float
    b: float
    eta: float
    rho: float


class GsrProcess(BaseModel):
    resource_name: Optional[Literal["GsrProcess"]] = "GsrProcess"
    times: Array
    vols: Array
    reversions: Array
    T: Optional[float] = None


class OrnsteinUhlenbeckProcess(BaseModel):
    resource_name: Optional[
        Literal["OrnsteinUhlenbeckProcess"]
    ] = "OrnsteinUhlenbeckProcess"
    speed: float
    vol: float
    x0: Optional[float] = None
    level: Optional[float] = None


class PricingEngine(BaseModel):
    resource_name: Optional[Literal["PricingEngine"]] = "PricingEngine"


class Instrument(BaseModel):
    resource_name: Optional[Literal["Instrument"]] = "Instrument"


class Stock(BaseModel):
    resource_name: Optional[Literal["Stock"]] = "Stock"
    quote: QuoteHandle


class CompositeInstrument(BaseModel):
    resource_name: Optional[Literal["CompositeInstrument"]] = "CompositeInstrument"


class Schedule(BaseModel):
    resource_name: Optional[Literal["Schedule"]] = "Schedule"
    effectiveDate: Optional[Date] = None
    terminationDate: Optional[Date] = None
    tenor: Optional[Period] = None
    calendar: Optional[CALENDAR] = None
    convention: Optional[enums.BusinessDayConvention] = None
    terminationDateConvention: Optional[enums.BusinessDayConvention] = None
    rule: Optional[enums.DateGenerationRule] = None
    endOfMonth: Optional[bool] = None
    firstDate: Optional[Date] = None
    nextToLastDate: Optional[Date] = None


class MakeSchedule(BaseModel):
    resource_name: Optional[Literal["MakeSchedule"]] = "MakeSchedule"


class CashFlow(BaseModel):
    resource_name: Optional[Literal["CashFlow"]] = "CashFlow"


class SimpleCashFlow(BaseModel):
    resource_name: Optional[Literal["SimpleCashFlow"]] = "SimpleCashFlow"
    amount: float
    date: Date


class Redemption(BaseModel):
    resource_name: Optional[Literal["Redemption"]] = "Redemption"
    amount: float
    date: Date


class AmortizingPayment(BaseModel):
    resource_name: Optional[Literal["AmortizingPayment"]] = "AmortizingPayment"
    amount: float
    date: Date


class Coupon(BaseModel):
    resource_name: Optional[Literal["Coupon"]] = "Coupon"


class FixedRateCoupon(BaseModel):
    resource_name: Optional[Literal["FixedRateCoupon"]] = "FixedRateCoupon"
    paymentDate: Date
    nominal: float
    rate: float
    dayCounter: DAYCOUNTER
    startDate: Date
    endDate: Date
    refPeriodStart: Optional[Date] = None
    refPeriodEnd: Optional[Date] = None
    exCouponDate: Optional[Date] = None


class FloatingRateCouponPricer(BaseModel):
    resource_name: Optional[
        Literal["FloatingRateCouponPricer"]
    ] = "FloatingRateCouponPricer"


class FloatingRateCoupon(BaseModel):
    resource_name: Optional[Literal["FloatingRateCoupon"]] = "FloatingRateCoupon"


class OvernightIndexedCoupon(BaseModel):
    resource_name: Optional[
        Literal["OvernightIndexedCoupon"]
    ] = "OvernightIndexedCoupon"
    paymentDate: Date
    nominal: float
    startDate: Date
    endDate: Date
    overnightIndex: OVERNIGHTINDEX
    gearing: Optional[float] = None
    spread: Optional[float] = None
    refPeriodStart: Optional[Date] = None
    refPeriodEnd: Optional[Date] = None
    dayCounter: Optional[DAYCOUNTER] = None
    telescopicValueDates: Optional[bool] = None


class CappedFlooredCoupon(BaseModel):
    resource_name: Optional[Literal["CappedFlooredCoupon"]] = "CappedFlooredCoupon"
    underlying: FLOATINGRATECOUPON
    cap: Optional[float] = None
    floor: Optional[float] = None


class IborCoupon(BaseModel):
    resource_name: Optional[Literal["IborCoupon"]] = "IborCoupon"
    paymentDate: Date
    nominal: float
    startDate: Date
    endDate: Date
    fixingDays: int
    index: IBORINDEX
    gearing: Optional[float] = None
    spread: Optional[float] = None
    refPeriodStart: Optional[Date] = None
    refPeriodEnd: Optional[Date] = None
    dayCounter: Optional[DAYCOUNTER] = None
    isInArrears: Optional[bool] = None
    exCouponDate: Optional[Date] = None


class CappedFlooredIborCoupon(BaseModel):
    resource_name: Optional[
        Literal["CappedFlooredIborCoupon"]
    ] = "CappedFlooredIborCoupon"
    paymentDate: Date
    nominal: float
    startDate: Date
    endDate: Date
    fixingDays: int
    index: IBORINDEX
    gearing: Optional[float] = None
    spread: Optional[float] = None
    cap: Optional[float] = None
    floor: Optional[float] = None
    refPeriodStart: Optional[Date] = None
    refPeriodEnd: Optional[Date] = None
    dayCounter: Optional[DAYCOUNTER] = None
    isInArrears: Optional[bool] = None
    exCouponDate: Optional[Date] = None


class IborCouponPricer(BaseModel):
    resource_name: Optional[Literal["IborCouponPricer"]] = "IborCouponPricer"


class BlackIborCouponPricer(BaseModel):
    resource_name: Optional[Literal["BlackIborCouponPricer"]] = "BlackIborCouponPricer"
    v: Optional[OptionletVolatilityStructureHandle] = None


class CmsCoupon(BaseModel):
    resource_name: Optional[Literal["CmsCoupon"]] = "CmsCoupon"
    paymentDate: Date
    nominal: float
    startDate: Date
    endDate: Date
    fixingDays: int
    index: SWAPINDEX
    gearing: Optional[float] = None
    spread: Optional[float] = None
    refPeriodStart: Optional[Date] = None
    refPeriodEnd: Optional[Date] = None
    dayCounter: Optional[DAYCOUNTER] = None
    isInArrears: Optional[bool] = None
    exCouponDate: Optional[Date] = None


class CmsSpreadCoupon(BaseModel):
    resource_name: Optional[Literal["CmsSpreadCoupon"]] = "CmsSpreadCoupon"
    paymentDate: Date
    nominal: float
    startDate: Date
    endDate: Date
    fixingDays: float
    index: SwapSpreadIndex
    gearing: Optional[float] = None
    spread: Optional[float] = None
    refPeriodStart: Optional[Date] = None
    refPeriodEnd: Optional[Date] = None
    dayCounter: Optional[DAYCOUNTER] = None
    isInArrears: Optional[bool] = None
    exCouponDate: Optional[Date] = None


class CmsCouponPricer(BaseModel):
    resource_name: Optional[Literal["CmsCouponPricer"]] = "CmsCouponPricer"


class GFunctionFactory(BaseModel):
    resource_name: Optional[Literal["GFunctionFactory"]] = "GFunctionFactory"


class CappedFlooredCmsCoupon(BaseModel):
    resource_name: Optional[
        Literal["CappedFlooredCmsCoupon"]
    ] = "CappedFlooredCmsCoupon"
    paymentDate: Date
    nominal: float
    startDate: Date
    endDate: Date
    fixingDays: float
    index: SWAPINDEX
    gearing: Optional[float] = None
    spread: Optional[float] = None
    cap: Optional[float] = None
    floor: Optional[float] = None
    refPeriodStart: Optional[Date] = None
    refPeriodEnd: Optional[Date] = None
    dayCounter: Optional[DAYCOUNTER] = None
    isInArrears: Optional[bool] = None
    exCouponDate: Optional[Date] = None


class CappedFlooredCmsSpreadCoupon(BaseModel):
    resource_name: Optional[
        Literal["CappedFlooredCmsSpreadCoupon"]
    ] = "CappedFlooredCmsSpreadCoupon"
    paymentDate: Date
    nominal: float
    startDate: Date
    endDate: Date
    fixingDays: float
    index: SwapSpreadIndex
    gearing: Optional[float] = None
    spread: Optional[float] = None
    cap: Optional[float] = None
    floor: Optional[float] = None
    refPeriodStart: Optional[Date] = None
    refPeriodEnd: Optional[Date] = None
    dayCounter: Optional[DAYCOUNTER] = None
    isInArrears: Optional[bool] = None
    exCouponDate: Optional[Date] = None


class LinearTsrPricerSettings(BaseModel):
    resource_name: Optional[
        Literal["LinearTsrPricerSettings"]
    ] = "LinearTsrPricerSettings"
    BSStdDevs = 3
    PriceThreshold = 2
    RateBound = 0
    VegaRatio = 1


class CmsSpreadCouponPricer(BaseModel):
    resource_name: Optional[Literal["CmsSpreadCouponPricer"]] = "CmsSpreadCouponPricer"


class Duration(BaseModel):
    resource_name: Optional[Literal["Duration"]] = "Duration"


class CashFlows(BaseModel):
    resource_name: Optional[Literal["CashFlows"]] = "CashFlows"
    value: Optional[CashFlows] = None


class CalibrationHelper(BaseModel):
    resource_name: Optional[Literal["CalibrationHelper"]] = "CalibrationHelper"


class BlackCalibrationHelper(BaseModel):
    resource_name: Optional[
        Literal["BlackCalibrationHelper"]
    ] = "BlackCalibrationHelper"


class CalibratedModel(BaseModel):
    resource_name: Optional[Literal["CalibratedModel"]] = "CalibratedModel"


class TermStructureConsistentModel(BaseModel):
    resource_name: Optional[
        Literal["TermStructureConsistentModel"]
    ] = "TermStructureConsistentModel"


class CalibratedModelHandle(BaseModel):
    resource_name: Optional[Literal["CalibratedModelHandle"]] = "CalibratedModelHandle"
    value: Optional[CALIBRATEDMODEL] = None


class TimeGrid(BaseModel):
    resource_name: Optional[Literal["TimeGrid"]] = "TimeGrid"
    end: Optional[float] = None
    steps: Optional[int] = None


class Parameter(BaseModel):
    resource_name: Optional[Literal["Parameter"]] = "Parameter"


class ConstantParameter(BaseModel):
    resource_name: Optional[Literal["ConstantParameter"]] = "ConstantParameter"
    constraint: CONSTRAINT


class NullParameter(BaseModel):
    resource_name: Optional[Literal["NullParameter"]] = "NullParameter"


class PiecewiseConstantParameter(BaseModel):
    resource_name: Optional[
        Literal["PiecewiseConstantParameter"]
    ] = "PiecewiseConstantParameter"
    times: List[float]
    constraint: Optional[CONSTRAINT] = None


class Payoff(BaseModel):
    resource_name: Optional[Literal["Payoff"]] = "Payoff"


class Option(BaseModel):
    resource_name: Optional[Literal["Option"]] = "Option"


class Barrier(BaseModel):
    resource_name: Optional[Literal["Barrier"]] = "Barrier"


class DoubleBarrier(BaseModel):
    resource_name: Optional[Literal["DoubleBarrier"]] = "DoubleBarrier"


class TypePayoff(BaseModel):
    resource_name: Optional[Literal["TypePayoff"]] = "TypePayoff"


class StrikedTypePayoff(BaseModel):
    resource_name: Optional[Literal["StrikedTypePayoff"]] = "StrikedTypePayoff"


class OneAssetOption(BaseModel):
    resource_name: Optional[Literal["OneAssetOption"]] = "OneAssetOption"


class MultiAssetOption(BaseModel):
    resource_name: Optional[Literal["MultiAssetOption"]] = "MultiAssetOption"


class AnalyticEuropeanEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticEuropeanEngine"]
    ] = "AnalyticEuropeanEngine"
    value: GENERALIZEDBLACKSCHOLESPROCESS


class HestonModel(BaseModel):
    resource_name: Optional[Literal["HestonModel"]] = "HestonModel"
    process: HESTONPROCESS


class HestonModelHandle(BaseModel):
    resource_name: Optional[Literal["HestonModelHandle"]] = "HestonModelHandle"
    value: Optional[HESTONMODEL] = None


class PiecewiseTimeDependentHestonModel(BaseModel):
    resource_name: Optional[
        Literal["PiecewiseTimeDependentHestonModel"]
    ] = "PiecewiseTimeDependentHestonModel"
    riskFreeRate: YieldTermStructureHandle
    dividendYield: YieldTermStructureHandle
    s0: QuoteHandle
    v0: float
    theta: PARAMETER
    kappa: PARAMETER
    sigma: PARAMETER
    rho: PARAMETER
    timeGrid: TimeGrid


class COSHestonEngine(BaseModel):
    resource_name: Optional[Literal["COSHestonEngine"]] = "COSHestonEngine"
    model: HESTONMODEL
    L: Optional[float] = None
    N: Optional[int] = None


class BatesModel(BaseModel):
    resource_name: Optional[Literal["BatesModel"]] = "BatesModel"
    process: BatesProcess


class BatesEngine(BaseModel):
    resource_name: Optional[Literal["BatesEngine"]] = "BatesEngine"
    model: BatesModel
    integrationOrder: Optional[int] = None


class IntegralEngine(BaseModel):
    resource_name: Optional[Literal["IntegralEngine"]] = "IntegralEngine"
    value: GENERALIZEDBLACKSCHOLESPROCESS


class FDBermudanEngine(BaseModel):
    resource_name: Optional[Literal["FDBermudanEngine"]] = "FDBermudanEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    timeSteps: Optional[int] = None
    gridPoints: Optional[int] = None
    timeDependent: Optional[bool] = None


class FDEuropeanEngine(BaseModel):
    resource_name: Optional[Literal["FDEuropeanEngine"]] = "FDEuropeanEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    timeSteps: Optional[int] = None
    gridPoints: Optional[int] = None
    timeDependent: Optional[bool] = None


class BinomialCRRVanillaEngine(BaseModel):
    resource_name: Optional[
        Literal["BinomialCRRVanillaEngine"]
    ] = "BinomialCRRVanillaEngine"
    value: GENERALIZEDBLACKSCHOLESPROCESS
    steps: int


class BinomialJRVanillaEngine(BaseModel):
    resource_name: Optional[
        Literal["BinomialJRVanillaEngine"]
    ] = "BinomialJRVanillaEngine"
    value: GENERALIZEDBLACKSCHOLESPROCESS
    steps: int


class BinomialTrigeorgisVanillaEngine(BaseModel):
    resource_name: Optional[
        Literal["BinomialTrigeorgisVanillaEngine"]
    ] = "BinomialTrigeorgisVanillaEngine"
    value: GENERALIZEDBLACKSCHOLESPROCESS
    steps: int


class BinomialTianVanillaEngine(BaseModel):
    resource_name: Optional[
        Literal["BinomialTianVanillaEngine"]
    ] = "BinomialTianVanillaEngine"
    value: GENERALIZEDBLACKSCHOLESPROCESS
    steps: int


class BinomialJoshi4VanillaEngine(BaseModel):
    resource_name: Optional[
        Literal["BinomialJoshi4VanillaEngine"]
    ] = "BinomialJoshi4VanillaEngine"
    value: GENERALIZEDBLACKSCHOLESPROCESS
    steps: int


class FDAmericanEngine(BaseModel):
    resource_name: Optional[Literal["FDAmericanEngine"]] = "FDAmericanEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    timeSteps: Optional[int] = None
    gridPoints: Optional[int] = None
    timeDependent: Optional[bool] = None


class FDShoutEngine(BaseModel):
    resource_name: Optional[Literal["FDShoutEngine"]] = "FDShoutEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    timeSteps: Optional[int] = None
    gridPoints: Optional[int] = None
    timeDependent: Optional[bool] = None


class ContinuousArithmeticAsianLevyEngine(BaseModel):
    resource_name: Optional[
        Literal["ContinuousArithmeticAsianLevyEngine"]
    ] = "ContinuousArithmeticAsianLevyEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    runningAverage: QuoteHandle
    startDate: Date


class FdBlackScholesAsianEngine(BaseModel):
    resource_name: Optional[
        Literal["FdBlackScholesAsianEngine"]
    ] = "FdBlackScholesAsianEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    tGrid: int
    xGrid: int
    aGrid: int


class BaroneAdesiWhaleyApproximationEngine(BaseModel):
    resource_name: Optional[
        Literal["BaroneAdesiWhaleyApproximationEngine"]
    ] = "BaroneAdesiWhaleyApproximationEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class BjerksundStenslandApproximationEngine(BaseModel):
    resource_name: Optional[
        Literal["BjerksundStenslandApproximationEngine"]
    ] = "BjerksundStenslandApproximationEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class JuQuadraticApproximationEngine(BaseModel):
    resource_name: Optional[
        Literal["JuQuadraticApproximationEngine"]
    ] = "JuQuadraticApproximationEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class AnalyticDigitalAmericanEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticDigitalAmericanEngine"]
    ] = "AnalyticDigitalAmericanEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class AnalyticDigitalAmericanKOEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticDigitalAmericanKOEngine"]
    ] = "AnalyticDigitalAmericanKOEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class AnalyticDividendEuropeanEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticDividendEuropeanEngine"]
    ] = "AnalyticDividendEuropeanEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class AnalyticBarrierEngine(BaseModel):
    resource_name: Optional[Literal["AnalyticBarrierEngine"]] = "AnalyticBarrierEngine"
    value: GENERALIZEDBLACKSCHOLESPROCESS


class FdmQuantoHelper(BaseModel):
    resource_name: Optional[Literal["FdmQuantoHelper"]] = "FdmQuantoHelper"
    rTS: YIELDTERMSTRUCTURE
    fTS: YIELDTERMSTRUCTURE
    fxVolTS: BLACKVOLTERMSTRUCTURE
    equityFxCorrelation: float
    exchRateATMlevel: float


class AnalyticCEVEngine(BaseModel):
    resource_name: Optional[Literal["AnalyticCEVEngine"]] = "AnalyticCEVEngine"
    f0: float
    alpha: float
    beta: float
    rTS: YieldTermStructureHandle


class AnalyticBinaryBarrierEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticBinaryBarrierEngine"]
    ] = "AnalyticBinaryBarrierEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class ForwardEuropeanEngine(BaseModel):
    resource_name: Optional[Literal["ForwardEuropeanEngine"]] = "ForwardEuropeanEngine"
    value: GENERALIZEDBLACKSCHOLESPROCESS


class QuantoEuropeanEngine(BaseModel):
    resource_name: Optional[Literal["QuantoEuropeanEngine"]] = "QuantoEuropeanEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    foreignRiskFreeRate: YieldTermStructureHandle
    exchangeRateVolatility: BlackVolTermStructureHandle
    correlation: QuoteHandle


class QuantoForwardEuropeanEngine(BaseModel):
    resource_name: Optional[
        Literal["QuantoForwardEuropeanEngine"]
    ] = "QuantoForwardEuropeanEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    foreignRiskFreeRate: YieldTermStructureHandle
    exchangeRateVolatility: BlackVolTermStructureHandle
    correlation: QuoteHandle


class BlackCalculator(BaseModel):
    resource_name: Optional[Literal["BlackCalculator"]] = "BlackCalculator"
    payoff: STRIKEDTYPEPAYOFF
    forward: float
    stdDev: float
    discount: Optional[float] = None


class AnalyticContinuousGeometricAveragePriceAsianEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticContinuousGeometricAveragePriceAsianEngine"]
    ] = "AnalyticContinuousGeometricAveragePriceAsianEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class AnalyticDiscreteGeometricAveragePriceAsianEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticDiscreteGeometricAveragePriceAsianEngine"]
    ] = "AnalyticDiscreteGeometricAveragePriceAsianEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class AnalyticDiscreteGeometricAverageStrikeAsianEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticDiscreteGeometricAverageStrikeAsianEngine"]
    ] = "AnalyticDiscreteGeometricAverageStrikeAsianEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class VarianceGammaEngine(BaseModel):
    resource_name: Optional[Literal["VarianceGammaEngine"]] = "VarianceGammaEngine"
    process: VarianceGammaProcess


class FFTVarianceGammaEngine(BaseModel):
    resource_name: Optional[
        Literal["FFTVarianceGammaEngine"]
    ] = "FFTVarianceGammaEngine"
    process: VarianceGammaProcess
    logStrikeSpacing: Optional[float] = None


class AnalyticDoubleBarrierEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticDoubleBarrierEngine"]
    ] = "AnalyticDoubleBarrierEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    series: Optional[int] = None


class WulinYongDoubleBarrierEngine(BaseModel):
    resource_name: Optional[
        Literal["WulinYongDoubleBarrierEngine"]
    ] = "WulinYongDoubleBarrierEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    series: Optional[int] = None


class AnalyticDoubleBarrierBinaryEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticDoubleBarrierBinaryEngine"]
    ] = "AnalyticDoubleBarrierBinaryEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class VanillaSwingOption(BaseModel):
    resource_name: Optional[Literal["VanillaSwingOption"]] = "VanillaSwingOption"
    payoff: PAYOFF
    ex: SwingExercise
    minExerciseRights: int
    maxExerciseRights: int


class FdSimpleExtOUJumpSwingEngine(BaseModel):
    resource_name: Optional[
        Literal["FdSimpleExtOUJumpSwingEngine"]
    ] = "FdSimpleExtOUJumpSwingEngine"


class PlainVanillaPayoff(BaseModel):
    resource_name: Optional[Literal["PlainVanillaPayoff"]] = "PlainVanillaPayoff"
    type: enums.OptionType
    strike: float


class PercentageStrikePayoff(BaseModel):
    resource_name: Optional[
        Literal["PercentageStrikePayoff"]
    ] = "PercentageStrikePayoff"
    type: enums.OptionType
    moneyness: float


class CashOrNothingPayoff(BaseModel):
    resource_name: Optional[Literal["CashOrNothingPayoff"]] = "CashOrNothingPayoff"
    type: enums.OptionType
    strike: float
    payoff: float


class AssetOrNothingPayoff(BaseModel):
    resource_name: Optional[Literal["AssetOrNothingPayoff"]] = "AssetOrNothingPayoff"
    type: enums.OptionType
    strike: float


class SuperSharePayoff(BaseModel):
    resource_name: Optional[Literal["SuperSharePayoff"]] = "SuperSharePayoff"
    type: enums.OptionType
    strike: float
    increment: float


class GapPayoff(BaseModel):
    resource_name: Optional[Literal["GapPayoff"]] = "GapPayoff"
    type: enums.OptionType
    strike: float
    strikePayoff: float


class VanillaForwardPayoff(BaseModel):
    resource_name: Optional[Literal["VanillaForwardPayoff"]] = "VanillaForwardPayoff"
    type: enums.OptionType
    strike: float


class BasketPayoff(BaseModel):
    resource_name: Optional[Literal["BasketPayoff"]] = "BasketPayoff"


class MinBasketPayoff(BaseModel):
    resource_name: Optional[Literal["MinBasketPayoff"]] = "MinBasketPayoff"
    p: PAYOFF


class MaxBasketPayoff(BaseModel):
    resource_name: Optional[Literal["MaxBasketPayoff"]] = "MaxBasketPayoff"
    p: PAYOFF


class AverageBasketPayoff(BaseModel):
    resource_name: Optional[Literal["AverageBasketPayoff"]] = "AverageBasketPayoff"
    p: PAYOFF
    a: Array


class SpreadBasketPayoff(BaseModel):
    resource_name: Optional[Literal["SpreadBasketPayoff"]] = "SpreadBasketPayoff"
    p: PAYOFF


class StulzEngine(BaseModel):
    resource_name: Optional[Literal["StulzEngine"]] = "StulzEngine"
    process1: GENERALIZEDBLACKSCHOLESPROCESS
    process2: GENERALIZEDBLACKSCHOLESPROCESS
    correlation: float


class KirkEngine(BaseModel):
    resource_name: Optional[Literal["KirkEngine"]] = "KirkEngine"
    process1: BlackProcess
    process2: BlackProcess
    correlation: float


class HimalayaOption(BaseModel):
    resource_name: Optional[Literal["HimalayaOption"]] = "HimalayaOption"
    fixingDates: List[Date]
    strike: float


class CallabilityPrice(BaseModel):
    resource_name: Optional[Literal["CallabilityPrice"]] = "CallabilityPrice"
    amount: float
    type: enums.CallabilityPriceType


class Callability(BaseModel):
    resource_name: Optional[Literal["Callability"]] = "Callability"
    price: CallabilityPrice
    type: enums.CallabilityType
    date: Date


class SoftCallability(BaseModel):
    resource_name: Optional[Literal["SoftCallability"]] = "SoftCallability"
    price: CallabilityPrice
    date: Date
    trigger: float


class TimeBasket(BaseModel):
    resource_name: Optional[Literal["TimeBasket"]] = "TimeBasket"
    arg_0: Optional[List[Date]] = None
    arg_1: Optional[List[float]] = None


class Swap(BaseModel):
    resource_name: Optional[Literal["Swap"]] = "Swap"
    firstLeg: List[CASHFLOW]
    secondLeg: List[CASHFLOW]


class VanillaSwap(BaseModel):
    resource_name: Optional[Literal["VanillaSwap"]] = "VanillaSwap"
    type: enums.VanillaSwapType
    nominal: float
    fixedSchedule: Schedule
    fixedRate: float
    fixedDayCount: DAYCOUNTER
    floatSchedule: Schedule
    index: IBORINDEX
    spread: float
    floatingDayCount: DAYCOUNTER


class MakeVanillaSwap(BaseModel):
    resource_name: Optional[Literal["MakeVanillaSwap"]] = "MakeVanillaSwap"
    swapTenor: Period
    index: IBORINDEX
    fixedRate: float
    forwardStart: Period


class NonstandardSwap(BaseModel):
    resource_name: Optional[Literal["NonstandardSwap"]] = "NonstandardSwap"
    type: enums.VanillaSwapType
    fixedNominal: List[float]
    floatingNominal: List[float]
    fixedSchedule: Schedule
    fixedRate: List[float]
    fixedDayCount: DAYCOUNTER
    floatSchedule: Schedule
    index: IBORINDEX
    gearing: List[float]
    spread: List[float]
    floatDayCount: DAYCOUNTER
    intermediateCapitalExchange: Optional[bool] = None
    finalCapitalExchange: Optional[bool] = None
    paymentConvention: Optional[enums.BusinessDayConvention] = None


class DiscountingSwapEngine(BaseModel):
    resource_name: Optional[Literal["DiscountingSwapEngine"]] = "DiscountingSwapEngine"
    discountCurve: YieldTermStructureHandle
    includeSettlementDateFlows: bool
    settlementDate: Optional[Date] = None
    npvDate: Optional[Date] = None


class FloatFloatSwap(BaseModel):
    resource_name: Optional[Literal["FloatFloatSwap"]] = "FloatFloatSwap"
    type: enums.VanillaSwapType
    nominal1: List[float]
    nominal2: List[float]
    schedule1: Schedule
    index1: INTERESTRATEINDEX
    dayCount1: DAYCOUNTER
    schedule2: Schedule
    index2: INTERESTRATEINDEX
    dayCount2: DAYCOUNTER
    intermediateCapitalExchange: Optional[bool] = None
    finalCapitalExchange: Optional[bool] = None
    gearing1: Optional[List[float]] = None
    spread1: Optional[List[float]] = None
    cappedRate1: Optional[List[float]] = None
    flooredRate1: Optional[List[float]] = None
    gearing2: Optional[List[float]] = None
    spread2: Optional[List[float]] = None
    cappedRate2: Optional[List[float]] = None
    flooredRate2: Optional[List[float]] = None
    paymentConvention1: Optional[enums.BusinessDayConvention] = None
    paymentConvention2: Optional[enums.BusinessDayConvention] = None


class MakeOIS(BaseModel):
    resource_name: Optional[Literal["MakeOIS"]] = "MakeOIS"
    swapTenor: Period
    overnightIndex: OVERNIGHTINDEX
    fixedRate: Optional[float] = None
    fwdStart: Optional[Period] = None


class OvernightIndexedSwapIndex(BaseModel):
    resource_name: Optional[
        Literal["OvernightIndexedSwapIndex"]
    ] = "OvernightIndexedSwapIndex"
    familyName: str
    tenor: Period
    settlementDays: float
    currency: CURRENCY
    overnightIndex: OVERNIGHTINDEX
    telescopicValueDates: Optional[bool] = None


class Seasonality(BaseModel):
    resource_name: Optional[Literal["Seasonality"]] = "Seasonality"


class MultiplicativePriceSeasonality(BaseModel):
    resource_name: Optional[
        Literal["MultiplicativePriceSeasonality"]
    ] = "MultiplicativePriceSeasonality"
    seasonalityBaseDate: Date
    frequency: float
    seasonalityFactors: List[float]


class InflationTermStructure(BaseModel):
    resource_name: Optional[
        Literal["InflationTermStructure"]
    ] = "InflationTermStructure"


class YoYInflationTermStructure(BaseModel):
    resource_name: Optional[
        Literal["YoYInflationTermStructure"]
    ] = "YoYInflationTermStructure"


class YoYInflationTermStructureHandle(BaseModel):
    resource_name: Optional[
        Literal["YoYInflationTermStructureHandle"]
    ] = "YoYInflationTermStructureHandle"
    value: Optional[YoYInflationTermStructure] = None


class ZeroInflationTermStructure(BaseModel):
    resource_name: Optional[
        Literal["ZeroInflationTermStructure"]
    ] = "ZeroInflationTermStructure"


class ZeroInflationTermStructureHandle(BaseModel):
    resource_name: Optional[
        Literal["ZeroInflationTermStructureHandle"]
    ] = "ZeroInflationTermStructureHandle"
    value: Optional[ZeroInflationTermStructure] = None


class Region(BaseModel):
    resource_name: Optional[Literal["Region"]] = "Region"


class CustomRegion(BaseModel):
    resource_name: Optional[Literal["CustomRegion"]] = "CustomRegion"
    name: str
    code: str


class InflationIndex(BaseModel):
    resource_name: Optional[Literal["InflationIndex"]] = "InflationIndex"


class ZeroInflationIndex(BaseModel):
    resource_name: Optional[Literal["ZeroInflationIndex"]] = "ZeroInflationIndex"
    familyName: str
    region: REGION
    revised: bool
    interpolated: bool
    frequency: float
    availabilityLag: Period
    currency: CURRENCY
    h: Optional[ZeroInflationTermStructureHandle] = None


class YoYInflationIndex(BaseModel):
    resource_name: Optional[Literal["YoYInflationIndex"]] = "YoYInflationIndex"


class EUHICP(BaseModel):
    resource_name: Optional[Literal["EUHICP"]] = "EUHICP"
    interpolated: bool
    h: Optional[ZeroInflationTermStructureHandle] = None


class EUHICPXT(BaseModel):
    resource_name: Optional[Literal["EUHICPXT"]] = "EUHICPXT"
    interpolated: bool
    h: Optional[ZeroInflationTermStructureHandle] = None


class FRHICP(BaseModel):
    resource_name: Optional[Literal["FRHICP"]] = "FRHICP"
    interpolated: bool
    h: Optional[ZeroInflationTermStructureHandle] = None


class UKRPI(BaseModel):
    resource_name: Optional[Literal["UKRPI"]] = "UKRPI"
    interpolated: bool
    h: Optional[ZeroInflationTermStructureHandle] = None


class USCPI(BaseModel):
    resource_name: Optional[Literal["USCPI"]] = "USCPI"
    interpolated: bool
    h: Optional[ZeroInflationTermStructureHandle] = None


class ZACPI(BaseModel):
    resource_name: Optional[Literal["ZACPI"]] = "ZACPI"
    interpolated: bool
    h: Optional[ZeroInflationTermStructureHandle] = None


class YYEUHICP(BaseModel):
    resource_name: Optional[Literal["YYEUHICP"]] = "YYEUHICP"
    interpolated: bool
    h: Optional[YoYInflationTermStructureHandle] = None


class YYEUHICPXT(BaseModel):
    resource_name: Optional[Literal["YYEUHICPXT"]] = "YYEUHICPXT"
    interpolated: bool
    h: Optional[YoYInflationTermStructureHandle] = None


class YYFRHICP(BaseModel):
    resource_name: Optional[Literal["YYFRHICP"]] = "YYFRHICP"
    interpolated: bool
    h: Optional[YoYInflationTermStructureHandle] = None


class YYUKRPI(BaseModel):
    resource_name: Optional[Literal["YYUKRPI"]] = "YYUKRPI"
    interpolated: bool
    h: Optional[YoYInflationTermStructureHandle] = None


class YYUSCPI(BaseModel):
    resource_name: Optional[Literal["YYUSCPI"]] = "YYUSCPI"
    interpolated: bool
    h: Optional[YoYInflationTermStructureHandle] = None


class YYZACPI(BaseModel):
    resource_name: Optional[Literal["YYZACPI"]] = "YYZACPI"
    interpolated: bool
    h: Optional[YoYInflationTermStructureHandle] = None


class CPI(BaseModel):
    resource_name: Optional[Literal["CPI"]] = "CPI"


class InflationCoupon(BaseModel):
    resource_name: Optional[Literal["InflationCoupon"]] = "InflationCoupon"


class CPICoupon(BaseModel):
    resource_name: Optional[Literal["CPICoupon"]] = "CPICoupon"


class ZeroCouponInflationSwapHelper(BaseModel):
    resource_name: Optional[
        Literal["ZeroCouponInflationSwapHelper"]
    ] = "ZeroCouponInflationSwapHelper"


class YearOnYearInflationSwapHelper(BaseModel):
    resource_name: Optional[
        Literal["YearOnYearInflationSwapHelper"]
    ] = "YearOnYearInflationSwapHelper"


class YoYInflationCap(BaseModel):
    resource_name: Optional[Literal["YoYInflationCap"]] = "YoYInflationCap"
    leg: List[CASHFLOW]
    capRates: List[float]


class YoYInflationFloor(BaseModel):
    resource_name: Optional[Literal["YoYInflationFloor"]] = "YoYInflationFloor"
    leg: List[CASHFLOW]
    floorRates: List[float]


class YoYInflationCollar(BaseModel):
    resource_name: Optional[Literal["YoYInflationCollar"]] = "YoYInflationCollar"
    leg: List[CASHFLOW]
    capRates: List[float]
    floorRates: List[float]


class ShortRateModel(BaseModel):
    resource_name: Optional[Literal["ShortRateModel"]] = "ShortRateModel"


class ShortRateModelHandle(BaseModel):
    resource_name: Optional[Literal["ShortRateModelHandle"]] = "ShortRateModelHandle"
    value: Optional[SHORTRATEMODEL] = None


class OneFactorAffineModel(BaseModel):
    resource_name: Optional[Literal["OneFactorAffineModel"]] = "OneFactorAffineModel"


class Vasicek(BaseModel):
    resource_name: Optional[Literal["Vasicek"]] = "Vasicek"
    r0: Optional[float] = None
    a: Optional[float] = None
    b: Optional[float] = None
    sigma: Optional[float] = None
    lambda_: Optional[float] = Field(None, alias="lambda")


class HullWhite(BaseModel):
    resource_name: Optional[Literal["HullWhite"]] = "HullWhite"
    termStructure: YieldTermStructureHandle
    a: Optional[float] = None
    sigma: Optional[float] = None


class BlackKarasinski(BaseModel):
    resource_name: Optional[Literal["BlackKarasinski"]] = "BlackKarasinski"
    termStructure: YieldTermStructureHandle
    a: Optional[float] = None
    sigma: Optional[float] = None


class G2(BaseModel):
    resource_name: Optional[Literal["G2"]] = "G2"
    termStructure: YieldTermStructureHandle
    a: Optional[float] = None
    sigma: Optional[float] = None
    b: Optional[float] = None
    eta: Optional[float] = None
    rho: Optional[float] = None


class JamshidianSwaptionEngine(BaseModel):
    resource_name: Optional[
        Literal["JamshidianSwaptionEngine"]
    ] = "JamshidianSwaptionEngine"
    model: ONEFACTORAFFINEMODEL
    termStructure: Optional[YieldTermStructureHandle] = None


class TreeSwaptionEngine(BaseModel):
    resource_name: Optional[Literal["TreeSwaptionEngine"]] = "TreeSwaptionEngine"
    model: ShortRateModelHandle
    timeSteps: int
    termStructure: Optional[YieldTermStructureHandle] = None


class AnalyticCapFloorEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticCapFloorEngine"]
    ] = "AnalyticCapFloorEngine"
    model: ONEFACTORAFFINEMODEL
    termStructure: Optional[YieldTermStructureHandle] = None


class TreeCapFloorEngine(BaseModel):
    resource_name: Optional[Literal["TreeCapFloorEngine"]] = "TreeCapFloorEngine"
    model: SHORTRATEMODEL
    grid: TimeGrid
    termStructure: Optional[YieldTermStructureHandle] = None


class G2SwaptionEngine(BaseModel):
    resource_name: Optional[Literal["G2SwaptionEngine"]] = "G2SwaptionEngine"
    model: G2
    range: float
    intervals: int


class ZeroCouponBond(BaseModel):
    resource_name: Optional[Literal["ZeroCouponBond"]] = "ZeroCouponBond"
    settlementDays: float
    calendar: CALENDAR
    faceAmount: float
    maturityDate: Date
    paymentConvention: Optional[enums.BusinessDayConvention] = None
    redemption: Optional[float] = None
    issueDate: Optional[Date] = None


class FixedRateBond(BaseModel):
    resource_name: Optional[Literal["FixedRateBond"]] = "FixedRateBond"
    settlementDays: int
    couponCalendar: CALENDAR
    faceAmount: float
    startDate: Date
    maturityDate: Date
    tenor: Period
    coupons: List[float]
    accrualDayCounter: DAYCOUNTER
    accrualConvention: Optional[enums.BusinessDayConvention] = None
    paymentConvention: Optional[enums.BusinessDayConvention] = None
    redemption: Optional[float] = None
    issueDate: Optional[Date] = None
    stubDate: Optional[Date] = None
    rule: Optional[enums.DateGenerationRule] = None
    endOfMonth: Optional[bool] = None
    paymentCalendar: Optional[CALENDAR] = None
    exCouponPeriod: Optional[Period] = None
    exCouponCalendar: Optional[CALENDAR] = None
    exCouponConvention: Optional[enums.BusinessDayConvention] = None
    exCouponEndOfMonth: Optional[bool] = None


class AmortizingFixedRateBond(BaseModel):
    resource_name: Optional[
        Literal["AmortizingFixedRateBond"]
    ] = "AmortizingFixedRateBond"
    settlementDays: int
    paymentCalendar: CALENDAR
    faceAmount: float
    startDate: Date
    bondTenor: Period
    sinkingFrequency: float
    coupon: float
    accrualDayCounter: DAYCOUNTER
    paymentConvention: Optional[enums.BusinessDayConvention] = None
    issueDate: Optional[Date] = None


class AmortizingFloatingRateBond(BaseModel):
    resource_name: Optional[
        Literal["AmortizingFloatingRateBond"]
    ] = "AmortizingFloatingRateBond"
    settlementDays: int
    notional: List[float]
    schedule: Schedule
    index: IBORINDEX
    accrualDayCounter: DAYCOUNTER
    paymentConvention: Optional[enums.BusinessDayConvention] = None
    fixingDays: Optional[int] = None
    gearings: Optional[List[float]] = None
    spreads: Optional[List[float]] = None
    caps: Optional[List[float]] = None
    floors: Optional[List[float]] = None
    inArrears: Optional[bool] = None
    issueDate: Optional[Date] = None


class FloatingRateBond(BaseModel):
    resource_name: Optional[Literal["FloatingRateBond"]] = "FloatingRateBond"
    settlementDays: int
    faceAmount: float
    schedule: Schedule
    index: IBORINDEX
    paymentDayCounter: DAYCOUNTER
    paymentConvention: Optional[enums.BusinessDayConvention] = None
    fixingDays: Optional[int] = None
    gearings: Optional[List[float]] = None
    spreads: Optional[List[float]] = None
    caps: Optional[List[float]] = None
    floors: Optional[List[float]] = None
    inArrears: Optional[bool] = None
    redemption: Optional[float] = None
    issueDate: Optional[Date] = None
    exCouponPeriod: Optional[Period] = None
    exCouponCalendar: Optional[CALENDAR] = None
    exCouponConvention: Optional[enums.BusinessDayConvention] = None
    exCouponEndOfMonth: Optional[bool] = None


class CmsRateBond(BaseModel):
    resource_name: Optional[Literal["CmsRateBond"]] = "CmsRateBond"
    settlementDays: int
    faceAmount: float
    schedule: Schedule
    index: SWAPINDEX
    paymentDayCounter: DAYCOUNTER
    paymentConvention: enums.BusinessDayConvention
    fixingDays: float
    gearings: List[float]
    spreads: List[float]
    caps: List[float]
    floors: List[float]
    inArrears: Optional[bool] = None
    redemption: Optional[float] = None
    issueDate: Optional[Date] = None


class DiscountingBondEngine(BaseModel):
    resource_name: Optional[Literal["DiscountingBondEngine"]] = "DiscountingBondEngine"
    discountCurve: YieldTermStructureHandle


class CallableBond(BaseModel):
    resource_name: Optional[Literal["CallableBond"]] = "CallableBond"


class CallableFixedRateBond(BaseModel):
    resource_name: Optional[Literal["CallableFixedRateBond"]] = "CallableFixedRateBond"
    settlementDays: int
    faceAmount: float
    schedule: Schedule
    coupons: List[float]
    accrualDayCounter: DAYCOUNTER
    paymentConvention: enums.BusinessDayConvention
    redemption: float
    issueDate: Date
    putCallSchedule: List[CALLABILITY]


class TreeCallableFixedRateBondEngine(BaseModel):
    resource_name: Optional[
        Literal["TreeCallableFixedRateBondEngine"]
    ] = "TreeCallableFixedRateBondEngine"
    model: SHORTRATEMODEL
    grid: TimeGrid
    termStructure: Optional[YieldTermStructureHandle] = None


class BlackCallableFixedRateBondEngine(BaseModel):
    resource_name: Optional[
        Literal["BlackCallableFixedRateBondEngine"]
    ] = "BlackCallableFixedRateBondEngine"
    fwdYieldVol: QuoteHandle
    discountCurve: YieldTermStructureHandle


class BondFunctions(BaseModel):
    resource_name: Optional[Literal["BondFunctions"]] = "BondFunctions"


class CapFloor(BaseModel):
    resource_name: Optional[Literal["CapFloor"]] = "CapFloor"
    Cap = 0
    Collar = 2
    Floor = 1


class Cap(BaseModel):
    resource_name: Optional[Literal["Cap"]] = "Cap"
    leg: List[CASHFLOW]
    capRates: List[float]


class Floor(BaseModel):
    resource_name: Optional[Literal["Floor"]] = "Floor"
    leg: List[CASHFLOW]
    floorRates: List[float]


class Collar(BaseModel):
    resource_name: Optional[Literal["Collar"]] = "Collar"
    leg: List[CASHFLOW]
    capRates: List[float]
    floorRates: List[float]


class BlackCapFloorEngine(BaseModel):
    resource_name: Optional[Literal["BlackCapFloorEngine"]] = "BlackCapFloorEngine"
    termStructure: YieldTermStructureHandle
    vol: QuoteHandle


class BachelierCapFloorEngine(BaseModel):
    resource_name: Optional[
        Literal["BachelierCapFloorEngine"]
    ] = "BachelierCapFloorEngine"
    termStructure: YieldTermStructureHandle
    vol: QuoteHandle


class Dividend(BaseModel):
    resource_name: Optional[Literal["Dividend"]] = "Dividend"


class FixedDividend(BaseModel):
    resource_name: Optional[Literal["FixedDividend"]] = "FixedDividend"
    amount: float
    date: Date


class FractionalDividend(BaseModel):
    resource_name: Optional[Literal["FractionalDividend"]] = "FractionalDividend"
    rate: float
    date: Date


class Futures(BaseModel):
    resource_name: Optional[Literal["Futures"]] = "Futures"


class OvernightIndexFuture(BaseModel):
    resource_name: Optional[Literal["OvernightIndexFuture"]] = "OvernightIndexFuture"


class Pillar(BaseModel):
    resource_name: Optional[Literal["Pillar"]] = "Pillar"


class RateHelper(BaseModel):
    resource_name: Optional[Literal["RateHelper"]] = "RateHelper"


class DepositRateHelper(BaseModel):
    resource_name: Optional[Literal["DepositRateHelper"]] = "DepositRateHelper"
    rate: QuoteHandle
    tenor: Period
    fixingDays: float
    calendar: CALENDAR
    convention: enums.BusinessDayConvention
    endOfMonth: bool
    dayCounter: DAYCOUNTER


class FixedRateBondHelper(BaseModel):
    resource_name: Optional[Literal["FixedRateBondHelper"]] = "FixedRateBondHelper"
    cleanPrice: QuoteHandle
    settlementDays: int
    faceAmount: float
    schedule: Schedule
    coupons: List[float]
    paymentDayCounter: DAYCOUNTER
    paymentConvention: Optional[enums.BusinessDayConvention] = None
    redemption: Optional[float] = None
    issueDate: Optional[Date] = None
    paymentCalendar: Optional[CALENDAR] = None
    exCouponPeriod: Optional[Period] = None
    exCouponCalendar: Optional[CALENDAR] = None
    exCouponConvention: Optional[enums.BusinessDayConvention] = None
    exCouponEndOfMonth: Optional[bool] = None
    useCleanPrice: Optional[bool] = None


class DatedOISRateHelper(BaseModel):
    resource_name: Optional[Literal["DatedOISRateHelper"]] = "DatedOISRateHelper"
    startDate: Date
    endDate: Date
    rate: QuoteHandle
    index: OVERNIGHTINDEX
    discountingCurve: Optional[YieldTermStructureHandle] = None


class FxSwapRateHelper(BaseModel):
    resource_name: Optional[Literal["FxSwapRateHelper"]] = "FxSwapRateHelper"
    fwdPoint: QuoteHandle
    spotFx: QuoteHandle
    tenor: Period
    fixingDays: float
    calendar: CALENDAR
    convention: enums.BusinessDayConvention
    endOfMonth: bool
    isFxBaseCurrencyCollateralCurrency: bool
    collateralCurve: YieldTermStructureHandle
    tradingCalendar: Optional[CALENDAR] = None


class Discount(BaseModel):
    resource_name: Optional[Literal["Discount"]] = "Discount"


class ZeroYield(BaseModel):
    resource_name: Optional[Literal["ZeroYield"]] = "ZeroYield"


class ForwardRate(BaseModel):
    resource_name: Optional[Literal["ForwardRate"]] = "ForwardRate"


class IterativeBootstrap(BaseModel):
    resource_name: Optional[Literal["IterativeBootstrap"]] = "IterativeBootstrap"
    accuracy: Optional[Optional[float]] = None
    minValue: Optional[Optional[float]] = None
    maxValue: Optional[Optional[float]] = None


class PiecewiseFlatForward(BaseModel):
    resource_name: Optional[Literal["PiecewiseFlatForward"]] = "PiecewiseFlatForward"


class PiecewiseLogLinearDiscount(BaseModel):
    resource_name: Optional[
        Literal["PiecewiseLogLinearDiscount"]
    ] = "PiecewiseLogLinearDiscount"


class PiecewiseLinearForward(BaseModel):
    resource_name: Optional[
        Literal["PiecewiseLinearForward"]
    ] = "PiecewiseLinearForward"


class PiecewiseLinearZero(BaseModel):
    resource_name: Optional[Literal["PiecewiseLinearZero"]] = "PiecewiseLinearZero"


class PiecewiseCubicZero(BaseModel):
    resource_name: Optional[Literal["PiecewiseCubicZero"]] = "PiecewiseCubicZero"


class PiecewiseLogCubicDiscount(BaseModel):
    resource_name: Optional[
        Literal["PiecewiseLogCubicDiscount"]
    ] = "PiecewiseLogCubicDiscount"


class PiecewiseSplineCubicDiscount(BaseModel):
    resource_name: Optional[
        Literal["PiecewiseSplineCubicDiscount"]
    ] = "PiecewiseSplineCubicDiscount"


class PiecewiseKrugerZero(BaseModel):
    resource_name: Optional[Literal["PiecewiseKrugerZero"]] = "PiecewiseKrugerZero"


class PiecewiseKrugerLogDiscount(BaseModel):
    resource_name: Optional[
        Literal["PiecewiseKrugerLogDiscount"]
    ] = "PiecewiseKrugerLogDiscount"


class PiecewiseConvexMonotoneZero(BaseModel):
    resource_name: Optional[
        Literal["PiecewiseConvexMonotoneZero"]
    ] = "PiecewiseConvexMonotoneZero"


class GlobalBootstrap(BaseModel):
    resource_name: Optional[Literal["GlobalBootstrap"]] = "GlobalBootstrap"
    additionalHelpers: List[RATEHELPER]
    additionalDates: List[Date]
    accuracy: Optional[Optional[float]] = None


class GlobalLinearSimpleZeroCurve(BaseModel):
    resource_name: Optional[
        Literal["GlobalLinearSimpleZeroCurve"]
    ] = "GlobalLinearSimpleZeroCurve"


class DefaultProbabilityTermStructure(BaseModel):
    resource_name: Optional[
        Literal["DefaultProbabilityTermStructure"]
    ] = "DefaultProbabilityTermStructure"


class DefaultProbabilityTermStructureHandle(BaseModel):
    resource_name: Optional[
        Literal["DefaultProbabilityTermStructureHandle"]
    ] = "DefaultProbabilityTermStructureHandle"
    value: Optional[DEFAULTPROBABILITYTERMSTRUCTURE] = None


class FlatHazardRate(BaseModel):
    resource_name: Optional[Literal["FlatHazardRate"]] = "FlatHazardRate"
    settlementDays: int
    calendar: CALENDAR
    hazardRate: QuoteHandle
    dayCounter: DAYCOUNTER


class DefaultProbabilityHelper(BaseModel):
    resource_name: Optional[
        Literal["DefaultProbabilityHelper"]
    ] = "DefaultProbabilityHelper"


class SpreadCdsHelper(BaseModel):
    resource_name: Optional[Literal["SpreadCdsHelper"]] = "SpreadCdsHelper"
    spread: float
    tenor: Period
    settlementDays: int
    calendar: CALENDAR
    frequency: float
    convention: enums.BusinessDayConvention
    rule: enums.DateGenerationRule
    dayCounter: DAYCOUNTER
    recoveryRate: float
    discountCurve: YieldTermStructureHandle
    settlesAccrual: Optional[bool] = None
    paysAtDefaultTime: Optional[bool] = None


class UpfrontCdsHelper(BaseModel):
    resource_name: Optional[Literal["UpfrontCdsHelper"]] = "UpfrontCdsHelper"
    upfront: float
    spread: float
    tenor: Period
    settlementDays: int
    calendar: CALENDAR
    frequency: float
    convention: enums.BusinessDayConvention
    rule: enums.DateGenerationRule
    dayCounter: DAYCOUNTER
    recoveryRate: float
    discountCurve: YieldTermStructureHandle
    upfrontSettlementDays: Optional[float] = None
    settlesAccrual: Optional[bool] = None
    paysAtDefaultTime: Optional[bool] = None


class HazardRate(BaseModel):
    resource_name: Optional[Literal["HazardRate"]] = "HazardRate"


class DefaultDensity(BaseModel):
    resource_name: Optional[Literal["DefaultDensity"]] = "DefaultDensity"


class PiecewiseFlatHazardRate(BaseModel):
    resource_name: Optional[
        Literal["PiecewiseFlatHazardRate"]
    ] = "PiecewiseFlatHazardRate"


class Protection(BaseModel):
    resource_name: Optional[Literal["Protection"]] = "Protection"


class Claim(BaseModel):
    resource_name: Optional[Literal["Claim"]] = "Claim"


class FaceValueClaim(BaseModel):
    resource_name: Optional[Literal["FaceValueClaim"]] = "FaceValueClaim"


class MidPointCdsEngine(BaseModel):
    resource_name: Optional[Literal["MidPointCdsEngine"]] = "MidPointCdsEngine"
    probability: DefaultProbabilityTermStructureHandle
    recoveryRate: float
    discountCurve: YieldTermStructureHandle


class IntegralCdsEngine(BaseModel):
    resource_name: Optional[Literal["IntegralCdsEngine"]] = "IntegralCdsEngine"
    integrationStep: Period
    probability: DefaultProbabilityTermStructureHandle
    recoveryRate: float
    discountCurve: YieldTermStructureHandle
    includeSettlementDateFlows: Optional[bool] = None


class BlackCdsOptionEngine(BaseModel):
    resource_name: Optional[Literal["BlackCdsOptionEngine"]] = "BlackCdsOptionEngine"
    value: DefaultProbabilityTermStructureHandle
    recoveryRate: float
    termStructure: YieldTermStructureHandle
    vol: QuoteHandle


class NormalDistribution(BaseModel):
    resource_name: Optional[Literal["NormalDistribution"]] = "NormalDistribution"
    average: Optional[float] = None
    sigma: Optional[float] = None


class CumulativeNormalDistribution(BaseModel):
    resource_name: Optional[
        Literal["CumulativeNormalDistribution"]
    ] = "CumulativeNormalDistribution"
    average: Optional[float] = None
    sigma: Optional[float] = None


class InverseCumulativeNormal(BaseModel):
    resource_name: Optional[
        Literal["InverseCumulativeNormal"]
    ] = "InverseCumulativeNormal"
    average: Optional[float] = None
    sigma: Optional[float] = None


class MoroInverseCumulativeNormal(BaseModel):
    resource_name: Optional[
        Literal["MoroInverseCumulativeNormal"]
    ] = "MoroInverseCumulativeNormal"
    average: Optional[float] = None
    sigma: Optional[float] = None


class BivariateCumulativeNormalDistribution(BaseModel):
    resource_name: Optional[
        Literal["BivariateCumulativeNormalDistribution"]
    ] = "BivariateCumulativeNormalDistribution"
    rho: float


class BinomialDistribution(BaseModel):
    resource_name: Optional[Literal["BinomialDistribution"]] = "BinomialDistribution"
    p: float
    n: float


class CumulativeBinomialDistribution(BaseModel):
    resource_name: Optional[
        Literal["CumulativeBinomialDistribution"]
    ] = "CumulativeBinomialDistribution"
    p: float
    n: float


class BivariateCumulativeNormalDistributionDr78(BaseModel):
    resource_name: Optional[
        Literal["BivariateCumulativeNormalDistributionDr78"]
    ] = "BivariateCumulativeNormalDistributionDr78"
    rho: float


class BivariateCumulativeNormalDistributionWe04DP(BaseModel):
    resource_name: Optional[
        Literal["BivariateCumulativeNormalDistributionWe04DP"]
    ] = "BivariateCumulativeNormalDistributionWe04DP"
    rho: float


class CumulativeChiSquareDistribution(BaseModel):
    resource_name: Optional[
        Literal["CumulativeChiSquareDistribution"]
    ] = "CumulativeChiSquareDistribution"
    df: float


class NonCentralCumulativeChiSquareDistribution(BaseModel):
    resource_name: Optional[
        Literal["NonCentralCumulativeChiSquareDistribution"]
    ] = "NonCentralCumulativeChiSquareDistribution"
    df: float
    ncp: float


class InverseNonCentralCumulativeChiSquareDistribution(BaseModel):
    resource_name: Optional[
        Literal["InverseNonCentralCumulativeChiSquareDistribution"]
    ] = "InverseNonCentralCumulativeChiSquareDistribution"
    df: float
    ncp: float
    maxEvaluations: Optional[int] = None
    accuracy: Optional[float] = None


class CumulativeGammaDistribution(BaseModel):
    resource_name: Optional[
        Literal["CumulativeGammaDistribution"]
    ] = "CumulativeGammaDistribution"
    a: float


class GammaFunction(BaseModel):
    resource_name: Optional[Literal["GammaFunction"]] = "GammaFunction"


class PoissonDistribution(BaseModel):
    resource_name: Optional[Literal["PoissonDistribution"]] = "PoissonDistribution"
    mu: float


class CumulativePoissonDistribution(BaseModel):
    resource_name: Optional[
        Literal["CumulativePoissonDistribution"]
    ] = "CumulativePoissonDistribution"
    mu: float


class InverseCumulativePoisson(BaseModel):
    resource_name: Optional[
        Literal["InverseCumulativePoisson"]
    ] = "InverseCumulativePoisson"
    lambda_: float = Field(..., alias="lambda")


class StudentDistribution(BaseModel):
    resource_name: Optional[Literal["StudentDistribution"]] = "StudentDistribution"
    n: int


class CumulativeStudentDistribution(BaseModel):
    resource_name: Optional[
        Literal["CumulativeStudentDistribution"]
    ] = "CumulativeStudentDistribution"
    n: int


class InverseCumulativeStudent(BaseModel):
    resource_name: Optional[
        Literal["InverseCumulativeStudent"]
    ] = "InverseCumulativeStudent"
    n: int
    accuracy: Optional[float] = None
    maxIterations: Optional[int] = None


class Money(BaseModel):
    resource_name: Optional[Literal["Money"]] = "Money"
    currency: CURRENCY
    value: float


class ExchangeRate(BaseModel):
    resource_name: Optional[Literal["ExchangeRate"]] = "ExchangeRate"
    source: CURRENCY
    target: CURRENCY
    rate: float


class ExchangeRateManager(BaseModel):
    resource_name: Optional[Literal["ExchangeRateManager"]] = "ExchangeRateManager"


class Settings(BaseModel):
    resource_name: Optional[Literal["Settings"]] = "Settings"


class Fdm1dMesher(BaseModel):
    resource_name: Optional[Literal["Fdm1dMesher"]] = "Fdm1dMesher"
    size: int


class FdmBlackScholesMesher(BaseModel):
    resource_name: Optional[Literal["FdmBlackScholesMesher"]] = "FdmBlackScholesMesher"
    size: int
    process: GENERALIZEDBLACKSCHOLESPROCESS
    maturity: float
    strike: float
    xMinConstraint: Optional[Optional[float]] = None
    xMaxConstraint: Optional[Optional[float]] = None
    eps: Optional[float] = None
    scaleFactor: Optional[float] = None
    cPoint: Optional[List[Union[float, float]]] = None
    dividendSchedule: Optional[List[DIVIDEND]] = None
    fdmQuantoHelper: Optional[FdmQuantoHelper] = None
    spotAdjustment: Optional[float] = None


class Concentrating1dMesher(BaseModel):
    resource_name: Optional[Literal["Concentrating1dMesher"]] = "Concentrating1dMesher"
    start: float
    end: float
    size: int
    cPoints: List[List[Union[float, float, bool]]]
    tol: Optional[float] = None


class ExponentialJump1dMesher(BaseModel):
    resource_name: Optional[
        Literal["ExponentialJump1dMesher"]
    ] = "ExponentialJump1dMesher"
    steps: int
    beta: float
    jumpIntensity: float
    eta: float
    eps: Optional[float] = None


class FdmCEV1dMesher(BaseModel):
    resource_name: Optional[Literal["FdmCEV1dMesher"]] = "FdmCEV1dMesher"
    size: int
    f0: float
    alpha: float
    beta: float
    maturity: float
    eps: Optional[float] = None
    scaleFactor: Optional[float] = None
    cPoint: Optional[List[Union[float, float]]] = None


class FdmHestonVarianceMesher(BaseModel):
    resource_name: Optional[
        Literal["FdmHestonVarianceMesher"]
    ] = "FdmHestonVarianceMesher"
    size: int
    process: HESTONPROCESS
    maturity: float
    tAvgSteps: Optional[int] = None
    epsilon: Optional[float] = None


class FdmHestonLocalVolatilityVarianceMesher(BaseModel):
    resource_name: Optional[
        Literal["FdmHestonLocalVolatilityVarianceMesher"]
    ] = "FdmHestonLocalVolatilityVarianceMesher"
    size: int
    process: HESTONPROCESS
    leverageFct: LOCALVOLTERMSTRUCTURE
    maturity: float
    tAvgSteps: Optional[int] = None
    epsilon: Optional[float] = None


class FdmSimpleProcess1dMesher(BaseModel):
    resource_name: Optional[
        Literal["FdmSimpleProcess1dMesher"]
    ] = "FdmSimpleProcess1dMesher"
    size: int
    process: STOCHASTICPROCESS1D
    maturity: float
    tAvgSteps: Optional[int] = None
    epsilon: Optional[float] = None
    mandatoryPoint: Optional[Optional[float]] = None


class Uniform1dMesher(BaseModel):
    resource_name: Optional[Literal["Uniform1dMesher"]] = "Uniform1dMesher"
    start: float
    end: float
    size: int


class Predefined1dMesher(BaseModel):
    resource_name: Optional[Literal["Predefined1dMesher"]] = "Predefined1dMesher"
    x: List[float]


class Glued1dMesher(BaseModel):
    resource_name: Optional[Literal["Glued1dMesher"]] = "Glued1dMesher"
    leftMesher: FDM1DMESHER
    rightMesher: FDM1DMESHER


class FdmLinearOpIterator(BaseModel):
    resource_name: Optional[Literal["FdmLinearOpIterator"]] = "FdmLinearOpIterator"


class FdmLinearOpLayout(BaseModel):
    resource_name: Optional[Literal["FdmLinearOpLayout"]] = "FdmLinearOpLayout"


class FdmMesher(BaseModel):
    resource_name: Optional[Literal["FdmMesher"]] = "FdmMesher"


class FdmMesherComposite(BaseModel):
    resource_name: Optional[Literal["FdmMesherComposite"]] = "FdmMesherComposite"
    m1: FDM1DMESHER
    m2: FDM1DMESHER
    m3: Optional[FDM1DMESHER] = None
    m4: Optional[FDM1DMESHER] = None


class FdmLinearOp(BaseModel):
    resource_name: Optional[Literal["FdmLinearOp"]] = "FdmLinearOp"


class FdmLinearOpComposite(BaseModel):
    resource_name: Optional[Literal["FdmLinearOpComposite"]] = "FdmLinearOpComposite"


class FdmBoundaryCondition(BaseModel):
    resource_name: Optional[Literal["FdmBoundaryCondition"]] = "FdmBoundaryCondition"


class FdmTimeDepDirichletBoundary(BaseModel):
    resource_name: Optional[
        Literal["FdmTimeDepDirichletBoundary"]
    ] = "FdmTimeDepDirichletBoundary"


class FdmBlackScholesOp(BaseModel):
    resource_name: Optional[Literal["FdmBlackScholesOp"]] = "FdmBlackScholesOp"
    mesher: FDMMESHER
    process: GENERALIZEDBLACKSCHOLESPROCESS
    strike: float
    localVol: Optional[bool] = None
    illegalLocalVolOverwrite: Optional[Optional[float]] = None
    direction: Optional[int] = None
    quantoHelper: Optional[FdmQuantoHelper] = None


class Fdm2dBlackScholesOp(BaseModel):
    resource_name: Optional[Literal["Fdm2dBlackScholesOp"]] = "Fdm2dBlackScholesOp"
    mesher: FDMMESHER
    p1: GENERALIZEDBLACKSCHOLESPROCESS
    p2: GENERALIZEDBLACKSCHOLESPROCESS
    correlation: float
    maturity: float
    localVol: Optional[bool] = None
    illegalLocalVolOverwrite: Optional[Optional[float]] = None


class FdmCEVOp(BaseModel):
    resource_name: Optional[Literal["FdmCEVOp"]] = "FdmCEVOp"
    mesher: FDMMESHER
    rTS: YIELDTERMSTRUCTURE
    f0: float
    alpha: float
    beta: float
    direction: int


class FdmG2Op(BaseModel):
    resource_name: Optional[Literal["FdmG2Op"]] = "FdmG2Op"
    mesher: FDMMESHER
    model: G2
    direction1: int
    direction2: int


class FdmHestonHullWhiteOp(BaseModel):
    resource_name: Optional[Literal["FdmHestonHullWhiteOp"]] = "FdmHestonHullWhiteOp"
    mesher: FDMMESHER
    hestonProcess: HESTONPROCESS
    hwProcess: HullWhiteProcess
    equityShortRateCorrelation: float


class FdmHestonOp(BaseModel):
    resource_name: Optional[Literal["FdmHestonOp"]] = "FdmHestonOp"
    mesher: FDMMESHER
    hestonProcess: HESTONPROCESS
    quantoHelper: Optional[FdmQuantoHelper] = None
    leverageFct: Optional[LOCALVOLTERMSTRUCTURE] = None


class FdmHullWhiteOp(BaseModel):
    resource_name: Optional[Literal["FdmHullWhiteOp"]] = "FdmHullWhiteOp"
    mesher: FDMMESHER
    model: HullWhite
    direction: int


class FdmLocalVolFwdOp(BaseModel):
    resource_name: Optional[Literal["FdmLocalVolFwdOp"]] = "FdmLocalVolFwdOp"
    mesher: FDMMESHER
    spot: QUOTE
    rTS: YIELDTERMSTRUCTURE
    qTS: YIELDTERMSTRUCTURE
    localVol: LOCALVOLTERMSTRUCTURE
    direction: Optional[int] = None


class FdmOrnsteinUhlenbeckOp(BaseModel):
    resource_name: Optional[
        Literal["FdmOrnsteinUhlenbeckOp"]
    ] = "FdmOrnsteinUhlenbeckOp"
    mesher: FDMMESHER
    p: OrnsteinUhlenbeckProcess
    rTS: YIELDTERMSTRUCTURE
    direction: Optional[int] = None


class FdmSabrOp(BaseModel):
    resource_name: Optional[Literal["FdmSabrOp"]] = "FdmSabrOp"
    mesher: FDMMESHER
    rTS: YIELDTERMSTRUCTURE
    f0: float
    alpha: float
    beta: float
    nu: float
    rho: float


class FdmZabrOp(BaseModel):
    resource_name: Optional[Literal["FdmZabrOp"]] = "FdmZabrOp"
    mesher: FDMMESHER
    beta: float
    nu: float
    rho: float
    gamma: float


class FdmDupire1dOp(BaseModel):
    resource_name: Optional[Literal["FdmDupire1dOp"]] = "FdmDupire1dOp"
    mesher: FDMMESHER
    localVolatility: Array


class FdmBlackScholesFwdOp(BaseModel):
    resource_name: Optional[Literal["FdmBlackScholesFwdOp"]] = "FdmBlackScholesFwdOp"
    mesher: FDMMESHER
    process: GENERALIZEDBLACKSCHOLESPROCESS
    strike: float
    localVol: Optional[bool] = None
    illegalLocalVolOverwrite: Optional[float] = None
    direction: Optional[int] = None


class TripleBandLinearOp(BaseModel):
    resource_name: Optional[Literal["TripleBandLinearOp"]] = "TripleBandLinearOp"
    direction: int
    mesher: FDMMESHER


class FirstDerivativeOp(BaseModel):
    resource_name: Optional[Literal["FirstDerivativeOp"]] = "FirstDerivativeOp"
    direction: int
    mesher: FDMMESHER


class SecondDerivativeOp(BaseModel):
    resource_name: Optional[Literal["SecondDerivativeOp"]] = "SecondDerivativeOp"
    direction: int
    mesher: FDMMESHER


class NinePointLinearOp(BaseModel):
    resource_name: Optional[Literal["NinePointLinearOp"]] = "NinePointLinearOp"
    d0: int
    d1: int
    mesher: FDMMESHER


class SecondOrderMixedDerivativeOp(BaseModel):
    resource_name: Optional[
        Literal["SecondOrderMixedDerivativeOp"]
    ] = "SecondOrderMixedDerivativeOp"
    d0: int
    d1: int
    mesher: FDMMESHER


class NthOrderDerivativeOp(BaseModel):
    resource_name: Optional[Literal["NthOrderDerivativeOp"]] = "NthOrderDerivativeOp"
    direction: int
    order: int
    nPoints: int
    mesher: FDMMESHER


class FdmInnerValueCalculator(BaseModel):
    resource_name: Optional[
        Literal["FdmInnerValueCalculator"]
    ] = "FdmInnerValueCalculator"


class FdmCellAveragingInnerValue(BaseModel):
    resource_name: Optional[
        Literal["FdmCellAveragingInnerValue"]
    ] = "FdmCellAveragingInnerValue"


class FdmLogInnerValue(BaseModel):
    resource_name: Optional[Literal["FdmLogInnerValue"]] = "FdmLogInnerValue"
    payoff: PAYOFF
    mesher: FDMMESHER
    direction: int


class FdmLogBasketInnerValue(BaseModel):
    resource_name: Optional[
        Literal["FdmLogBasketInnerValue"]
    ] = "FdmLogBasketInnerValue"
    payoff: BASKETPAYOFF
    mesher: FDMMESHER


class FdmZeroInnerValue(BaseModel):
    resource_name: Optional[Literal["FdmZeroInnerValue"]] = "FdmZeroInnerValue"


class FdmSnapshotCondition(BaseModel):
    resource_name: Optional[Literal["FdmSnapshotCondition"]] = "FdmSnapshotCondition"
    t: float


class FdmStepConditionComposite(BaseModel):
    resource_name: Optional[
        Literal["FdmStepConditionComposite"]
    ] = "FdmStepConditionComposite"


class FdmAmericanStepCondition(BaseModel):
    resource_name: Optional[
        Literal["FdmAmericanStepCondition"]
    ] = "FdmAmericanStepCondition"
    mesher: FDMMESHER
    calculator: FDMINNERVALUECALCULATOR


class FdmArithmeticAverageCondition(BaseModel):
    resource_name: Optional[
        Literal["FdmArithmeticAverageCondition"]
    ] = "FdmArithmeticAverageCondition"
    averageTimes: List[float]
    real: float
    pastFixings: int
    mesher: FDMMESHER
    equityDirection: int


class FdmBermudanStepCondition(BaseModel):
    resource_name: Optional[
        Literal["FdmBermudanStepCondition"]
    ] = "FdmBermudanStepCondition"
    exerciseDates: List[Date]
    referenceDate: Date
    dayCounter: DAYCOUNTER
    mesher: FDMMESHER
    calculator: FDMINNERVALUECALCULATOR


class FdmSimpleStorageCondition(BaseModel):
    resource_name: Optional[
        Literal["FdmSimpleStorageCondition"]
    ] = "FdmSimpleStorageCondition"
    exerciseTimes: List[float]
    mesher: FDMMESHER
    calculator: FDMINNERVALUECALCULATOR
    changeRate: float


class FdmSimpleSwingCondition(BaseModel):
    resource_name: Optional[
        Literal["FdmSimpleSwingCondition"]
    ] = "FdmSimpleSwingCondition"
    exerciseTimes: List[float]
    mesher: FDMMESHER
    calculator: FDMINNERVALUECALCULATOR
    swingDirection: int
    minExercises: Optional[int] = None


class FdmDividendHandler(BaseModel):
    resource_name: Optional[Literal["FdmDividendHandler"]] = "FdmDividendHandler"
    schedule: List[DIVIDEND]
    mesher: FDMMESHER
    referenceDate: Date
    dayCounter: DAYCOUNTER
    equityDirection: int


class FdmSolverDesc(BaseModel):
    resource_name: Optional[Literal["FdmSolverDesc"]] = "FdmSolverDesc"


class Fdm2dBlackScholesSolver(BaseModel):
    resource_name: Optional[
        Literal["Fdm2dBlackScholesSolver"]
    ] = "Fdm2dBlackScholesSolver"


class FdmG2Solver(BaseModel):
    resource_name: Optional[Literal["FdmG2Solver"]] = "FdmG2Solver"


class FdmHestonHullWhiteSolver(BaseModel):
    resource_name: Optional[
        Literal["FdmHestonHullWhiteSolver"]
    ] = "FdmHestonHullWhiteSolver"


class FdmHestonSolver(BaseModel):
    resource_name: Optional[Literal["FdmHestonSolver"]] = "FdmHestonSolver"


class FdmHullWhiteSolver(BaseModel):
    resource_name: Optional[Literal["FdmHullWhiteSolver"]] = "FdmHullWhiteSolver"


class RiskNeutralDensityCalculator(BaseModel):
    resource_name: Optional[
        Literal["RiskNeutralDensityCalculator"]
    ] = "RiskNeutralDensityCalculator"


class BSMRNDCalculator(BaseModel):
    resource_name: Optional[Literal["BSMRNDCalculator"]] = "BSMRNDCalculator"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class CEVRNDCalculator(BaseModel):
    resource_name: Optional[Literal["CEVRNDCalculator"]] = "CEVRNDCalculator"
    f0: float
    alpha: float
    beta: float


class GBSMRNDCalculator(BaseModel):
    resource_name: Optional[Literal["GBSMRNDCalculator"]] = "GBSMRNDCalculator"
    process: GENERALIZEDBLACKSCHOLESPROCESS


class HestonRNDCalculator(BaseModel):
    resource_name: Optional[Literal["HestonRNDCalculator"]] = "HestonRNDCalculator"
    hestonProcess: HESTONPROCESS
    integrationEps: Optional[float] = None
    maxIntegrationIterations: Optional[int] = None


class LocalVolRNDCalculator(BaseModel):
    resource_name: Optional[Literal["LocalVolRNDCalculator"]] = "LocalVolRNDCalculator"
    spot: QUOTE
    rTS: YIELDTERMSTRUCTURE
    qTS: YIELDTERMSTRUCTURE
    localVol: LOCALVOLTERMSTRUCTURE
    xGrid: Optional[int] = None
    tGrid: Optional[int] = None
    x0Density: Optional[float] = None
    localVolProbEps: Optional[float] = None
    maxIter: Optional[int] = None
    gaussianStepSize: Optional[float] = None


class SquareRootProcessRNDCalculator(BaseModel):
    resource_name: Optional[
        Literal["SquareRootProcessRNDCalculator"]
    ] = "SquareRootProcessRNDCalculator"
    v0: float
    kappa: float
    theta: float
    sigma: float


class FittingMethod(BaseModel):
    resource_name: Optional[Literal["FittingMethod"]] = "FittingMethod"


class ExponentialSplinesFitting(BaseModel):
    resource_name: Optional[
        Literal["ExponentialSplinesFitting"]
    ] = "ExponentialSplinesFitting"
    constrainAtZero: Optional[bool] = None
    weights: Optional[Array] = None


class NelsonSiegelFitting(BaseModel):
    resource_name: Optional[Literal["NelsonSiegelFitting"]] = "NelsonSiegelFitting"
    weights: Optional[Array] = None


class SvenssonFitting(BaseModel):
    resource_name: Optional[Literal["SvenssonFitting"]] = "SvenssonFitting"
    weights: Optional[Array] = None


class CubicBSplinesFitting(BaseModel):
    resource_name: Optional[Literal["CubicBSplinesFitting"]] = "CubicBSplinesFitting"
    knotVector: List[float]
    constrainAtZero: Optional[bool] = None
    weights: Optional[Array] = None


class SimplePolynomialFitting(BaseModel):
    resource_name: Optional[
        Literal["SimplePolynomialFitting"]
    ] = "SimplePolynomialFitting"
    degree: float
    constrainAtZero: Optional[bool] = None
    weights: Optional[Array] = None


class Forward(BaseModel):
    resource_name: Optional[Literal["Forward"]] = "Forward"


class Position(BaseModel):
    resource_name: Optional[Literal["Position"]] = "Position"


class Gaussian1dModel(BaseModel):
    resource_name: Optional[Literal["Gaussian1dModel"]] = "Gaussian1dModel"


class Gsr(BaseModel):
    resource_name: Optional[Literal["Gsr"]] = "Gsr"
    termStructure: YieldTermStructureHandle
    volstepdates: List[Date]
    volatilities: List[QuoteHandle]
    reversions: List[QuoteHandle]
    T: Optional[float] = None


class MarkovFunctionalModelSettings(BaseModel):
    resource_name: Optional[
        Literal["MarkovFunctionalModelSettings"]
    ] = "MarkovFunctionalModelSettings"
    yGridPoints: Optional[int] = None
    yStdDevs: Optional[float] = None
    gaussHermitePoints: Optional[int] = None
    digitalGap: Optional[float] = None
    marketRateAccuracy: Optional[float] = None
    lowerRateBound: Optional[float] = None
    upperRateBound: Optional[float] = None
    adjustments: Optional[int] = None
    smileMoneyCheckpoints: Optional[List[float]] = None


class Gaussian1dJamshidianSwaptionEngine(BaseModel):
    resource_name: Optional[
        Literal["Gaussian1dJamshidianSwaptionEngine"]
    ] = "Gaussian1dJamshidianSwaptionEngine"
    model: GAUSSIAN1DMODEL


class SegmentIntegral(BaseModel):
    resource_name: Optional[Literal["SegmentIntegral"]] = "SegmentIntegral"
    intervals: int


class SimpsonIntegral(BaseModel):
    resource_name: Optional[Literal["SimpsonIntegral"]] = "SimpsonIntegral"
    accuracy: float
    maxIterations: int


class GaussKronrodAdaptive(BaseModel):
    resource_name: Optional[Literal["GaussKronrodAdaptive"]] = "GaussKronrodAdaptive"
    tolerance: float
    maxFunctionEvaluations: Optional[int] = None


class GaussKronrodNonAdaptive(BaseModel):
    resource_name: Optional[
        Literal["GaussKronrodNonAdaptive"]
    ] = "GaussKronrodNonAdaptive"
    absoluteAccuracy: float
    maxEvaluations: int
    relativeAccuracy: float


class GaussLobattoIntegral(BaseModel):
    resource_name: Optional[Literal["GaussLobattoIntegral"]] = "GaussLobattoIntegral"
    maxIterations: int
    absAccuracy: float
    relAccuracy: Optional[float] = None
    useConvergenceEstimate: Optional[bool] = None


class GaussLaguerreIntegration(BaseModel):
    resource_name: Optional[
        Literal["GaussLaguerreIntegration"]
    ] = "GaussLaguerreIntegration"
    n: int
    s: Optional[float] = None


class GaussHermiteIntegration(BaseModel):
    resource_name: Optional[
        Literal["GaussHermiteIntegration"]
    ] = "GaussHermiteIntegration"
    n: int
    mu: Optional[float] = None


class GaussJacobiIntegration(BaseModel):
    resource_name: Optional[
        Literal["GaussJacobiIntegration"]
    ] = "GaussJacobiIntegration"
    n: int
    alpha: float
    beta: float


class GaussHyperbolicIntegration(BaseModel):
    resource_name: Optional[
        Literal["GaussHyperbolicIntegration"]
    ] = "GaussHyperbolicIntegration"
    n: int


class GaussLegendreIntegration(BaseModel):
    resource_name: Optional[
        Literal["GaussLegendreIntegration"]
    ] = "GaussLegendreIntegration"
    n: int


class GaussChebyshevIntegration(BaseModel):
    resource_name: Optional[
        Literal["GaussChebyshevIntegration"]
    ] = "GaussChebyshevIntegration"
    n: int


class GaussChebyshev2ndIntegration(BaseModel):
    resource_name: Optional[
        Literal["GaussChebyshev2ndIntegration"]
    ] = "GaussChebyshev2ndIntegration"
    n: int


class GaussGegenbauerIntegration(BaseModel):
    resource_name: Optional[
        Literal["GaussGegenbauerIntegration"]
    ] = "GaussGegenbauerIntegration"
    n: int
    lambda_: float = Field(..., alias="lambda")


class LecuyerUniformRng(BaseModel):
    resource_name: Optional[Literal["LecuyerUniformRng"]] = "LecuyerUniformRng"
    seed: Optional[int] = None


class KnuthUniformRng(BaseModel):
    resource_name: Optional[Literal["KnuthUniformRng"]] = "KnuthUniformRng"
    seed: Optional[int] = None


class MersenneTwisterUniformRng(BaseModel):
    resource_name: Optional[
        Literal["MersenneTwisterUniformRng"]
    ] = "MersenneTwisterUniformRng"
    seed: Optional[int] = None


class UniformRandomGenerator(BaseModel):
    resource_name: Optional[
        Literal["UniformRandomGenerator"]
    ] = "UniformRandomGenerator"
    seed: Optional[int] = None


class GaussianRandomGenerator(BaseModel):
    resource_name: Optional[
        Literal["GaussianRandomGenerator"]
    ] = "GaussianRandomGenerator"
    rng: UniformRandomGenerator


class HaltonRsg(BaseModel):
    resource_name: Optional[Literal["HaltonRsg"]] = "HaltonRsg"
    dimensionality: int
    seed: Optional[int] = None
    randomStart: Optional[bool] = None
    randomShift: Optional[bool] = None


class SobolBrownianBridgeRsg(BaseModel):
    resource_name: Optional[
        Literal["SobolBrownianBridgeRsg"]
    ] = "SobolBrownianBridgeRsg"
    factors: int
    steps: int


class UniformRandomSequenceGenerator(BaseModel):
    resource_name: Optional[
        Literal["UniformRandomSequenceGenerator"]
    ] = "UniformRandomSequenceGenerator"
    dimensionality: int
    rng: UniformRandomGenerator


class GaussianRandomSequenceGenerator(BaseModel):
    resource_name: Optional[
        Literal["GaussianRandomSequenceGenerator"]
    ] = "GaussianRandomSequenceGenerator"
    uniformSequenceGenerator: UniformRandomSequenceGenerator


class Path(BaseModel):
    resource_name: Optional[Literal["Path"]] = "Path"


class MultiPath(BaseModel):
    resource_name: Optional[Literal["MultiPath"]] = "MultiPath"


class GaussianMultiPathGenerator(BaseModel):
    resource_name: Optional[
        Literal["GaussianMultiPathGenerator"]
    ] = "GaussianMultiPathGenerator"


class BrownianBridge(BaseModel):
    resource_name: Optional[Literal["BrownianBridge"]] = "BrownianBridge"
    steps: int


class DefaultBoundaryCondition(BaseModel):
    resource_name: Optional[
        Literal["DefaultBoundaryCondition"]
    ] = "DefaultBoundaryCondition"


class TridiagonalOperator(BaseModel):
    resource_name: Optional[Literal["TridiagonalOperator"]] = "TridiagonalOperator"
    low: Array
    mid: Array
    high: Array


class DPlus(BaseModel):
    resource_name: Optional[Literal["DPlus"]] = "DPlus"
    gridPoints: int
    h: float


class DMinus(BaseModel):
    resource_name: Optional[Literal["DMinus"]] = "DMinus"
    gridPoints: int
    h: float


class DZero(BaseModel):
    resource_name: Optional[Literal["DZero"]] = "DZero"
    gridPoints: int
    h: float


class DPlusDMinus(BaseModel):
    resource_name: Optional[Literal["DPlusDMinus"]] = "DPlusDMinus"
    gridPoints: int
    h: float


class SampledCurve(BaseModel):
    resource_name: Optional[Literal["SampledCurve"]] = "SampledCurve"
    value: Optional[Array] = None


class HestonSLVProcess(BaseModel):
    resource_name: Optional[Literal["HestonSLVProcess"]] = "HestonSLVProcess"
    hestonProcess: HESTONPROCESS
    leverageFct: LOCALVOLTERMSTRUCTURE


class BrownianGeneratorFactory(BaseModel):
    resource_name: Optional[
        Literal["BrownianGeneratorFactory"]
    ] = "BrownianGeneratorFactory"


class MTBrownianGeneratorFactory(BaseModel):
    resource_name: Optional[
        Literal["MTBrownianGeneratorFactory"]
    ] = "MTBrownianGeneratorFactory"
    seed: Optional[int] = None


class SobolBrownianGenerator(BaseModel):
    resource_name: Optional[
        Literal["SobolBrownianGenerator"]
    ] = "SobolBrownianGenerator"


class HestonSLVMCModel(BaseModel):
    resource_name: Optional[Literal["HestonSLVMCModel"]] = "HestonSLVMCModel"


class FdmHestonGreensFct(BaseModel):
    resource_name: Optional[Literal["FdmHestonGreensFct"]] = "FdmHestonGreensFct"
    Gaussian = 1
    SemiAnalytical = 2
    ZeroCorrelation = 0


class HestonSLVFokkerPlanckFdmParams(BaseModel):
    resource_name: Optional[
        Literal["HestonSLVFokkerPlanckFdmParams"]
    ] = "HestonSLVFokkerPlanckFdmParams"


class HestonSLVFDMModel(BaseModel):
    resource_name: Optional[Literal["HestonSLVFDMModel"]] = "HestonSLVFDMModel"


class Statistics(BaseModel):
    resource_name: Optional[Literal["Statistics"]] = "Statistics"


class IncrementalStatistics(BaseModel):
    resource_name: Optional[Literal["IncrementalStatistics"]] = "IncrementalStatistics"


class RiskStatistics(BaseModel):
    resource_name: Optional[Literal["RiskStatistics"]] = "RiskStatistics"


class CapFloorTermVolatilityStructure(BaseModel):
    resource_name: Optional[
        Literal["CapFloorTermVolatilityStructure"]
    ] = "CapFloorTermVolatilityStructure"


class CapFloorTermVolatilityStructureHandle(BaseModel):
    resource_name: Optional[
        Literal["CapFloorTermVolatilityStructureHandle"]
    ] = "CapFloorTermVolatilityStructureHandle"
    value: Optional[CAPFLOORTERMVOLATILITYSTRUCTURE] = None


class CapFloorTermVolCurve(BaseModel):
    resource_name: Optional[Literal["CapFloorTermVolCurve"]] = "CapFloorTermVolCurve"
    settlementDays: float
    calendar: CALENDAR
    bdc: enums.BusinessDayConvention
    lengths: List[Period]
    vols: List[float]
    dc: Optional[DAYCOUNTER] = None


class CapFloorTermVolSurface(BaseModel):
    resource_name: Optional[
        Literal["CapFloorTermVolSurface"]
    ] = "CapFloorTermVolSurface"
    settlementDays: float
    calendar: CALENDAR
    bdc: enums.BusinessDayConvention
    optionTenors: List[Period]
    strikes: List[float]
    volatilities: Matrix
    dc: Optional[DAYCOUNTER] = None


class StrippedOptionletBase(BaseModel):
    resource_name: Optional[Literal["StrippedOptionletBase"]] = "StrippedOptionletBase"


class StrippedOptionletAdapter(BaseModel):
    resource_name: Optional[
        Literal["StrippedOptionletAdapter"]
    ] = "StrippedOptionletAdapter"
    value: STRIPPEDOPTIONLETBASE


class Settlement(BaseModel):
    resource_name: Optional[Literal["Settlement"]] = "Settlement"


class BlackSwaptionEngine(BaseModel):
    resource_name: Optional[Literal["BlackSwaptionEngine"]] = "BlackSwaptionEngine"
    discountCurve: YieldTermStructureHandle
    vol: QuoteHandle
    dc: Optional[DAYCOUNTER] = None
    displacement: Optional[float] = None


class BachelierSwaptionEngine(BaseModel):
    resource_name: Optional[
        Literal["BachelierSwaptionEngine"]
    ] = "BachelierSwaptionEngine"
    discountCurve: YieldTermStructureHandle
    vol: QuoteHandle
    dc: Optional[DAYCOUNTER] = None


class ConstantEstimator(BaseModel):
    resource_name: Optional[Literal["ConstantEstimator"]] = "ConstantEstimator"
    size: int


class ParkinsonSigma(BaseModel):
    resource_name: Optional[Literal["ParkinsonSigma"]] = "ParkinsonSigma"
    yearFraction: float


class GarmanKlassSigma1(BaseModel):
    resource_name: Optional[Literal["GarmanKlassSigma1"]] = "GarmanKlassSigma1"
    yearFraction: float
    marketOpenFraction: float


class GarmanKlassSigma3(BaseModel):
    resource_name: Optional[Literal["GarmanKlassSigma3"]] = "GarmanKlassSigma3"
    yearFraction: float
    marketOpenFraction: float


class GarmanKlassSigma4(BaseModel):
    resource_name: Optional[Literal["GarmanKlassSigma4"]] = "GarmanKlassSigma4"
    yearFraction: float


class GarmanKlassSigma5(BaseModel):
    resource_name: Optional[Literal["GarmanKlassSigma5"]] = "GarmanKlassSigma5"
    yearFraction: float


class GarmanKlassSigma6(BaseModel):
    resource_name: Optional[Literal["GarmanKlassSigma6"]] = "GarmanKlassSigma6"
    yearFraction: float
    marketOpenFraction: float


class ZeroCurve(BaseModel):
    resource_name: Optional[Literal["ZeroCurve"]] = "ZeroCurve"
    dates: List[Date]
    yields: List[float]
    dayCounter: DAYCOUNTER
    calendar: Optional[CALENDAR] = None
    i: Optional[Linear] = None
    compounding: Optional[enums.Compounding] = None
    frequency: Optional[float] = None


class LogLinearInterpolatedZeroCurve(BaseModel):
    resource_name: Optional[
        Literal["LogLinearInterpolatedZeroCurve"]
    ] = "LogLinearInterpolatedZeroCurve"
    dates: List[Date]
    yields: List[float]
    dayCounter: DAYCOUNTER
    calendar: Optional[CALENDAR] = None
    i: Optional[LogLinear] = None
    compounding: Optional[enums.Compounding] = None
    frequency: Optional[float] = None


class CubicInterpolatedZeroCurve(BaseModel):
    resource_name: Optional[
        Literal["CubicInterpolatedZeroCurve"]
    ] = "CubicInterpolatedZeroCurve"
    dates: List[Date]
    yields: List[float]
    dayCounter: DAYCOUNTER
    calendar: Optional[CALENDAR] = None
    i: Optional[Cubic] = None
    compounding: Optional[enums.Compounding] = None
    frequency: Optional[float] = None


class SplineCubicInterpolatedZeroCurve(BaseModel):
    resource_name: Optional[
        Literal["SplineCubicInterpolatedZeroCurve"]
    ] = "SplineCubicInterpolatedZeroCurve"
    dates: List[Date]
    yields: List[float]
    dayCounter: DAYCOUNTER
    calendar: Optional[CALENDAR] = None
    i: Optional[SplineCubic] = None
    compounding: Optional[enums.Compounding] = None
    frequency: Optional[float] = None


class DefaultLogCubicInterpolatedZeroCurve(BaseModel):
    resource_name: Optional[
        Literal["DefaultLogCubicInterpolatedZeroCurve"]
    ] = "DefaultLogCubicInterpolatedZeroCurve"
    dates: List[Date]
    yields: List[float]
    dayCounter: DAYCOUNTER
    calendar: Optional[CALENDAR] = None
    i: Optional[DefaultLogCubic] = None
    compounding: Optional[enums.Compounding] = None
    frequency: Optional[float] = None


class MonotonicCubicInterpolatedZeroCurve(BaseModel):
    resource_name: Optional[
        Literal["MonotonicCubicInterpolatedZeroCurve"]
    ] = "MonotonicCubicInterpolatedZeroCurve"
    dates: List[Date]
    yields: List[float]
    dayCounter: DAYCOUNTER
    calendar: Optional[CALENDAR] = None
    i: Optional[MonotonicCubic] = None
    compounding: Optional[enums.Compounding] = None
    frequency: Optional[float] = None


class Gaussian1dNonstandardSwaptionEngineProbabilities(BaseModel):
    resource_name: Optional[
        Literal["Gaussian1dNonstandardSwaptionEngineProbabilities"]
    ] = "Gaussian1dNonstandardSwaptionEngineProbabilities"
    Digital = 2
    Naive = 1
    NoProb = 0


class SettlementMethod(BaseModel):
    __root__: Any


class IsdaCdsEngineForwardsInCouponPeriod(BaseModel):
    __root__: Any


class GaussianQuadrature(BaseModel):
    __root__: Any


class FdmBoundaryConditionSet(BaseModel):
    resource_name: Optional[
        Literal["FdmBoundaryConditionSet"]
    ] = "FdmBoundaryConditionSet"
    Lower = 2
    NoSide = 0
    Upper = 1


class FdmDiscountDirichletBoundarySide(BaseModel):
    __root__: Any


class DefaultBoundaryConditionSide(BaseModel):
    resource_name: Optional[
        Literal["DefaultBoundaryConditionSide"]
    ] = "DefaultBoundaryConditionSide"
    Lower = 2
    NoSide = 0
    Upper = 1


class FdmDirichletBoundarySide(BaseModel):
    __root__: Any


class AndreasenHugeVolatilityInterplCalibrationType(BaseModel):
    __root__: Any


class DiscountFactor(BaseModel):
    __root__: Any


class IsdaCdsEngineAccrualBias(BaseModel):
    __root__: Any


class AndreasenHugeVolatilityInterplCalibrationSet(BaseModel):
    __root__: Any


class Leg(BaseModel):
    __root__: Any


class VolatilityType(BaseModel):
    __root__: Any


class AnalyticPTDHestonEngineIntegration(BaseModel):
    __root__: Any


class Exercise(BaseModel):
    resource_name: Optional[Literal["Exercise"]] = "Exercise"
    type: enums.ExerciseType


class RebatedExercise(BaseModel):
    resource_name: Optional[Literal["RebatedExercise"]] = "RebatedExercise"
    exercise: EXERCISE
    rebates: List[float]
    rebateSettlementDays: Optional[float] = None
    rebatePaymentCalendar: Optional[CALENDAR] = None
    rebatePaymentConvention: Optional[enums.BusinessDayConvention] = None


class JointCalendar(BaseModel):
    resource_name: Optional[Literal["JointCalendar"]] = "JointCalendar"
    arg_0: CALENDAR
    arg_1: CALENDAR
    arg_2: CALENDAR
    arg_3: CALENDAR
    rule: Optional[enums.JointCalendarRule] = None


class Actual365Fixed(BaseModel):
    resource_name: Optional[Literal["Actual365Fixed"]] = "Actual365Fixed"
    c: Optional[enums.Actual365FixedConvention] = None


class Thirty360(BaseModel):
    resource_name: Optional[Literal["Thirty360"]] = "Thirty360"
    c: Optional[enums.Thirty360Convention] = None


class ActualActual(BaseModel):
    resource_name: Optional[Literal["ActualActual"]] = "ActualActual"
    c: Optional[enums.ActualActualConvention] = None
    schedule: Optional[Schedule] = None


class GaussianSimulatedAnnealing(BaseModel):
    resource_name: Optional[
        Literal["GaussianSimulatedAnnealing"]
    ] = "GaussianSimulatedAnnealing"
    sampler: SamplerGaussian
    probability: ProbabilityBoltzmannDownhill
    temperature: TemperatureExponential
    reannealing: Optional[ReannealingTrivial] = None
    startTemperature: Optional[float] = None
    endTemperature: Optional[float] = None
    reAnnealSteps: Optional[int] = None
    resetScheme: Optional[enums.GaussianSimulatedAnnealingResetScheme] = None
    resetSteps: Optional[int] = None


class MirrorGaussianSimulatedAnnealing(BaseModel):
    resource_name: Optional[
        Literal["MirrorGaussianSimulatedAnnealing"]
    ] = "MirrorGaussianSimulatedAnnealing"
    sampler: SamplerMirrorGaussian
    probability: ProbabilityBoltzmannDownhill
    temperature: TemperatureExponential
    reannealing: Optional[ReannealingTrivial] = None
    startTemperature: Optional[float] = None
    endTemperature: Optional[float] = None
    reAnnealSteps: Optional[int] = None
    resetScheme: Optional[enums.MirrorGaussianSimulatedAnnealingResetScheme] = None
    resetSteps: Optional[int] = None


class LogNormalSimulatedAnnealing(BaseModel):
    resource_name: Optional[
        Literal["LogNormalSimulatedAnnealing"]
    ] = "LogNormalSimulatedAnnealing"
    sampler: SamplerLogNormal
    probability: ProbabilityBoltzmannDownhill
    temperature: TemperatureExponential
    reannealing: Optional[ReannealingTrivial] = None
    startTemperature: Optional[float] = None
    endTemperature: Optional[float] = None
    reAnnealSteps: Optional[int] = None
    resetScheme: Optional[enums.LogNormalSimulatedAnnealingResetScheme] = None
    resetSteps: Optional[int] = None


class ConstantOptionletVolatility(BaseModel):
    resource_name: Optional[
        Literal["ConstantOptionletVolatility"]
    ] = "ConstantOptionletVolatility"
    settlementDays: float
    cal: CALENDAR
    bdc: enums.BusinessDayConvention
    volatility: QuoteHandle
    dayCounter: DAYCOUNTER
    type: Optional[VolatilityType] = None
    shift: Optional[float] = None


class ConstantSwaptionVolatility(BaseModel):
    resource_name: Optional[
        Literal["ConstantSwaptionVolatility"]
    ] = "ConstantSwaptionVolatility"
    referenceDate: Date
    cal: CALENDAR
    bdc: enums.BusinessDayConvention
    volatility: float
    dc: DAYCOUNTER
    type: Optional[VolatilityType] = None
    shift: Optional[float] = None


class SwaptionVolatilityMatrix(BaseModel):
    resource_name: Optional[
        Literal["SwaptionVolatilityMatrix"]
    ] = "SwaptionVolatilityMatrix"
    calendar: CALENDAR
    bdc: enums.BusinessDayConvention
    optionTenors: List[Period]
    swapTenors: List[Period]
    vols: Matrix
    dayCounter: DAYCOUNTER
    flatExtrapolation: Optional[bool] = None
    type: Optional[VolatilityType] = None
    shifts: Optional[Matrix] = None


class FlatSmileSection(BaseModel):
    resource_name: Optional[Literal["FlatSmileSection"]] = "FlatSmileSection"
    exerciseTime: float
    vol: float
    dc: DAYCOUNTER
    atmLevel: Optional[float] = None
    type: Optional[VolatilityType] = None
    shift: Optional[float] = None


class AndreasenHugeVolatilityInterpl(BaseModel):
    resource_name: Optional[
        Literal["AndreasenHugeVolatilityInterpl"]
    ] = "AndreasenHugeVolatilityInterpl"
    calibrationSet: AndreasenHugeVolatilityInterplCalibrationSet
    spot: QuoteHandle
    rTS: YieldTermStructureHandle
    qTS: YieldTermStructureHandle
    interpolationType: Optional[
        enums.AndreasenHugeVolatilityInterplInterpolationType
    ] = None
    calibrationType: Optional[AndreasenHugeVolatilityInterplCalibrationType] = None
    nGridPoints: Optional[int] = None
    minStrike: Optional[float] = None
    maxStrike: Optional[float] = None
    optimizationMethod: Optional[OPTIMIZATIONMETHOD] = None
    endCriteria: Optional[EndCriteria] = None


class AndreasenHugeVolatilityAdapter(BaseModel):
    resource_name: Optional[
        Literal["AndreasenHugeVolatilityAdapter"]
    ] = "AndreasenHugeVolatilityAdapter"
    volInterpl: AndreasenHugeVolatilityInterpl
    eps: Optional[float] = None


class AndreasenHugeLocalVolAdapter(BaseModel):
    resource_name: Optional[
        Literal["AndreasenHugeLocalVolAdapter"]
    ] = "AndreasenHugeLocalVolAdapter"
    localVol: AndreasenHugeVolatilityInterpl


class CmsMarket(BaseModel):
    resource_name: Optional[Literal["CmsMarket"]] = "CmsMarket"
    swapLengths: List[Period]
    swapIndexes: List[SWAPINDEX]
    iborIndex: IBORINDEX
    bidAskSpreads: List[List[QuoteHandle]]
    pricers: List[CMSCOUPONPRICER]
    discountingTS: YieldTermStructureHandle


class CmsMarketCalibration(BaseModel):
    resource_name: Optional[Literal["CmsMarketCalibration"]] = "CmsMarketCalibration"
    volCube: SwaptionVolatilityStructureHandle
    cmsMarket: CmsMarket
    weights: Matrix
    calibrationType: enums.CmsMarketCalibrationCalibrationType


class ExtOUWithJumpsProcess(BaseModel):
    resource_name: Optional[Literal["ExtOUWithJumpsProcess"]] = "ExtOUWithJumpsProcess"
    process: ExtendedOrnsteinUhlenbeckProcess
    Y0: float
    beta: float
    jumpIntensity: float
    eta: float


class KlugeExtOUProcess(BaseModel):
    resource_name: Optional[Literal["KlugeExtOUProcess"]] = "KlugeExtOUProcess"
    rho: float
    kluge: ExtOUWithJumpsProcess
    extOU: ExtendedOrnsteinUhlenbeckProcess


class GJRGARCHProcess(BaseModel):
    resource_name: Optional[Literal["GJRGARCHProcess"]] = "GJRGARCHProcess"
    riskFreeRate: YieldTermStructureHandle
    dividendYield: YieldTermStructureHandle
    s0: QuoteHandle
    v0: float
    omega: float
    alpha: float
    beta: float
    gamma: float
    lambda_: float = Field(..., alias="lambda")
    daysPerYear: Optional[float] = None
    d: Optional[enums.GJRGARCHProcessDiscretization] = None


class AnalyticHaganPricer(BaseModel):
    resource_name: Optional[Literal["AnalyticHaganPricer"]] = "AnalyticHaganPricer"
    v: SwaptionVolatilityStructureHandle
    model: enums.GFunctionFactoryYieldCurveModel
    meanReversion: QuoteHandle


class NumericHaganPricer(BaseModel):
    resource_name: Optional[Literal["NumericHaganPricer"]] = "NumericHaganPricer"
    v: SwaptionVolatilityStructureHandle
    model: enums.GFunctionFactoryYieldCurveModel
    meanReversion: QuoteHandle
    lowerLimit: Optional[float] = None
    upperLimit: Optional[float] = None
    precision: Optional[float] = None


class LinearTsrPricer(BaseModel):
    resource_name: Optional[Literal["LinearTsrPricer"]] = "LinearTsrPricer"
    swaptionVol: SwaptionVolatilityStructureHandle
    meanReversion: QuoteHandle
    couponDiscountCurve: Optional[YieldTermStructureHandle] = None
    settings: Optional[LinearTsrPricerSettings] = None


class LognormalCmsSpreadPricer(BaseModel):
    resource_name: Optional[
        Literal["LognormalCmsSpreadPricer"]
    ] = "LognormalCmsSpreadPricer"
    cmsPricer: CMSCOUPONPRICER
    correlation: QuoteHandle
    couponDiscountCurve: Optional[YieldTermStructureHandle] = None
    IntegrationPoints: Optional[int] = None
    volatilityType: Optional[VolatilityType] = None
    shift1: Optional[float] = None
    shift2: Optional[float] = None


class SwaptionHelper(BaseModel):
    resource_name: Optional[Literal["SwaptionHelper"]] = "SwaptionHelper"
    exerciseDate: Date
    endDate: Date
    volatility: QuoteHandle
    index: IBORINDEX
    fixedLegTenor: Period
    fixedLegDayCounter: DAYCOUNTER
    floatingLegDayCounter: DAYCOUNTER
    termStructure: YieldTermStructureHandle
    errorType: Optional[enums.BlackCalibrationHelperCalibrationErrorType] = None
    strike: Optional[float] = None
    nominal: Optional[float] = None
    type: Optional[VolatilityType] = None
    shift: Optional[float] = None


class CapHelper(BaseModel):
    resource_name: Optional[Literal["CapHelper"]] = "CapHelper"
    length: Period
    volatility: QuoteHandle
    index: IBORINDEX
    fixedLegFrequency: float
    fixedLegDayCounter: DAYCOUNTER
    includeFirstSwaplet: bool
    termStructure: YieldTermStructureHandle
    errorType: Optional[enums.BlackCalibrationHelperCalibrationErrorType] = None
    type: Optional[VolatilityType] = None
    shift: Optional[float] = None


class HestonModelHelper(BaseModel):
    resource_name: Optional[Literal["HestonModelHelper"]] = "HestonModelHelper"
    maturity: Period
    calendar: CALENDAR
    s0: float
    strikePrice: float
    volatility: QuoteHandle
    riskFreeRate: YieldTermStructureHandle
    dividendYield: YieldTermStructureHandle
    errorType: Optional[enums.BlackCalibrationHelperCalibrationErrorType] = None


class VanillaOption(BaseModel):
    resource_name: Optional[Literal["VanillaOption"]] = "VanillaOption"
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE


class EuropeanOption(BaseModel):
    resource_name: Optional[Literal["EuropeanOption"]] = "EuropeanOption"
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE


class ForwardVanillaOption(BaseModel):
    resource_name: Optional[Literal["ForwardVanillaOption"]] = "ForwardVanillaOption"
    moneyness: float
    resetDate: Date
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE


class QuantoVanillaOption(BaseModel):
    resource_name: Optional[Literal["QuantoVanillaOption"]] = "QuantoVanillaOption"
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE


class QuantoForwardVanillaOption(BaseModel):
    resource_name: Optional[
        Literal["QuantoForwardVanillaOption"]
    ] = "QuantoForwardVanillaOption"
    moneyness: float
    resetDate: Date
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE


class AnalyticHestonEngineIntegration(BaseModel):
    resource_name: Optional[
        Literal["AnalyticHestonEngineIntegration"]
    ] = "AnalyticHestonEngineIntegration"
    intAlgo: enums.AnalyticHestonEngineIntegrationAlgorithm
    quadrature: GaussianQuadrature


class AnalyticPTDHestonEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticPTDHestonEngine"]
    ] = "AnalyticPTDHestonEngine"
    model: PiecewiseTimeDependentHestonModel
    cpxLog: enums.AnalyticPTDHestonEngineComplexLogFormula
    itg: AnalyticPTDHestonEngineIntegration
    andersenPiterbargEpsilon: Optional[float] = None


class DividendVanillaOption(BaseModel):
    resource_name: Optional[Literal["DividendVanillaOption"]] = "DividendVanillaOption"
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE
    dividendDates: List[Date]
    dividends: List[float]


class BarrierOption(BaseModel):
    resource_name: Optional[Literal["BarrierOption"]] = "BarrierOption"
    barrierType: enums.BarrierType
    barrier: float
    rebate: float
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE


class FdmSchemeDesc(BaseModel):
    resource_name: Optional[Literal["FdmSchemeDesc"]] = "FdmSchemeDesc"
    type: enums.FdmSchemeDescFdmSchemeType
    theta: float
    mu: float


class FdBlackScholesVanillaEngine(BaseModel):
    resource_name: Optional[
        Literal["FdBlackScholesVanillaEngine"]
    ] = "FdBlackScholesVanillaEngine"
    value: GENERALIZEDBLACKSCHOLESPROCESS
    quantoHelper: FdmQuantoHelper
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    schemeDesc: Optional[FdmSchemeDesc] = None
    localVol: Optional[bool] = None
    illegalLocalVolOverwrite: Optional[float] = None
    cashDividendModel: Optional[
        enums.FdBlackScholesVanillaEngineCashDividendModel
    ] = None


class FdOrnsteinUhlenbeckVanillaEngine(BaseModel):
    resource_name: Optional[
        Literal["FdOrnsteinUhlenbeckVanillaEngine"]
    ] = "FdOrnsteinUhlenbeckVanillaEngine"
    value: OrnsteinUhlenbeckProcess
    rTS: YIELDTERMSTRUCTURE
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    epsilon: Optional[float] = None
    schemeDesc: Optional[FdmSchemeDesc] = None


class FdBatesVanillaEngine(BaseModel):
    resource_name: Optional[Literal["FdBatesVanillaEngine"]] = "FdBatesVanillaEngine"
    model: BatesModel
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    vGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    schemeDesc: Optional[FdmSchemeDesc] = None


class FdHestonVanillaEngine(BaseModel):
    resource_name: Optional[Literal["FdHestonVanillaEngine"]] = "FdHestonVanillaEngine"
    model: HESTONMODEL
    quantoHelper: FdmQuantoHelper
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    vGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    schemeDesc: Optional[FdmSchemeDesc] = None
    leverageFct: Optional[LOCALVOLTERMSTRUCTURE] = None


class FdCEVVanillaEngine(BaseModel):
    resource_name: Optional[Literal["FdCEVVanillaEngine"]] = "FdCEVVanillaEngine"
    f0: float
    alpha: float
    beta: float
    rTS: YieldTermStructureHandle
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    scalingFactor: Optional[float] = None
    eps: Optional[float] = None
    schemeDesc: Optional[FdmSchemeDesc] = None


class FdSabrVanillaEngine(BaseModel):
    resource_name: Optional[Literal["FdSabrVanillaEngine"]] = "FdSabrVanillaEngine"
    f0: float
    alpha: float
    beta: float
    nu: float
    rho: float
    rTS: YieldTermStructureHandle
    tGrid: Optional[int] = None
    fGrid: Optional[int] = None
    xGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    scalingFactor: Optional[float] = None
    eps: Optional[float] = None
    schemeDesc: Optional[FdmSchemeDesc] = None


class FdBlackScholesBarrierEngine(BaseModel):
    resource_name: Optional[
        Literal["FdBlackScholesBarrierEngine"]
    ] = "FdBlackScholesBarrierEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    schemeDesc: Optional[FdmSchemeDesc] = None
    localVol: Optional[bool] = None
    illegalLocalVolOverwrite: Optional[float] = None


class ContinuousAveragingAsianOption(BaseModel):
    resource_name: Optional[
        Literal["ContinuousAveragingAsianOption"]
    ] = "ContinuousAveragingAsianOption"
    averageType: enums.AverageType
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE


class DiscreteAveragingAsianOption(BaseModel):
    resource_name: Optional[
        Literal["DiscreteAveragingAsianOption"]
    ] = "DiscreteAveragingAsianOption"
    averageType: enums.AverageType
    runningAccumulator: float
    pastFixings: int
    fixingDates: List[Date]
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE


class DoubleBarrierOption(BaseModel):
    resource_name: Optional[Literal["DoubleBarrierOption"]] = "DoubleBarrierOption"
    barrierType: enums.DoubleBarrierType
    barrier_lo: float
    barrier_hi: float
    rebate: float
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE


class QuantoDoubleBarrierOption(BaseModel):
    resource_name: Optional[
        Literal["QuantoDoubleBarrierOption"]
    ] = "QuantoDoubleBarrierOption"
    barrierType: enums.DoubleBarrierType
    barrier_lo: float
    barrier_hi: float
    rebate: float
    payoff: STRIKEDTYPEPAYOFF
    exercise: EXERCISE


class DeltaVolQuote(BaseModel):
    resource_name: Optional[Literal["DeltaVolQuote"]] = "DeltaVolQuote"
    delta: float
    vol: QuoteHandle
    maturity: float
    deltaType: enums.DeltaVolQuoteDeltaType


class DeltaVolQuoteHandle(BaseModel):
    resource_name: Optional[Literal["DeltaVolQuoteHandle"]] = "DeltaVolQuoteHandle"
    value: Optional[DeltaVolQuote] = None


class VannaVolgaBarrierEngine(BaseModel):
    resource_name: Optional[
        Literal["VannaVolgaBarrierEngine"]
    ] = "VannaVolgaBarrierEngine"
    atmVol: DeltaVolQuoteHandle
    vol25Put: DeltaVolQuoteHandle
    vol25Call: DeltaVolQuoteHandle
    spotFX: QuoteHandle
    domesticTS: YieldTermStructureHandle
    foreignTS: YieldTermStructureHandle
    adaptVanDelta: Optional[bool] = None
    bsPriceWithSmile: Optional[float] = None


class FdSimpleBSSwingEngine(BaseModel):
    resource_name: Optional[Literal["FdSimpleBSSwingEngine"]] = "FdSimpleBSSwingEngine"
    process: GENERALIZEDBLACKSCHOLESPROCESS
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    schemeDesc: Optional[FdmSchemeDesc] = None


class GJRGARCHModel(BaseModel):
    resource_name: Optional[Literal["GJRGARCHModel"]] = "GJRGARCHModel"
    process: GJRGARCHProcess


class AnalyticGJRGARCHEngine(BaseModel):
    resource_name: Optional[
        Literal["AnalyticGJRGARCHEngine"]
    ] = "AnalyticGJRGARCHEngine"
    process: GJRGARCHModel


class BasketOption(BaseModel):
    resource_name: Optional[Literal["BasketOption"]] = "BasketOption"
    payoff: BASKETPAYOFF
    exercise: EXERCISE


class Fd2dBlackScholesVanillaEngine(BaseModel):
    resource_name: Optional[
        Literal["Fd2dBlackScholesVanillaEngine"]
    ] = "Fd2dBlackScholesVanillaEngine"
    p1: GENERALIZEDBLACKSCHOLESPROCESS
    p2: GENERALIZEDBLACKSCHOLESPROCESS
    correlation: float
    xGrid: Optional[int] = None
    yGrid: Optional[int] = None
    tGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    schemeDesc: Optional[FdmSchemeDesc] = None
    localVol: Optional[bool] = None
    illegalLocalVolOverwrite: Optional[float] = None


class EverestOption(BaseModel):
    resource_name: Optional[Literal["EverestOption"]] = "EverestOption"
    notional: float
    guarantee: float
    exercise: EXERCISE


class BlackDeltaCalculator(BaseModel):
    resource_name: Optional[Literal["BlackDeltaCalculator"]] = "BlackDeltaCalculator"
    ot: enums.OptionType
    dt: enums.DeltaVolQuoteDeltaType
    spot: float
    dDiscount: DiscountFactor
    fDiscount: DiscountFactor
    stDev: float


class OvernightIndexedSwap(BaseModel):
    resource_name: Optional[Literal["OvernightIndexedSwap"]] = "OvernightIndexedSwap"
    type: enums.OvernightIndexedSwapType
    nominals: List[float]
    schedule: Schedule
    fixedRate: float
    fixedDC: DAYCOUNTER
    index: OVERNIGHTINDEX
    spread: Optional[float] = None
    paymentLag: Optional[float] = None
    paymentAdjustment: Optional[enums.BusinessDayConvention] = None
    paymentCalendar: Optional[CALENDAR] = None
    telescopicValueDates: Optional[bool] = None


class ZeroCouponInflationSwap(BaseModel):
    resource_name: Optional[
        Literal["ZeroCouponInflationSwap"]
    ] = "ZeroCouponInflationSwap"
    type: enums.ZeroCouponInflationSwapType
    nominal: float
    start: Date
    maturity: Date
    calendar: CALENDAR
    convention: enums.BusinessDayConvention
    dayCounter: DAYCOUNTER
    fixedRate: float
    index: ZEROINFLATIONINDEX
    lag: Period
    adjustInfObsDates: Optional[bool] = None
    infCalendar: Optional[CALENDAR] = None
    infConvention: Optional[enums.BusinessDayConvention] = None


class YearOnYearInflationSwap(BaseModel):
    resource_name: Optional[
        Literal["YearOnYearInflationSwap"]
    ] = "YearOnYearInflationSwap"
    type: enums.YearOnYearInflationSwapType
    nominal: float
    fixedSchedule: Schedule
    fixedRate: float
    fixedDayCounter: DAYCOUNTER
    yoySchedule: Schedule
    index: YOYINFLATIONINDEX
    lag: Period
    spread: float
    yoyDayCounter: DAYCOUNTER
    paymentCalendar: CALENDAR
    paymentConvention: Optional[enums.BusinessDayConvention] = None


class CPISwap(BaseModel):
    resource_name: Optional[Literal["CPISwap"]] = "CPISwap"
    type: enums.CPISwapType
    nominal: float
    subtractInflationNominal: bool
    spread: float
    floatDayCount: DAYCOUNTER
    floatSchedule: Schedule
    floatRoll: enums.BusinessDayConvention
    fixingDays: float
    floatIndex: IBORINDEX
    fixedRate: float
    baseCPI: float
    fixedDayCount: DAYCOUNTER
    fixedSchedule: Schedule
    fixedRoll: enums.BusinessDayConvention
    observationLag: Period
    fixedIndex: ZEROINFLATIONINDEX
    observationInterpolation: Optional[enums.CPIInterpolationType] = None
    inflationNominal: Optional[float] = None


class FdG2SwaptionEngine(BaseModel):
    resource_name: Optional[Literal["FdG2SwaptionEngine"]] = "FdG2SwaptionEngine"
    model: G2
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    yGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    invEps: Optional[float] = None
    schemeDesc: Optional[FdmSchemeDesc] = None


class FdHullWhiteSwaptionEngine(BaseModel):
    resource_name: Optional[
        Literal["FdHullWhiteSwaptionEngine"]
    ] = "FdHullWhiteSwaptionEngine"
    model: HullWhite
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    invEps: Optional[float] = None
    schemeDesc: Optional[FdmSchemeDesc] = None


class Bond(BaseModel):
    resource_name: Optional[Literal["Bond"]] = "Bond"
    settlementDays: float
    calendar: CALENDAR
    issueDate: Optional[Date] = None
    coupons: Optional[Leg] = None


class CPIBond(BaseModel):
    resource_name: Optional[Literal["CPIBond"]] = "CPIBond"
    settlementDays: float
    faceAmount: float
    growthOnly: bool
    baseCPI: float
    observationLag: Period
    cpiIndex: ZEROINFLATIONINDEX
    observationInterpolation: enums.CPIInterpolationType
    schedule: Schedule
    coupons: List[float]
    accrualDayCounter: DAYCOUNTER
    paymentConvention: Optional[enums.BusinessDayConvention] = None
    issueDate: Optional[Date] = None
    paymentCalendar: Optional[CALENDAR] = None
    exCouponPeriod: Optional[Period] = None
    exCouponCalendar: Optional[CALENDAR] = None
    exCouponConvention: Optional[enums.BusinessDayConvention] = None
    exCouponEndOfMonth: Optional[bool] = None


class ConvertibleZeroCouponBond(BaseModel):
    resource_name: Optional[
        Literal["ConvertibleZeroCouponBond"]
    ] = "ConvertibleZeroCouponBond"
    exercise: EXERCISE
    conversionRatio: float
    dividends: List[DIVIDEND]
    callability: List[CALLABILITY]
    creditSpread: QuoteHandle
    issueDate: Date
    settlementDays: int
    dayCounter: DAYCOUNTER
    schedule: Schedule
    redemption: Optional[float] = None


class ConvertibleFixedCouponBond(BaseModel):
    resource_name: Optional[
        Literal["ConvertibleFixedCouponBond"]
    ] = "ConvertibleFixedCouponBond"
    exercise: EXERCISE
    conversionRatio: float
    dividends: List[DIVIDEND]
    callability: List[CALLABILITY]
    creditSpread: QuoteHandle
    issueDate: Date
    settlementDays: int
    coupons: List[float]
    dayCounter: DAYCOUNTER
    schedule: Schedule
    redemption: Optional[float] = None


class ConvertibleFloatingRateBond(BaseModel):
    resource_name: Optional[
        Literal["ConvertibleFloatingRateBond"]
    ] = "ConvertibleFloatingRateBond"
    exercise: EXERCISE
    conversionRatio: float
    dividends: List[DIVIDEND]
    callability: List[CALLABILITY]
    creditSpread: QuoteHandle
    issueDate: Date
    settlementDays: int
    index: IBORINDEX
    fixingDays: int
    spreads: List[float]
    dayCounter: DAYCOUNTER
    schedule: Schedule
    redemption: Optional[float] = None


class FraRateHelper(BaseModel):
    resource_name: Optional[Literal["FraRateHelper"]] = "FraRateHelper"
    rate: float
    immOffsetStart: float
    immOffsetEnd: float
    iborIndex: IBORINDEX
    pillar: Optional[enums.PillarChoice] = None
    customPillarDate: Optional[Date] = None
    useIndexedCoupon: Optional[bool] = None


class FuturesRateHelper(BaseModel):
    resource_name: Optional[Literal["FuturesRateHelper"]] = "FuturesRateHelper"
    price: float
    iborStartDate: Date
    index: IBORINDEX
    convexityAdjustment: Optional[float] = None
    type: Optional[enums.FuturesType] = None


class SwapRateHelper(BaseModel):
    resource_name: Optional[Literal["SwapRateHelper"]] = "SwapRateHelper"
    rate: float
    index: SWAPINDEX
    spread: Optional[QuoteHandle] = None
    fwdStart: Optional[Period] = None
    discountingCurve: Optional[YieldTermStructureHandle] = None
    pillar: Optional[enums.PillarChoice] = None
    customPillarDate: Optional[Date] = None


class BondHelper(BaseModel):
    resource_name: Optional[Literal["BondHelper"]] = "BondHelper"
    cleanPrice: QuoteHandle
    bond: BOND
    useCleanPrice: Optional[bool] = None


class OISRateHelper(BaseModel):
    resource_name: Optional[Literal["OISRateHelper"]] = "OISRateHelper"
    settlementDays: float
    tenor: Period
    rate: QuoteHandle
    index: OVERNIGHTINDEX
    discountingCurve: Optional[YieldTermStructureHandle] = None
    telescopicValueDates: Optional[bool] = None
    paymentLag: Optional[float] = None
    paymentConvention: Optional[enums.BusinessDayConvention] = None
    paymentFrequency: Optional[float] = None
    paymentCalendar: Optional[CALENDAR] = None
    forwardStart: Optional[Period] = None
    overnightSpread: Optional[float] = None
    pillar: Optional[enums.PillarChoice] = None
    customPillarDate: Optional[Date] = None


class OvernightIndexFutureRateHelper(BaseModel):
    resource_name: Optional[
        Literal["OvernightIndexFutureRateHelper"]
    ] = "OvernightIndexFutureRateHelper"
    price: QuoteHandle
    valueDate: Date
    maturityDate: Date
    index: OVERNIGHTINDEX
    convexityAdjustment: Optional[QuoteHandle] = None
    type: Optional[enums.OvernightIndexFutureNettingType] = None


class SofrFutureRateHelper(BaseModel):
    resource_name: Optional[Literal["SofrFutureRateHelper"]] = "SofrFutureRateHelper"
    price: float
    referenceMonth: conint(ge=1, le=12)  # type: ignore
    referenceYear: conint(ge=1900, le=2999)  # type: ignore
    referenceFreq: float
    index: OVERNIGHTINDEX
    convexityAdjustment: Optional[float] = None
    type: Optional[enums.OvernightIndexFutureNettingType] = None


class FaceValueAccrualClaim(BaseModel):
    resource_name: Optional[Literal["FaceValueAccrualClaim"]] = "FaceValueAccrualClaim"
    bond: BOND


class CreditDefaultSwap(BaseModel):
    resource_name: Optional[Literal["CreditDefaultSwap"]] = "CreditDefaultSwap"
    side: enums.ProtectionSide
    notional: float
    upfront: float
    spread: float
    schedule: Schedule
    paymentConvention: enums.BusinessDayConvention
    dayCounter: DAYCOUNTER
    settlesAccrual: Optional[bool] = None
    paysAtDefaultTime: Optional[bool] = None
    protectionStart: Optional[Date] = None
    upfrontDate: Optional[Date] = None
    claim: Optional[CLAIM] = None
    lastPeriodDayCounter: Optional[DAYCOUNTER] = None
    rebatesAccrual: Optional[bool] = None


class IsdaCdsEngine(BaseModel):
    resource_name: Optional[Literal["IsdaCdsEngine"]] = "IsdaCdsEngine"
    probability: DefaultProbabilityTermStructureHandle
    recoveryRate: float
    discountCurve: YieldTermStructureHandle
    includeSettlementDateFlows: Optional[bool] = None
    numericalFix: Optional[enums.IsdaCdsEngineNumericalFix] = None
    accrualBias: Optional[IsdaCdsEngineAccrualBias] = None
    forwardsInCouponPeriod: Optional[IsdaCdsEngineForwardsInCouponPeriod] = None


class CdsOption(BaseModel):
    resource_name: Optional[Literal["CdsOption"]] = "CdsOption"
    swap: CreditDefaultSwap
    exercise: EXERCISE
    knocksOut: Optional[bool] = None


class FdmDirichletBoundary(BaseModel):
    resource_name: Optional[Literal["FdmDirichletBoundary"]] = "FdmDirichletBoundary"
    mesher: FDMMESHER
    valueOnBoundary: float
    direction: int
    side: FdmDirichletBoundarySide


class FdmDiscountDirichletBoundary(BaseModel):
    resource_name: Optional[
        Literal["FdmDiscountDirichletBoundary"]
    ] = "FdmDiscountDirichletBoundary"
    mesher: FDMMESHER
    rTS: YIELDTERMSTRUCTURE
    maturityTime: float
    valueOnBoundary: float
    direction: int
    side: FdmDiscountDirichletBoundarySide


class FdmBatesOp(BaseModel):
    resource_name: Optional[Literal["FdmBatesOp"]] = "FdmBatesOp"
    mesher: FDMMESHER
    batesProcess: BatesProcess
    bcSet: FdmBoundaryConditionSet
    integroIntegrationOrder: int
    quantoHelper: Optional[FdmQuantoHelper] = None


class FdmSquareRootFwdOp(BaseModel):
    resource_name: Optional[Literal["FdmSquareRootFwdOp"]] = "FdmSquareRootFwdOp"
    mesher: FDMMESHER
    kappa: float
    theta: float
    sigma: float
    direction: int
    type: Optional[enums.FdmSquareRootFwdOpTransformationType] = None


class FdmHestonFwdOp(BaseModel):
    resource_name: Optional[Literal["FdmHestonFwdOp"]] = "FdmHestonFwdOp"
    mesher: FDMMESHER
    process: HESTONPROCESS
    type: Optional[enums.FdmSquareRootFwdOpTransformationType] = None
    leverageFct: Optional[LOCALVOLTERMSTRUCTURE] = None


class CraigSneydScheme(BaseModel):
    resource_name: Optional[Literal["CraigSneydScheme"]] = "CraigSneydScheme"
    theta: float
    mu: float
    map: FDMLINEAROPCOMPOSITE
    bcSet: Optional[FdmBoundaryConditionSet] = None


class ImplicitEulerScheme(BaseModel):
    resource_name: Optional[Literal["ImplicitEulerScheme"]] = "ImplicitEulerScheme"
    map: FDMLINEAROPCOMPOSITE
    bcSet: Optional[FdmBoundaryConditionSet] = None
    relTol: Optional[float] = None
    solverType: Optional[enums.ImplicitEulerSchemeSolverType] = None


class CrankNicolsonScheme(BaseModel):
    resource_name: Optional[Literal["CrankNicolsonScheme"]] = "CrankNicolsonScheme"
    theta: float
    map: FDMLINEAROPCOMPOSITE
    bcSet: Optional[FdmBoundaryConditionSet] = None
    relTol: Optional[float] = None
    solverType: Optional[enums.ImplicitEulerSchemeSolverType] = None


class DouglasScheme(BaseModel):
    resource_name: Optional[Literal["DouglasScheme"]] = "DouglasScheme"
    theta: float
    map: FDMLINEAROPCOMPOSITE
    bcSet: Optional[FdmBoundaryConditionSet] = None


class ExplicitEulerScheme(BaseModel):
    resource_name: Optional[Literal["ExplicitEulerScheme"]] = "ExplicitEulerScheme"
    map: FDMLINEAROPCOMPOSITE
    bcSet: Optional[FdmBoundaryConditionSet] = None


class HundsdorferScheme(BaseModel):
    resource_name: Optional[Literal["HundsdorferScheme"]] = "HundsdorferScheme"
    theta: float
    mu: float
    map: FDMLINEAROPCOMPOSITE
    bcSet: Optional[FdmBoundaryConditionSet] = None


class MethodOfLinesScheme(BaseModel):
    resource_name: Optional[Literal["MethodOfLinesScheme"]] = "MethodOfLinesScheme"
    eps: float
    relInitStepSize: float
    map: FDMLINEAROPCOMPOSITE
    bcSet: Optional[FdmBoundaryConditionSet] = None


class ModifiedCraigSneydScheme(BaseModel):
    resource_name: Optional[
        Literal["ModifiedCraigSneydScheme"]
    ] = "ModifiedCraigSneydScheme"
    theta: float
    mu: float
    map: FDMLINEAROPCOMPOSITE
    bcSet: Optional[FdmBoundaryConditionSet] = None


class Fdm1DimSolver(BaseModel):
    resource_name: Optional[Literal["Fdm1DimSolver"]] = "Fdm1DimSolver"
    solverDesc: FdmSolverDesc
    schemeDesc: FdmSchemeDesc
    op: FDMLINEAROPCOMPOSITE


class FdmBackwardSolver(BaseModel):
    resource_name: Optional[Literal["FdmBackwardSolver"]] = "FdmBackwardSolver"
    map: FDMLINEAROPCOMPOSITE
    bcSet: FdmBoundaryConditionSet
    condition: FdmStepConditionComposite
    schemeDesc: FdmSchemeDesc


class Fdm2DimSolver(BaseModel):
    resource_name: Optional[Literal["Fdm2DimSolver"]] = "Fdm2DimSolver"
    solverDesc: FdmSolverDesc
    schemeDesc: FdmSchemeDesc
    op: FDMLINEAROPCOMPOSITE


class Fdm3DimSolver(BaseModel):
    resource_name: Optional[Literal["Fdm3DimSolver"]] = "Fdm3DimSolver"
    solverDesc: FdmSolverDesc
    schemeDesc: FdmSchemeDesc
    op: FDMLINEAROPCOMPOSITE


class FdmIndicesOnBoundary(BaseModel):
    resource_name: Optional[Literal["FdmIndicesOnBoundary"]] = "FdmIndicesOnBoundary"
    l: FdmLinearOpLayout
    direction: int
    side: FdmDirichletBoundarySide


class FittedBondDiscountCurve(BaseModel):
    resource_name: Optional[
        Literal["FittedBondDiscountCurve"]
    ] = "FittedBondDiscountCurve"
    referenceDate: Date
    helpers: List[BONDHELPER]
    dayCounter: DAYCOUNTER
    fittingMethod: FITTINGMETHOD
    accuracy: Optional[float] = None
    maxEvaluations: Optional[int] = None
    guess: Optional[Array] = None
    simplexLambda: Optional[float] = None


class FixedRateBondForward(BaseModel):
    resource_name: Optional[Literal["FixedRateBondForward"]] = "FixedRateBondForward"
    valueDate: Date
    maturityDate: Date
    type: enums.PositionType
    strike: float
    settlementDays: float
    dayCounter: DAYCOUNTER
    calendar: CALENDAR
    businessDayConvention: enums.BusinessDayConvention
    fixedBond: FixedRateBond
    discountCurve: Optional[YieldTermStructureHandle] = None
    incomeDiscountCurve: Optional[YieldTermStructureHandle] = None


class ForwardRateAgreement(BaseModel):
    resource_name: Optional[Literal["ForwardRateAgreement"]] = "ForwardRateAgreement"
    valueDate: Date
    maturityDate: Date
    type: enums.PositionType
    strikeForwardRate: float
    notionalAmount: float
    index: IBORINDEX
    discountCurve: Optional[YieldTermStructureHandle] = None
    useIndexedCoupon: Optional[bool] = None


class MarkovFunctional(BaseModel):
    resource_name: Optional[Literal["MarkovFunctional"]] = "MarkovFunctional"
    termStructure: YieldTermStructureHandle
    reversion: float
    volstepdates: List[Date]
    volatilities: List[float]
    capletVol: OptionletVolatilityStructureHandle
    capletExpiries: List[Date]
    iborIndex: IBORINDEX
    modelSettings: Optional[MarkovFunctionalModelSettings] = None


class Gaussian1dSwaptionEngine(BaseModel):
    resource_name: Optional[
        Literal["Gaussian1dSwaptionEngine"]
    ] = "Gaussian1dSwaptionEngine"
    model: GAUSSIAN1DMODEL
    integrationPoints: Optional[int] = None
    stddevs: Optional[float] = None
    extrapolatePayoff: Optional[bool] = None
    flatPayoffExtrapolation: Optional[bool] = None
    discountCurve: Optional[YieldTermStructureHandle] = None
    probabilities: Optional[enums.Gaussian1dSwaptionEngineProbabilities] = None


class Gaussian1dNonstandardSwaptionEngine(BaseModel):
    resource_name: Optional[
        Literal["Gaussian1dNonstandardSwaptionEngine"]
    ] = "Gaussian1dNonstandardSwaptionEngine"
    model: GAUSSIAN1DMODEL
    integrationPoints: Optional[int] = None
    stddevs: Optional[float] = None
    extrapolatePayoff: Optional[bool] = None
    flatPayoffExtrapolation: Optional[bool] = None
    oas: Optional[QuoteHandle] = None
    discountCurve: Optional[YieldTermStructureHandle] = None
    probabilities: Optional[Gaussian1dNonstandardSwaptionEngineProbabilities] = None


class Gaussian1dFloatFloatSwaptionEngine(BaseModel):
    resource_name: Optional[
        Literal["Gaussian1dFloatFloatSwaptionEngine"]
    ] = "Gaussian1dFloatFloatSwaptionEngine"
    model: GAUSSIAN1DMODEL
    integrationPoints: Optional[int] = None
    stddevs: Optional[float] = None
    extrapolatePayoff: Optional[bool] = None
    flatPayoffExtrapolation: Optional[bool] = None
    oas: Optional[QuoteHandle] = None
    discountCurve: Optional[YieldTermStructureHandle] = None
    includeTodaysExercise: Optional[bool] = None
    probabilities: Optional[
        enums.Gaussian1dFloatFloatSwaptionEngineProbabilities
    ] = None


class SobolRsg(BaseModel):
    resource_name: Optional[Literal["SobolRsg"]] = "SobolRsg"
    dimensionality: int
    seed: Optional[int] = None
    directionIntegers: Optional[enums.SobolRsgDirectionIntegers] = None


class UniformLowDiscrepancySequenceGenerator(BaseModel):
    resource_name: Optional[
        Literal["UniformLowDiscrepancySequenceGenerator"]
    ] = "UniformLowDiscrepancySequenceGenerator"
    dimensionality: int
    seed: Optional[int] = None
    directionIntegers: Optional[enums.SobolRsgDirectionIntegers] = None


class GaussianLowDiscrepancySequenceGenerator(BaseModel):
    resource_name: Optional[
        Literal["GaussianLowDiscrepancySequenceGenerator"]
    ] = "GaussianLowDiscrepancySequenceGenerator"
    u: UniformLowDiscrepancySequenceGenerator


class NeumannBC(BaseModel):
    resource_name: Optional[Literal["NeumannBC"]] = "NeumannBC"
    value: float
    side: DefaultBoundaryConditionSide


class DirichletBC(BaseModel):
    resource_name: Optional[Literal["DirichletBC"]] = "DirichletBC"
    value: float
    side: DefaultBoundaryConditionSide


class SobolBrownianGeneratorFactory(BaseModel):
    resource_name: Optional[
        Literal["SobolBrownianGeneratorFactory"]
    ] = "SobolBrownianGeneratorFactory"
    ordering: enums.SobolBrownianGeneratorOrdering
    seed: Optional[int] = None
    directionIntegers: Optional[enums.SobolRsgDirectionIntegers] = None


class FdHestonBarrierEngine(BaseModel):
    resource_name: Optional[Literal["FdHestonBarrierEngine"]] = "FdHestonBarrierEngine"
    model: HESTONMODEL
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    vGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    schemeDesc: Optional[FdmSchemeDesc] = None
    leverageFct: Optional[LOCALVOLTERMSTRUCTURE] = None


class FdHestonDoubleBarrierEngine(BaseModel):
    resource_name: Optional[
        Literal["FdHestonDoubleBarrierEngine"]
    ] = "FdHestonDoubleBarrierEngine"
    model: HESTONMODEL
    tGrid: Optional[int] = None
    xGrid: Optional[int] = None
    vGrid: Optional[int] = None
    dampingSteps: Optional[int] = None
    schemeDesc: Optional[FdmSchemeDesc] = None
    leverageFct: Optional[LOCALVOLTERMSTRUCTURE] = None


class OptionletStripper1(BaseModel):
    resource_name: Optional[Literal["OptionletStripper1"]] = "OptionletStripper1"
    parVolSurface: CapFloorTermVolSurface
    index: IBORINDEX
    switchStrikes: Optional[float] = None
    accuracy: Optional[float] = None
    maxIter: Optional[float] = None
    discount: Optional[YieldTermStructureHandle] = None
    type: Optional[VolatilityType] = None
    displacement: Optional[float] = None
    dontThrow: Optional[bool] = None


class Swaption(BaseModel):
    resource_name: Optional[Literal["Swaption"]] = "Swaption"
    swap: VanillaSwap
    exercise: EXERCISE
    type: Optional[enums.SettlementType] = None
    settlementMethod: Optional[SettlementMethod] = None


class NonstandardSwaption(BaseModel):
    resource_name: Optional[Literal["NonstandardSwaption"]] = "NonstandardSwaption"
    swap: NonstandardSwap
    exercise: EXERCISE
    type: Optional[enums.SettlementType] = None
    settlementMethod: Optional[SettlementMethod] = None


class FloatFloatSwaption(BaseModel):
    resource_name: Optional[Literal["FloatFloatSwaption"]] = "FloatFloatSwaption"
    swap: FloatFloatSwap
    exercise: EXERCISE
    delivery: Optional[enums.SettlementType] = None
    settlementMethod: Optional[SettlementMethod] = None


class AnalyticHestonEngineComplexLogFormula(BaseModel):
    resource_name: Optional[
        Literal["AnalyticHestonEngineComplexLogFormula"]
    ] = "AnalyticHestonEngineComplexLogFormula"


class HestonBlackVolSurface(BaseModel):
    resource_name: Optional[Literal["HestonBlackVolSurface"]] = "HestonBlackVolSurface"
    hestonModel: HestonModelHandle
    cpxLogFormula: Optional[AnalyticHestonEngineComplexLogFormula] = None
    integration: Optional[AnalyticHestonEngineIntegration] = None


class AnalyticHestonEngine(BaseModel):
    resource_name: Optional[Literal["AnalyticHestonEngine"]] = "AnalyticHestonEngine"
    model: HESTONMODEL
    cpxLog: AnalyticHestonEngineComplexLogFormula
    itg: AnalyticHestonEngineIntegration
    andersenPiterbargEpsilon: Optional[float] = None


class AssetSwap(BaseModel):
    resource_name: Optional[Literal["AssetSwap"]] = "AssetSwap"
    payFixedRate: bool
    bond: BOND
    bondCleanPrice: float
    index: IBORINDEX
    spread: float
    floatSchedule: Optional[Schedule] = None
    floatingDayCount: Optional[DAYCOUNTER] = None
    parAssetSwap: Optional[bool] = None


class ExtendedOrnsteinUhlenbeckProcess(BaseModel):
    resource_name: Optional[
        Literal["ExtendedOrnsteinUhlenbeckProcess"]
    ] = "ExtendedOrnsteinUhlenbeckProcess"


class YoYInflationCapFloor(BaseModel):
    resource_name: Optional[Literal["YoYInflationCapFloor"]] = "YoYInflationCapFloor"


CashFlows.update_forward_refs()


STRIPPEDOPTIONLETBASE = Union[StrippedOptionletBase, OptionletStripper1]
CAPFLOORTERMVOLATILITYSTRUCTURE = Union[
    CapFloorTermVolatilityStructure, CapFloorTermVolCurve, CapFloorTermVolSurface
]
STATISTICS = Union[Statistics, RiskStatistics]
BROWNIANGENERATORFACTORY = Union[
    BrownianGeneratorFactory, MTBrownianGeneratorFactory, SobolBrownianGeneratorFactory
]
TRIDIAGONALOPERATOR = Union[TridiagonalOperator, DPlus, DMinus, DZero, DPlusDMinus]
DEFAULTBOUNDARYCONDITION = Union[DefaultBoundaryCondition, NeumannBC, DirichletBC]
GAUSSIAN1DMODEL = Union[Gaussian1dModel, Gsr, MarkovFunctional]
TERMSTRUCTURECONSISTENTMODEL = Union[TermStructureConsistentModel, GAUSSIAN1DMODEL]
FORWARD = Union[Forward, FixedRateBondForward, ForwardRateAgreement]
FITTINGMETHOD = Union[
    FittingMethod,
    ExponentialSplinesFitting,
    NelsonSiegelFitting,
    SvenssonFitting,
    CubicBSplinesFitting,
    SimplePolynomialFitting,
]
RISKNEUTRALDENSITYCALCULATOR = Union[
    RiskNeutralDensityCalculator,
    BSMRNDCalculator,
    CEVRNDCalculator,
    GBSMRNDCalculator,
    HestonRNDCalculator,
    LocalVolRNDCalculator,
    SquareRootProcessRNDCalculator,
]
FDMCELLAVERAGINGINNERVALUE = Union[FdmCellAveragingInnerValue, FdmLogInnerValue]
FDMINNERVALUECALCULATOR = Union[
    FdmInnerValueCalculator,
    FDMCELLAVERAGINGINNERVALUE,
    FdmLogBasketInnerValue,
    FdmZeroInnerValue,
]
NINEPOINTLINEAROP = Union[NinePointLinearOp, SecondOrderMixedDerivativeOp]
TRIPLEBANDLINEAROP = Union[TripleBandLinearOp, FirstDerivativeOp, SecondDerivativeOp]
FDMBOUNDARYCONDITION = Union[
    FdmBoundaryCondition,
    FdmDirichletBoundary,
    FdmDiscountDirichletBoundary,
    FdmTimeDepDirichletBoundary,
]
FDMLINEAROPCOMPOSITE = Union[
    FdmLinearOpComposite,
    FdmBatesOp,
    FdmBlackScholesOp,
    Fdm2dBlackScholesOp,
    FdmCEVOp,
    FdmG2Op,
    FdmHestonHullWhiteOp,
    FdmHestonOp,
    FdmHullWhiteOp,
    FdmLocalVolFwdOp,
    FdmOrnsteinUhlenbeckOp,
    FdmSabrOp,
    FdmZabrOp,
    FdmDupire1dOp,
    FdmBlackScholesFwdOp,
    FdmSquareRootFwdOp,
    FdmHestonFwdOp,
]
FDMLINEAROP = Union[
    FdmLinearOp,
    FDMLINEAROPCOMPOSITE,
    TRIPLEBANDLINEAROP,
    NINEPOINTLINEAROP,
    NthOrderDerivativeOp,
]
FDMMESHER = Union[FdmMesher, FdmMesherComposite]
FDM1DMESHER = Union[
    Fdm1dMesher,
    FdmBlackScholesMesher,
    Concentrating1dMesher,
    ExponentialJump1dMesher,
    FdmCEV1dMesher,
    FdmHestonVarianceMesher,
    FdmHestonLocalVolatilityVarianceMesher,
    FdmSimpleProcess1dMesher,
    Uniform1dMesher,
    Predefined1dMesher,
    Glued1dMesher,
]
CLAIM = Union[Claim, FaceValueClaim, FaceValueAccrualClaim]
DEFAULTPROBABILITYHELPER = Union[
    DefaultProbabilityHelper, SpreadCdsHelper, UpfrontCdsHelper
]
DEFAULTPROBABILITYTERMSTRUCTURE = Union[
    DefaultProbabilityTermStructure, FlatHazardRate, PiecewiseFlatHazardRate
]
OVERNIGHTINDEXFUTURERATEHELPER = Union[
    OvernightIndexFutureRateHelper, SofrFutureRateHelper
]
BONDHELPER = Union[BondHelper, FixedRateBondHelper]
RATEHELPER = Union[
    RateHelper,
    DepositRateHelper,
    FraRateHelper,
    FuturesRateHelper,
    SwapRateHelper,
    BONDHELPER,
    OISRateHelper,
    DatedOISRateHelper,
    FxSwapRateHelper,
    OVERNIGHTINDEXFUTURERATEHELPER,
]
DIVIDEND = Union[Dividend, FixedDividend, FractionalDividend]
CAPFLOOR = Union[CapFloor, Cap, Floor, Collar]
CALLABLEBOND = Union[CallableBond, CallableFixedRateBond]
BOND = Union[
    Bond,
    ZeroCouponBond,
    FixedRateBond,
    AmortizingFixedRateBond,
    AmortizingFloatingRateBond,
    FloatingRateBond,
    CmsRateBond,
    CALLABLEBOND,
    CPIBond,
    ConvertibleZeroCouponBond,
    ConvertibleFixedCouponBond,
    ConvertibleFloatingRateBond,
]
VASICEK = Union[Vasicek, HullWhite]
ONEFACTORAFFINEMODEL = Union[OneFactorAffineModel, VASICEK]
SHORTRATEMODEL = Union[ShortRateModel, ONEFACTORAFFINEMODEL, BlackKarasinski, G2]
YOYINFLATIONCAPFLOOR = Union[
    YoYInflationCapFloor, YoYInflationCap, YoYInflationFloor, YoYInflationCollar
]
INFLATIONCOUPON = Union[InflationCoupon, CPICoupon]
YOYINFLATIONINDEX = Union[
    YoYInflationIndex, YYEUHICP, YYEUHICPXT, YYFRHICP, YYUKRPI, YYUSCPI, YYZACPI
]
ZEROINFLATIONINDEX = Union[
    ZeroInflationIndex, EUHICP, EUHICPXT, FRHICP, UKRPI, USCPI, ZACPI
]
INFLATIONINDEX = Union[InflationIndex, ZEROINFLATIONINDEX, YOYINFLATIONINDEX]
REGION = Union[Region, CustomRegion]
INFLATIONTERMSTRUCTURE = Union[
    InflationTermStructure, YoYInflationTermStructure, ZeroInflationTermStructure
]
SEASONALITY = Union[Seasonality, MultiplicativePriceSeasonality]
SWAP = Union[
    Swap,
    VanillaSwap,
    NonstandardSwap,
    AssetSwap,
    FloatFloatSwap,
    OvernightIndexedSwap,
    ZeroCouponInflationSwap,
    YearOnYearInflationSwap,
    CPISwap,
]
CALLABILITY = Union[Callability, SoftCallability]
MULTIASSETOPTION = Union[MultiAssetOption, BasketOption, EverestOption, HimalayaOption]
BASKETPAYOFF = Union[
    BasketPayoff,
    MinBasketPayoff,
    MaxBasketPayoff,
    AverageBasketPayoff,
    SpreadBasketPayoff,
]
STRIKEDTYPEPAYOFF = Union[
    StrikedTypePayoff,
    PlainVanillaPayoff,
    PercentageStrikePayoff,
    CashOrNothingPayoff,
    AssetOrNothingPayoff,
    SuperSharePayoff,
    GapPayoff,
    VanillaForwardPayoff,
]
DOUBLEBARRIEROPTION = Union[DoubleBarrierOption, QuantoDoubleBarrierOption]
HESTONMODEL = Union[HestonModel, BatesModel]
CALIBRATEDMODEL = Union[
    CalibratedModel,
    HESTONMODEL,
    PiecewiseTimeDependentHestonModel,
    GJRGARCHModel,
    SHORTRATEMODEL,
]
PRICINGENGINE = Union[
    PricingEngine,
    AnalyticEuropeanEngine,
    AnalyticHestonEngine,
    COSHestonEngine,
    AnalyticPTDHestonEngine,
    BatesEngine,
    IntegralEngine,
    FDBermudanEngine,
    FDEuropeanEngine,
    BinomialCRRVanillaEngine,
    BinomialJRVanillaEngine,
    BinomialTrigeorgisVanillaEngine,
    BinomialTianVanillaEngine,
    BinomialJoshi4VanillaEngine,
    FDAmericanEngine,
    FDShoutEngine,
    ContinuousArithmeticAsianLevyEngine,
    FdBlackScholesAsianEngine,
    BaroneAdesiWhaleyApproximationEngine,
    BjerksundStenslandApproximationEngine,
    JuQuadraticApproximationEngine,
    AnalyticDigitalAmericanEngine,
    AnalyticDigitalAmericanKOEngine,
    AnalyticDividendEuropeanEngine,
    AnalyticBarrierEngine,
    FdBlackScholesVanillaEngine,
    FdOrnsteinUhlenbeckVanillaEngine,
    FdBatesVanillaEngine,
    FdHestonVanillaEngine,
    AnalyticCEVEngine,
    FdCEVVanillaEngine,
    FdSabrVanillaEngine,
    FdBlackScholesBarrierEngine,
    AnalyticBinaryBarrierEngine,
    ForwardEuropeanEngine,
    QuantoEuropeanEngine,
    QuantoForwardEuropeanEngine,
    AnalyticContinuousGeometricAveragePriceAsianEngine,
    AnalyticDiscreteGeometricAveragePriceAsianEngine,
    AnalyticDiscreteGeometricAverageStrikeAsianEngine,
    VarianceGammaEngine,
    FFTVarianceGammaEngine,
    AnalyticDoubleBarrierEngine,
    WulinYongDoubleBarrierEngine,
    VannaVolgaBarrierEngine,
    AnalyticDoubleBarrierBinaryEngine,
    FdSimpleBSSwingEngine,
    FdSimpleExtOUJumpSwingEngine,
    AnalyticGJRGARCHEngine,
    StulzEngine,
    KirkEngine,
    Fd2dBlackScholesVanillaEngine,
    DiscountingSwapEngine,
    JamshidianSwaptionEngine,
    TreeSwaptionEngine,
    AnalyticCapFloorEngine,
    TreeCapFloorEngine,
    G2SwaptionEngine,
    FdG2SwaptionEngine,
    FdHullWhiteSwaptionEngine,
    DiscountingBondEngine,
    TreeCallableFixedRateBondEngine,
    BlackCallableFixedRateBondEngine,
    BlackCapFloorEngine,
    BachelierCapFloorEngine,
    MidPointCdsEngine,
    IntegralCdsEngine,
    IsdaCdsEngine,
    BlackCdsOptionEngine,
    Gaussian1dSwaptionEngine,
    Gaussian1dJamshidianSwaptionEngine,
    Gaussian1dNonstandardSwaptionEngine,
    Gaussian1dFloatFloatSwaptionEngine,
    FdHestonBarrierEngine,
    FdHestonDoubleBarrierEngine,
    BlackSwaptionEngine,
    BachelierSwaptionEngine,
]
FORWARDVANILLAOPTION = Union[ForwardVanillaOption, QuantoForwardVanillaOption]
VANILLAOPTION = Union[VanillaOption, EuropeanOption]
ONEASSETOPTION = Union[
    OneAssetOption,
    VANILLAOPTION,
    FORWARDVANILLAOPTION,
    QuantoVanillaOption,
    DividendVanillaOption,
    BarrierOption,
    ContinuousAveragingAsianOption,
    DiscreteAveragingAsianOption,
    DOUBLEBARRIEROPTION,
    VanillaSwingOption,
]
OPTION = Union[Option, ONEASSETOPTION, MULTIASSETOPTION, CdsOption, Swaption]
TYPEPAYOFF = Union[TypePayoff, STRIKEDTYPEPAYOFF]
PAYOFF = Union[Payoff, TYPEPAYOFF, BASKETPAYOFF]
PARAMETER = Union[
    Parameter, ConstantParameter, NullParameter, PiecewiseConstantParameter
]
BLACKCALIBRATIONHELPER = Union[
    BlackCalibrationHelper, SwaptionHelper, CapHelper, HestonModelHelper
]
CALIBRATIONHELPER = Union[CalibrationHelper, BLACKCALIBRATIONHELPER]
CMSSPREADCOUPONPRICER = Union[CmsSpreadCouponPricer, LognormalCmsSpreadPricer]
CMSCOUPONPRICER = Union[
    CmsCouponPricer, AnalyticHaganPricer, NumericHaganPricer, LinearTsrPricer
]
IBORCOUPONPRICER = Union[IborCouponPricer, BlackIborCouponPricer]
FLOATINGRATECOUPONPRICER = Union[
    FloatingRateCouponPricer, IBORCOUPONPRICER, CMSCOUPONPRICER, CMSSPREADCOUPONPRICER
]
CAPPEDFLOOREDCOUPON = Union[
    CappedFlooredCoupon,
    CappedFlooredIborCoupon,
    CappedFlooredCmsCoupon,
    CappedFlooredCmsSpreadCoupon,
]
FLOATINGRATECOUPON = Union[
    FloatingRateCoupon,
    OvernightIndexedCoupon,
    CAPPEDFLOOREDCOUPON,
    IborCoupon,
    CmsCoupon,
    CmsSpreadCoupon,
]
COUPON = Union[Coupon, FixedRateCoupon, FLOATINGRATECOUPON, INFLATIONCOUPON]
CASHFLOW = Union[
    CashFlow, SimpleCashFlow, Redemption, AmortizingPayment, COUPON, DIVIDEND
]
INSTRUMENT = Union[
    Instrument,
    Stock,
    CompositeInstrument,
    OPTION,
    SWAP,
    YOYINFLATIONCAPFLOOR,
    BOND,
    CAPFLOOR,
    CreditDefaultSwap,
    FORWARD,
    NonstandardSwaption,
    FloatFloatSwaption,
]
HESTONPROCESS = Union[HestonProcess, BatesProcess]
GENERALIZEDBLACKSCHOLESPROCESS = Union[
    GeneralizedBlackScholesProcess,
    BlackScholesProcess,
    BlackScholesMertonProcess,
    BlackProcess,
    GarmanKohlagenProcess,
]
STOCHASTICPROCESS1D = Union[
    StochasticProcess1D,
    GENERALIZEDBLACKSCHOLESPROCESS,
    Merton76Process,
    GeometricBrownianMotionProcess,
    VarianceGammaProcess,
    HullWhiteProcess,
    HullWhiteForwardProcess,
    GsrProcess,
    OrnsteinUhlenbeckProcess,
    ExtendedOrnsteinUhlenbeckProcess,
]
STOCHASTICPROCESS = Union[
    StochasticProcess,
    STOCHASTICPROCESS1D,
    StochasticProcessArray,
    HESTONPROCESS,
    G2Process,
    G2ForwardProcess,
    ExtOUWithJumpsProcess,
    KlugeExtOUProcess,
    GJRGARCHProcess,
    HestonSLVProcess,
]
SMILESECTION = Union[
    SmileSection,
    FlatSmileSection,
    SabrSmileSection,
    KahaleSmileSection,
    NoArbSabrSmileSection,
    NoArbSabrInterpolatedSmileSection,
]
SWAPTIONVOLATILITYDISCRETE = Union[
    SwaptionVolatilityDiscrete,
    SwaptionVolatilityMatrix,
    SwaptionVolCube1,
    SwaptionVolCube2,
]
SWAPTIONVOLATILITYSTRUCTURE = Union[
    SwaptionVolatilityStructure, ConstantSwaptionVolatility, SWAPTIONVOLATILITYDISCRETE
]
OPTIONLETVOLATILITYSTRUCTURE = Union[
    OptionletVolatilityStructure, ConstantOptionletVolatility, StrippedOptionletAdapter
]
LOCALVOLTERMSTRUCTURE = Union[
    LocalVolTermStructure,
    LocalConstantVol,
    LocalVolSurface,
    AndreasenHugeLocalVolAdapter,
]
BLACKVOLTERMSTRUCTURE = Union[
    BlackVolTermStructure,
    BlackConstantVol,
    BlackVarianceCurve,
    BlackVarianceSurface,
    AndreasenHugeVolatilityAdapter,
    HestonBlackVolSurface,
]
VOLATILITYTERMSTRUCTURE = Union[
    VolatilityTermStructure,
    BLACKVOLTERMSTRUCTURE,
    LOCALVOLTERMSTRUCTURE,
    OPTIONLETVOLATILITYSTRUCTURE,
    SWAPTIONVOLATILITYSTRUCTURE,
    CAPFLOORTERMVOLATILITYSTRUCTURE,
]
OPTIMIZATIONMETHOD = Union[
    OptimizationMethod,
    ConjugateGradient,
    Simplex,
    SteepestDescent,
    BFGS,
    LevenbergMarquardt,
    DifferentialEvolution,
    GaussianSimulatedAnnealing,
    MirrorGaussianSimulatedAnnealing,
    LogNormalSimulatedAnnealing,
]
CONSTRAINT = Union[
    Constraint,
    BoundaryConstraint,
    NoConstraint,
    PositiveConstraint,
    CompositeConstraint,
    NonhomogeneousBoundaryConstraint,
]
BIBOR = Union[Bibor, BiborSW, Bibor1M, Bibor2M, Bibor3M, Bibor6M, Bibor9M, Bibor1Y]
SWAPINDEX = Union[
    SwapIndex,
    EuriborSwapIsdaFixA,
    EuriborSwapIsdaFixB,
    EuriborSwapIfrFix,
    EurLiborSwapIsdaFixA,
    EurLiborSwapIsdaFixB,
    EurLiborSwapIfrFix,
    ChfLiborSwapIsdaFix,
    GbpLiborSwapIsdaFix,
    JpyLiborSwapIsdaFixAm,
    JpyLiborSwapIsdaFixPm,
    UsdLiborSwapIsdaFixAm,
    UsdLiborSwapIsdaFixPm,
    OvernightIndexedSwapIndex,
]
OVERNIGHTINDEX = Union[OvernightIndex, Aonia, Eonia, Sonia, FedFunds, Nzocr, Sofr]
EURLIBOR = Union[
    EURLibor,
    EURLiborSW,
    EURLibor2W,
    EURLibor1M,
    EURLibor2M,
    EURLibor3M,
    EURLibor4M,
    EURLibor5M,
    EURLibor6M,
    EURLibor7M,
    EURLibor8M,
    EURLibor9M,
    EURLibor10M,
    EURLibor11M,
    EURLibor1Y,
]
EURIBOR365 = Union[Euribor365]
EURIBOR = Union[
    Euribor,
    EuriborSW,
    Euribor2W,
    Euribor3W,
    Euribor1M,
    Euribor2M,
    Euribor3M,
    Euribor4M,
    Euribor5M,
    Euribor6M,
    Euribor7M,
    Euribor8M,
    Euribor9M,
    Euribor10M,
    Euribor11M,
    Euribor1Y,
]
BKBM = Union[Bkbm, Bkbm1M, Bkbm2M, Bkbm3M, Bkbm4M, Bkbm5M, Bkbm6M]
BBSW = Union[Bbsw, Bbsw1M, Bbsw2M, Bbsw3M, Bbsw4M, Bbsw5M, Bbsw6M]
DAILYTENORLIBOR = Union[DailyTenorLibor, CADLiborON, GBPLiborON, USDLiborON]
IBORINDEX = Union[
    IborIndex,
    OVERNIGHTINDEX,
    Libor,
    DAILYTENORLIBOR,
    AUDLibor,
    CADLibor,
    Cdor,
    CHFLibor,
    DKKLibor,
    BBSW,
    BKBM,
    EURIBOR,
    EURIBOR365,
    EURLIBOR,
    GBPLibor,
    Jibar,
    JPYLibor,
    Mosprime,
    NZDLibor,
    Pribor,
    Robor,
    SEKLibor,
    Shibor,
    Tibor,
    THBFIX,
    TRLibor,
    USDLibor,
    Wibor,
    Zibor,
    BIBOR,
]
INTERESTRATEINDEX = Union[InterestRateIndex, IBORINDEX, SWAPINDEX, SwapSpreadIndex]
INDEX = Union[Index, INTERESTRATEINDEX, INFLATIONINDEX]
YIELDTERMSTRUCTURE = Union[
    YieldTermStructure,
    ImpliedTermStructure,
    ZeroSpreadedTermStructure,
    ForwardSpreadedTermStructure,
    FlatForward,
    PiecewiseFlatForward,
    PiecewiseLogLinearDiscount,
    PiecewiseLinearForward,
    PiecewiseLinearZero,
    PiecewiseCubicZero,
    PiecewiseLogCubicDiscount,
    PiecewiseSplineCubicDiscount,
    PiecewiseKrugerZero,
    PiecewiseKrugerLogDiscount,
    PiecewiseConvexMonotoneZero,
    GlobalLinearSimpleZeroCurve,
    FittedBondDiscountCurve,
    ZeroCurve,
    LogLinearInterpolatedZeroCurve,
    CubicInterpolatedZeroCurve,
    SplineCubicInterpolatedZeroCurve,
    DefaultLogCubicInterpolatedZeroCurve,
    MonotonicCubicInterpolatedZeroCurve,
]
TERMSTRUCTURE = Union[
    TermStructure,
    YIELDTERMSTRUCTURE,
    VOLATILITYTERMSTRUCTURE,
    INFLATIONTERMSTRUCTURE,
    DEFAULTPROBABILITYTERMSTRUCTURE,
]
CURRENCY = Union[
    Currency,
    ARSCurrency,
    ATSCurrency,
    AUDCurrency,
    BDTCurrency,
    BEFCurrency,
    BGLCurrency,
    BRLCurrency,
    BYRCurrency,
    CADCurrency,
    CHFCurrency,
    CLPCurrency,
    CNYCurrency,
    COPCurrency,
    CYPCurrency,
    CZKCurrency,
    DEMCurrency,
    DKKCurrency,
    EEKCurrency,
    ESPCurrency,
    EURCurrency,
    FIMCurrency,
    FRFCurrency,
    GBPCurrency,
    GRDCurrency,
    HKDCurrency,
    HUFCurrency,
    IEPCurrency,
    IDRCurrency,
    ILSCurrency,
    INRCurrency,
    IQDCurrency,
    IRRCurrency,
    ISKCurrency,
    ITLCurrency,
    JPYCurrency,
    KRWCurrency,
    KWDCurrency,
    LTLCurrency,
    LUFCurrency,
    LVLCurrency,
    MTLCurrency,
    MXNCurrency,
    MYRCurrency,
    NLGCurrency,
    NOKCurrency,
    NPRCurrency,
    NZDCurrency,
    PEHCurrency,
    PEICurrency,
    PENCurrency,
    PKRCurrency,
    PLNCurrency,
    PTECurrency,
    ROLCurrency,
    RONCurrency,
    RUBCurrency,
    SARCurrency,
    SEKCurrency,
    SGDCurrency,
    SITCurrency,
    SKKCurrency,
    THBCurrency,
    TRLCurrency,
    TRYCurrency,
    TTDCurrency,
    TWDCurrency,
    USDCurrency,
    VEBCurrency,
    VNDCurrency,
    ZARCurrency,
]
ROUNDING = Union[
    Rounding,
    UpRounding,
    DownRounding,
    ClosestRounding,
    CeilingTruncation,
    FloorTruncation,
]
DAYCOUNTER = Union[
    DayCounter,
    Actual360,
    Actual365Fixed,
    Thirty360,
    ActualActual,
    OneDayCounter,
    SimpleDayCounter,
    Business252,
]
CALENDAR = Union[
    Calendar,
    Argentina,
    Australia,
    Brazil,
    Canada,
    China,
    CzechRepublic,
    Denmark,
    Finland,
    France,
    Germany,
    HongKong,
    Hungary,
    Iceland,
    India,
    Indonesia,
    Israel,
    Italy,
    Japan,
    Mexico,
    NewZealand,
    Norway,
    Poland,
    Russia,
    Romania,
    SaudiArabia,
    Singapore,
    Slovakia,
    SouthAfrica,
    SouthKorea,
    Sweden,
    Switzerland,
    Taiwan,
    TARGET,
    Thailand,
    Turkey,
    Ukraine,
    UnitedKingdom,
    UnitedStates,
    NullCalendar,
    WeekendsOnly,
    JointCalendar,
    BespokeCalendar,
]
QUOTE = Union[Quote, SimpleQuote, DerivedQuote, CompositeQuote, DeltaVolQuote]
OBSERVABLE = Union[
    QUOTE,
    TERMSTRUCTURE,
    INDEX,
    SMILESECTION,
    AndreasenHugeVolatilityInterpl,
    STOCHASTICPROCESS,
    PRICINGENGINE,
    INSTRUMENT,
    CASHFLOW,
    CALIBRATEDMODEL,
    TERMSTRUCTURECONSISTENTMODEL,
    RATEHELPER,
    DEFAULTPROBABILITYHELPER,
]
EXERCISE = Union[
    Exercise,
    EuropeanExercise,
    AmericanExercise,
    BermudanExercise,
    RebatedExercise,
    SwingExercise,
]


BSMRNDCalculator.update_forward_refs()
FDBermudanEngine.update_forward_refs()
Ukraine.update_forward_refs()
Bibor3M.update_forward_refs()
UniformRandomSequenceGenerator.update_forward_refs()
TemperatureExponential.update_forward_refs()
FdmHullWhiteOp.update_forward_refs()
InverseCumulativeStudent.update_forward_refs()
FdmDirichletBoundarySide.update_forward_refs()
PercentageStrikePayoff.update_forward_refs()
BermudanExercise.update_forward_refs()
CreditDefaultSwap.update_forward_refs()
SafeBicubicSpline.update_forward_refs()
MidPointCdsEngine.update_forward_refs()
Euribor10M.update_forward_refs()
NonstandardSwaption.update_forward_refs()
AnalyticEuropeanEngine.update_forward_refs()
CumulativeBinomialDistribution.update_forward_refs()
FedFunds.update_forward_refs()
BlackScholesProcess.update_forward_refs()
GarmanKohlagenProcess.update_forward_refs()
ZeroInflationIndex.update_forward_refs()
VanillaSwap.update_forward_refs()
Mosprime.update_forward_refs()
SplineCubicInterpolatedZeroCurve.update_forward_refs()
Bond.update_forward_refs()
ContinuousAveragingAsianOption.update_forward_refs()
SafeBackwardFlatInterpolation.update_forward_refs()
NumericHaganPricer.update_forward_refs()
BaroneAdesiWhaleyApproximationEngine.update_forward_refs()
SafeFritschButlandLogCubic.update_forward_refs()
FxSwapRateHelper.update_forward_refs()
SuperSharePayoff.update_forward_refs()
UniformLowDiscrepancySequenceGenerator.update_forward_refs()
ZeroSpreadedTermStructure.update_forward_refs()
GbpLiborSwapIsdaFix.update_forward_refs()
Euribor3M.update_forward_refs()
Fdm3DimSolver.update_forward_refs()
JamshidianSwaptionEngine.update_forward_refs()
AnalyticCapFloorEngine.update_forward_refs()
FDAmericanEngine.update_forward_refs()
KahaleSmileSection.update_forward_refs()
Singapore.update_forward_refs()
GaussJacobiIntegration.update_forward_refs()
France.update_forward_refs()
BatesModel.update_forward_refs()
USDLiborON.update_forward_refs()
Aonia.update_forward_refs()
KnuthUniformRng.update_forward_refs()
CalibratedModelHandle.update_forward_refs()
CeilingTruncation.update_forward_refs()
StudentDistribution.update_forward_refs()
SettlementMethod.update_forward_refs()
OptionletStripper1.update_forward_refs()
FdmDividendHandler.update_forward_refs()
ExponentialSplinesFitting.update_forward_refs()
FdHestonVanillaEngine.update_forward_refs()
FuturesRateHelper.update_forward_refs()
Merton76Process.update_forward_refs()
CumulativeChiSquareDistribution.update_forward_refs()
Bbsw.update_forward_refs()
OptionletVolatilityStructureHandle.update_forward_refs()
FdSimpleBSSwingEngine.update_forward_refs()
AUDLibor.update_forward_refs()
FdmDirichletBoundary.update_forward_refs()
GaussLegendreIntegration.update_forward_refs()
Euribor365SW.update_forward_refs()
FdBlackScholesBarrierEngine.update_forward_refs()
IsdaCdsEngineAccrualBias.update_forward_refs()
EURLibor2W.update_forward_refs()
AnalyticDiscreteGeometricAveragePriceAsianEngine.update_forward_refs()
Euribor7M.update_forward_refs()
SVD.update_forward_refs()
SwingExercise.update_forward_refs()
Euribor4M.update_forward_refs()
HullWhiteForwardProcess.update_forward_refs()
FdmHestonOp.update_forward_refs()
FDEuropeanEngine.update_forward_refs()
FdmBlackScholesFwdOp.update_forward_refs()
CubicBSplinesFitting.update_forward_refs()
FdmCEV1dMesher.update_forward_refs()
NonCentralCumulativeChiSquareDistribution.update_forward_refs()
Germany.update_forward_refs()
HestonModelHelper.update_forward_refs()
USDLibor.update_forward_refs()
Euribor.update_forward_refs()
Concentrating1dMesher.update_forward_refs()
SafeMonotonicLogParabolic.update_forward_refs()
GaussianSimulatedAnnealing.update_forward_refs()
ClosestRounding.update_forward_refs()
CapHelper.update_forward_refs()
Bbsw4M.update_forward_refs()
FdHullWhiteSwaptionEngine.update_forward_refs()
YYUKRPI.update_forward_refs()
Swap.update_forward_refs()
MaxBasketPayoff.update_forward_refs()
MarkovFunctionalModelSettings.update_forward_refs()
ConvertibleZeroCouponBond.update_forward_refs()
ExplicitEulerScheme.update_forward_refs()
SwaptionVolCube1.update_forward_refs()
EurLiborSwapIfrFix.update_forward_refs()
ForwardEuropeanEngine.update_forward_refs()
FloatingRateBond.update_forward_refs()
CumulativeGammaDistribution.update_forward_refs()
EndCriteria.update_forward_refs()
LocalVolRNDCalculator.update_forward_refs()
CumulativeNormalDistribution.update_forward_refs()
FirstDerivativeOp.update_forward_refs()
SobolRsg.update_forward_refs()
AmericanExercise.update_forward_refs()
TreeCapFloorEngine.update_forward_refs()
SwapSpreadIndex.update_forward_refs()
EurLiborSwapIsdaFixB.update_forward_refs()
FdmZabrOp.update_forward_refs()
JuQuadraticApproximationEngine.update_forward_refs()
India.update_forward_refs()
CappedFlooredCoupon.update_forward_refs()
FittedBondDiscountCurve.update_forward_refs()
SquareRootProcessRNDCalculator.update_forward_refs()
China.update_forward_refs()
DiscountFactor.update_forward_refs()
QuantoDoubleBarrierOption.update_forward_refs()
GaussChebyshev2ndIntegration.update_forward_refs()
DirichletBC.update_forward_refs()
SegmentIntegral.update_forward_refs()
FixedRateBondForward.update_forward_refs()
BatesProcess.update_forward_refs()
Wibor.update_forward_refs()
YoYInflationCollar.update_forward_refs()
EURLibor10M.update_forward_refs()
SafeMonotonicLogCubicNaturalSpline.update_forward_refs()
NoArbSabrSmileSection.update_forward_refs()
Swaption.update_forward_refs()
ImpliedTermStructure.update_forward_refs()
Euribor11M.update_forward_refs()
CumulativeStudentDistribution.update_forward_refs()
AnalyticPTDHestonEngineIntegration.update_forward_refs()
CmsSpreadCoupon.update_forward_refs()
Business252.update_forward_refs()
LocalConstantVol.update_forward_refs()
MirrorGaussianSimulatedAnnealing.update_forward_refs()
Mexico.update_forward_refs()
BivariateCumulativeNormalDistribution.update_forward_refs()
FdmHestonFwdOp.update_forward_refs()
UpRounding.update_forward_refs()
OvernightIndex.update_forward_refs()
HundsdorferScheme.update_forward_refs()
BjerksundStenslandApproximationEngine.update_forward_refs()
SafeParabolic.update_forward_refs()
Predefined1dMesher.update_forward_refs()
Fdm1dMesher.update_forward_refs()
DailyTenorLibor.update_forward_refs()
FdmSchemeDesc.update_forward_refs()
CmsMarketCalibration.update_forward_refs()
# AverageType.update_forward_refs()
MethodOfLinesScheme.update_forward_refs()
SpreadCdsHelper.update_forward_refs()
Euribor3652M.update_forward_refs()
OvernightIndexFutureRateHelper.update_forward_refs()
FdmBackwardSolver.update_forward_refs()
EuriborSwapIsdaFixB.update_forward_refs()
GaussHyperbolicIntegration.update_forward_refs()
ZACPI.update_forward_refs()
BinomialCRRVanillaEngine.update_forward_refs()
HullWhite.update_forward_refs()
# PositionType.update_forward_refs()
ConvexMonotone.update_forward_refs()
FdmBlackScholesOp.update_forward_refs()
Taiwan.update_forward_refs()
# YearOnYearInflationSwapType.update_forward_refs()
LognormalCmsSpreadPricer.update_forward_refs()
EURLibor.update_forward_refs()
AnalyticDigitalAmericanKOEngine.update_forward_refs()
GaussianQuadrature.update_forward_refs()
Bkbm3M.update_forward_refs()
KirkEngine.update_forward_refs()
DefaultProbabilityTermStructureHandle.update_forward_refs()
AmortizingPayment.update_forward_refs()
GBSMRNDCalculator.update_forward_refs()
ConvertibleFloatingRateBond.update_forward_refs()
SaudiArabia.update_forward_refs()
Libor.update_forward_refs()
CallableFixedRateBond.update_forward_refs()
Gaussian1dFloatFloatSwaptionEngine.update_forward_refs()
HongKong.update_forward_refs()
Euribor3651Y.update_forward_refs()
ChfLiborSwapIsdaFix.update_forward_refs()
FractionalDividend.update_forward_refs()
DMinus.update_forward_refs()
FdG2SwaptionEngine.update_forward_refs()
ExtOUWithJumpsProcess.update_forward_refs()
USCPI.update_forward_refs()
GaussianRandomSequenceGenerator.update_forward_refs()
Robor.update_forward_refs()
CapFloorTermVolatilityStructureHandle.update_forward_refs()
VanillaForwardPayoff.update_forward_refs()
ConstantSwaptionVolatility.update_forward_refs()
EUHICP.update_forward_refs()
Argentina.update_forward_refs()
BasketOption.update_forward_refs()
BivariateCumulativeNormalDistributionWe04DP.update_forward_refs()
AndreasenHugeLocalVolAdapter.update_forward_refs()
DiscountingBondEngine.update_forward_refs()
FdmSquareRootFwdOp.update_forward_refs()
NeumannBC.update_forward_refs()
FdmQuantoHelper.update_forward_refs()
Period.update_forward_refs()
SwapIndex.update_forward_refs()
BlackProcess.update_forward_refs()
FdmBatesOp.update_forward_refs()
LocalVolTermStructureHandle.update_forward_refs()
FlatForward.update_forward_refs()
Bibor6M.update_forward_refs()
Vasicek.update_forward_refs()
Euribor2W.update_forward_refs()
EuriborSwapIsdaFixA.update_forward_refs()
EuriborSW.update_forward_refs()
EuropeanExercise.update_forward_refs()
BlackCdsOptionEngine.update_forward_refs()
FraRateHelper.update_forward_refs()
IntegralEngine.update_forward_refs()
YieldTermStructureHandle.update_forward_refs()
YYUSCPI.update_forward_refs()
SimpleQuote.update_forward_refs()
MarkovFunctional.update_forward_refs()
Euribor3658M.update_forward_refs()
YYZACPI.update_forward_refs()
GJRGARCHProcess.update_forward_refs()
GaussLobattoIntegral.update_forward_refs()
HullWhiteProcess.update_forward_refs()
Bkbm2M.update_forward_refs()
JpyLiborSwapIsdaFixAm.update_forward_refs()
QuantoVanillaOption.update_forward_refs()
FlatSmileSection.update_forward_refs()
Bkbm5M.update_forward_refs()
IborCoupon.update_forward_refs()
ConstantOptionletVolatility.update_forward_refs()
EURLibor7M.update_forward_refs()
SobolBrownianGeneratorFactory.update_forward_refs()
G2SwaptionEngine.update_forward_refs()
FdmBoundaryConditionSet.update_forward_refs()
ForwardRateAgreement.update_forward_refs()
ForwardSpreadedTermStructure.update_forward_refs()
Money.update_forward_refs()
Italy.update_forward_refs()
VarianceGammaProcess.update_forward_refs()
LogLinearInterpolatedZeroCurve.update_forward_refs()
MonotonicCubicInterpolatedZeroCurve.update_forward_refs()
Bibor1Y.update_forward_refs()
SafeLogParabolic.update_forward_refs()
CappedFlooredCmsSpreadCoupon.update_forward_refs()
StrippedOptionletAdapter.update_forward_refs()
Indonesia.update_forward_refs()
SEKLibor.update_forward_refs()
Gaussian1dJamshidianSwaptionEngine.update_forward_refs()
SvenssonFitting.update_forward_refs()
CappedFlooredCmsCoupon.update_forward_refs()
ConstantParameter.update_forward_refs()
SecondDerivativeOp.update_forward_refs()
SafeBilinearInterpolation.update_forward_refs()
ParkinsonSigma.update_forward_refs()
SimpleCashFlow.update_forward_refs()
COSHestonEngine.update_forward_refs()
PiecewiseConstantParameter.update_forward_refs()
AnalyticDoubleBarrierEngine.update_forward_refs()
SpreadBasketPayoff.update_forward_refs()
Sofr.update_forward_refs()
BinomialTrigeorgisVanillaEngine.update_forward_refs()
CHFLibor.update_forward_refs()
Euribor3656M.update_forward_refs()
DefaultLogCubicInterpolatedZeroCurve.update_forward_refs()
ZeroCouponBond.update_forward_refs()
InverseCumulativePoisson.update_forward_refs()
AndreasenHugeVolatilityAdapter.update_forward_refs()
Cap.update_forward_refs()
GarmanKlassSigma5.update_forward_refs()
FloatFloatSwaption.update_forward_refs()
SafeForwardFlatInterpolation.update_forward_refs()
InverseNonCentralCumulativeChiSquareDistribution.update_forward_refs()
BlackVolTermStructureHandle.update_forward_refs()
FixedDividend.update_forward_refs()
Callability.update_forward_refs()
EURLibor8M.update_forward_refs()
HestonSLVProcess.update_forward_refs()
Jibar.update_forward_refs()
FdBlackScholesVanillaEngine.update_forward_refs()
BlackCapFloorEngine.update_forward_refs()
VarianceGammaEngine.update_forward_refs()
Euribor3651M.update_forward_refs()
Euribor1M.update_forward_refs()
Euribor3653W.update_forward_refs()
SecondOrderMixedDerivativeOp.update_forward_refs()
NonhomogeneousBoundaryConstraint.update_forward_refs()
OISRateHelper.update_forward_refs()
SwapRateHelper.update_forward_refs()
SamplerGaussian.update_forward_refs()
FdmCEVOp.update_forward_refs()
Euribor3652W.update_forward_refs()
CraigSneydScheme.update_forward_refs()
PoissonDistribution.update_forward_refs()
BlackVarianceCurve.update_forward_refs()
SoftCallability.update_forward_refs()
BiborSW.update_forward_refs()
ShortRateModelHandle.update_forward_refs()
GapPayoff.update_forward_refs()
LecuyerUniformRng.update_forward_refs()
CPIBond.update_forward_refs()
TripleBandLinearOp.update_forward_refs()
GsrProcess.update_forward_refs()
Euribor6M.update_forward_refs()
Nzocr.update_forward_refs()
YYEUHICPXT.update_forward_refs()
Euribor1Y.update_forward_refs()
BinomialJRVanillaEngine.update_forward_refs()
IntegralCdsEngine.update_forward_refs()
SafeMonotonicParabolic.update_forward_refs()
Bkbm.update_forward_refs()
TRLibor.update_forward_refs()
FdmSimpleProcess1dMesher.update_forward_refs()
ConstantEstimator.update_forward_refs()
Russia.update_forward_refs()
Euribor3653M.update_forward_refs()
BlackDeltaCalculator.update_forward_refs()
GaussHermiteIntegration.update_forward_refs()
GaussChebyshevIntegration.update_forward_refs()
BrownianBridge.update_forward_refs()
BondHelper.update_forward_refs()
YoYInflationCap.update_forward_refs()
DatedOISRateHelper.update_forward_refs()
HestonModel.update_forward_refs()
EuropeanOption.update_forward_refs()
Actual365Fixed.update_forward_refs()
Euribor36511M.update_forward_refs()
YYFRHICP.update_forward_refs()
BachelierSwaptionEngine.update_forward_refs()
UsdLiborSwapIsdaFixPm.update_forward_refs()
Euribor3655M.update_forward_refs()
CdsOption.update_forward_refs()
IsdaCdsEngineForwardsInCouponPeriod.update_forward_refs()
DownRounding.update_forward_refs()
FaceValueAccrualClaim.update_forward_refs()
BlackKarasinski.update_forward_refs()
Euribor3W.update_forward_refs()
Gsr.update_forward_refs()
ModifiedCraigSneydScheme.update_forward_refs()
SamplerLogNormal.update_forward_refs()
IsdaCdsEngine.update_forward_refs()
ProbabilityBoltzmannDownhill.update_forward_refs()
Gaussian1dSwaptionEngine.update_forward_refs()
SafeCubicNaturalSpline.update_forward_refs()
BoundaryConstraint.update_forward_refs()
TridiagonalOperator.update_forward_refs()
Thirty360.update_forward_refs()
GJRGARCHModel.update_forward_refs()
AmortizingFixedRateBond.update_forward_refs()
NelsonSiegelFitting.update_forward_refs()
G2Process.update_forward_refs()
VanillaSwingOption.update_forward_refs()
BlackSwaptionEngine.update_forward_refs()
CapFloorTermVolSurface.update_forward_refs()
WulinYongDoubleBarrierEngine.update_forward_refs()
Gaussian1dNonstandardSwaptionEngine.update_forward_refs()
FloatFloatSwap.update_forward_refs()
EURLibor3M.update_forward_refs()
BatesEngine.update_forward_refs()
FdmMesherComposite.update_forward_refs()
TimeBasket.update_forward_refs()
OvernightIndexedCoupon.update_forward_refs()
AnalyticContinuousGeometricAveragePriceAsianEngine.update_forward_refs()
Uniform1dMesher.update_forward_refs()
GaussianLowDiscrepancySequenceGenerator.update_forward_refs()
FdHestonDoubleBarrierEngine.update_forward_refs()
Euribor9M.update_forward_refs()
FdmHestonLocalVolatilityVarianceMesher.update_forward_refs()
Actual360.update_forward_refs()
AmortizingFloatingRateBond.update_forward_refs()
CallabilityPrice.update_forward_refs()
InverseCumulativeNormal.update_forward_refs()
StochasticProcessArray.update_forward_refs()
SafeLinearInterpolation.update_forward_refs()
FdmOrnsteinUhlenbeckOp.update_forward_refs()
Floor.update_forward_refs()
AndreasenHugeVolatilityInterplCalibrationType.update_forward_refs()
EURLibor1M.update_forward_refs()
StulzEngine.update_forward_refs()
AnalyticCEVEngine.update_forward_refs()
YearOnYearInflationSwap.update_forward_refs()
DoubleBarrierOption.update_forward_refs()
PiecewiseTimeDependentHestonModel.update_forward_refs()
MultiplicativePriceSeasonality.update_forward_refs()
FdmSimpleStorageCondition.update_forward_refs()
DividendVanillaOption.update_forward_refs()
FdmDupire1dOp.update_forward_refs()
ExchangeRate.update_forward_refs()
CumulativePoissonDistribution.update_forward_refs()
FFTVarianceGammaEngine.update_forward_refs()
UpfrontCdsHelper.update_forward_refs()
NinePointLinearOp.update_forward_refs()
FdmSnapshotCondition.update_forward_refs()
ContinuousArithmeticAsianLevyEngine.update_forward_refs()
UnitedStates.update_forward_refs()
JPYLibor.update_forward_refs()
Fdm1DimSolver.update_forward_refs()
Euribor8M.update_forward_refs()
BlackIborCouponPricer.update_forward_refs()
FdmBlackScholesMesher.update_forward_refs()
FdmHestonHullWhiteOp.update_forward_refs()
AnalyticDiscreteGeometricAverageStrikeAsianEngine.update_forward_refs()
HestonModelHandle.update_forward_refs()
MTBrownianGeneratorFactory.update_forward_refs()
SafeConvexMonotoneInterpolation.update_forward_refs()
Bbsw6M.update_forward_refs()
Slovakia.update_forward_refs()
FDShoutEngine.update_forward_refs()
Bibor2M.update_forward_refs()
MoroInverseCumulativeNormal.update_forward_refs()
SafeKrugerLogCubic.update_forward_refs()
BlackConstantVol.update_forward_refs()
ZeroCurve.update_forward_refs()
AndreasenHugeVolatilityInterpl.update_forward_refs()
AnalyticHestonEngineIntegration.update_forward_refs()
MakeOIS.update_forward_refs()
YYEUHICP.update_forward_refs()
THBFIX.update_forward_refs()
NZDLibor.update_forward_refs()
FdmIndicesOnBoundary.update_forward_refs()
SampledCurve.update_forward_refs()
GeometricBrownianMotionProcess.update_forward_refs()
FdmBermudanStepCondition.update_forward_refs()
NthOrderDerivativeOp.update_forward_refs()
LocalVolSurface.update_forward_refs()
IborIndex.update_forward_refs()
BinomialDistribution.update_forward_refs()
GarmanKlassSigma3.update_forward_refs()
QuantoForwardEuropeanEngine.update_forward_refs()
IterativeBootstrap.update_forward_refs()
UniformRandomGenerator.update_forward_refs()
AssetSwap.update_forward_refs()
CmsMarket.update_forward_refs()
ImplicitEulerScheme.update_forward_refs()
Bbsw3M.update_forward_refs()
SwaptionVolatilityMatrix.update_forward_refs()
FdmSabrOp.update_forward_refs()
Fd2dBlackScholesVanillaEngine.update_forward_refs()
FixedRateBond.update_forward_refs()
VannaVolgaBarrierEngine.update_forward_refs()
Euribor3654M.update_forward_refs()
SabrSmileSection.update_forward_refs()
KlugeExtOUProcess.update_forward_refs()
LogNormalSimulatedAnnealing.update_forward_refs()
DKKLibor.update_forward_refs()
YoYInflationTermStructureHandle.update_forward_refs()
RebatedExercise.update_forward_refs()
FlatHazardRate.update_forward_refs()
HestonRNDCalculator.update_forward_refs()
Collar.update_forward_refs()
CubicInterpolatedZeroCurve.update_forward_refs()
Shibor.update_forward_refs()
CEVRNDCalculator.update_forward_refs()
FdmDiscountDirichletBoundarySide.update_forward_refs()
SafeFritschButlandCubic.update_forward_refs()
DefaultBoundaryConditionSide.update_forward_refs()
PlainVanillaPayoff.update_forward_refs()
FdmAmericanStepCondition.update_forward_refs()
UsdLiborSwapIsdaFixAm.update_forward_refs()
FdmDiscountDirichletBoundary.update_forward_refs()
VanillaOption.update_forward_refs()
ZeroCouponInflationSwap.update_forward_refs()
Pribor.update_forward_refs()
Exercise.update_forward_refs()
BachelierCapFloorEngine.update_forward_refs()
Euribor3657M.update_forward_refs()
Bbsw5M.update_forward_refs()
CADLibor.update_forward_refs()
Euribor3659M.update_forward_refs()
Glued1dMesher.update_forward_refs()
DeltaVolQuoteHandle.update_forward_refs()
EURLibor9M.update_forward_refs()
AnalyticPTDHestonEngine.update_forward_refs()
FloorTruncation.update_forward_refs()
CmsCoupon.update_forward_refs()
CPISwap.update_forward_refs()
SafeLogLinearInterpolation.update_forward_refs()
QuantoForwardVanillaOption.update_forward_refs()
Euribor36510M.update_forward_refs()
BlackCalculator.update_forward_refs()
OvernightIndexedSwap.update_forward_refs()
Stock.update_forward_refs()
HaltonRsg.update_forward_refs()
Tibor.update_forward_refs()
DiscountingSwapEngine.update_forward_refs()
Bibor1M.update_forward_refs()
SobolBrownianBridgeRsg.update_forward_refs()
GarmanKlassSigma4.update_forward_refs()
ConvertibleFixedCouponBond.update_forward_refs()
InterestRate.update_forward_refs()
Cdor.update_forward_refs()
FdmSimpleSwingCondition.update_forward_refs()
G2ForwardProcess.update_forward_refs()
UnitedKingdom.update_forward_refs()
LevenbergMarquardt.update_forward_refs()
Simplex.update_forward_refs()
AnalyticHaganPricer.update_forward_refs()
CzechRepublic.update_forward_refs()
ExponentialJump1dMesher.update_forward_refs()
ZeroInflationTermStructureHandle.update_forward_refs()
ActualActual.update_forward_refs()
Brazil.update_forward_refs()
EUHICPXT.update_forward_refs()
DouglasScheme.update_forward_refs()
Schedule.update_forward_refs()
CustomRegion.update_forward_refs()
MakeVanillaSwap.update_forward_refs()
SafeKrugerCubic.update_forward_refs()
Bbsw2M.update_forward_refs()
TreeCallableFixedRateBondEngine.update_forward_refs()
SimpsonIntegral.update_forward_refs()
DPlusDMinus.update_forward_refs()
CapFloorTermVolCurve.update_forward_refs()
Redemption.update_forward_refs()
JpyLiborSwapIsdaFixPm.update_forward_refs()
HimalayaOption.update_forward_refs()
SwaptionVolatilityStructureHandle.update_forward_refs()
FdmArithmeticAverageCondition.update_forward_refs()
CrankNicolsonScheme.update_forward_refs()
TimeGrid.update_forward_refs()
Fdm2dBlackScholesOp.update_forward_refs()
BinomialTianVanillaEngine.update_forward_refs()
EurLiborSwapIsdaFixA.update_forward_refs()
MersenneTwisterUniformRng.update_forward_refs()
FdmLogBasketInnerValue.update_forward_refs()
SafeMonotonicCubicNaturalSpline.update_forward_refs()
AverageBasketPayoff.update_forward_refs()
GeneralizedBlackScholesProcess.update_forward_refs()
BivariateCumulativeNormalDistributionDr78.update_forward_refs()
FixedRateCoupon.update_forward_refs()
TreeSwaptionEngine.update_forward_refs()
SamplerMirrorGaussian.update_forward_refs()
AndreasenHugeVolatilityInterplCalibrationSet.update_forward_refs()
GaussLaguerreIntegration.update_forward_refs()
FdmHestonVarianceMesher.update_forward_refs()
FdCEVVanillaEngine.update_forward_refs()
AnalyticHestonEngine.update_forward_refs()
CompositeConstraint.update_forward_refs()
AnalyticBarrierEngine.update_forward_refs()
GarmanKlassSigma6.update_forward_refs()
Iceland.update_forward_refs()
CappedFlooredIborCoupon.update_forward_refs()
EURLibor4M.update_forward_refs()
Bibor9M.update_forward_refs()
DiscreteAveragingAsianOption.update_forward_refs()
BarrierOption.update_forward_refs()
SofrFutureRateHelper.update_forward_refs()
Matrix.update_forward_refs()
NormalDistribution.update_forward_refs()
Gaussian1dNonstandardSwaptionEngineProbabilities.update_forward_refs()
GBPLibor.update_forward_refs()
Euribor2M.update_forward_refs()
FRHICP.update_forward_refs()
Bbsw1M.update_forward_refs()
Bkbm4M.update_forward_refs()
GaussGegenbauerIntegration.update_forward_refs()
EURLibor5M.update_forward_refs()
Bkbm1M.update_forward_refs()
FdOrnsteinUhlenbeckVanillaEngine.update_forward_refs()
Sonia.update_forward_refs()
FixedRateBondHelper.update_forward_refs()
EuriborSwapIfrFix.update_forward_refs()
QuoteHandle.update_forward_refs()
BespokeCalendar.update_forward_refs()
Euribor365.update_forward_refs()
UKRPI.update_forward_refs()
Zibor.update_forward_refs()
Eonia.update_forward_refs()
MinBasketPayoff.update_forward_refs()
DeltaVolQuote.update_forward_refs()
Bkbm6M.update_forward_refs()
DZero.update_forward_refs()
Israel.update_forward_refs()
IntervalPrice.update_forward_refs()
GlobalBootstrap.update_forward_refs()
NoArbSabrInterpolatedSmileSection.update_forward_refs()
EURLiborSW.update_forward_refs()
Euribor5M.update_forward_refs()
GaussianRandomGenerator.update_forward_refs()
DPlus.update_forward_refs()
ForwardVanillaOption.update_forward_refs()
HestonBlackVolSurface.update_forward_refs()
OvernightIndexedSwapIndex.update_forward_refs()
EverestOption.update_forward_refs()
GBPLiborON.update_forward_refs()
EURLibor1Y.update_forward_refs()
FdSabrVanillaEngine.update_forward_refs()
GarmanKlassSigma1.update_forward_refs()
Bibor.update_forward_refs()
NonstandardSwap.update_forward_refs()
SwaptionHelper.update_forward_refs()
LinearTsrPricer.update_forward_refs()
CashFlows.update_forward_refs()
Date.update_forward_refs()
BlackScholesMertonProcess.update_forward_refs()
EURLibor2M.update_forward_refs()
SouthKorea.update_forward_refs()
YoYInflationFloor.update_forward_refs()
Leg.update_forward_refs()
Canada.update_forward_refs()
FdmG2Op.update_forward_refs()
FdHestonBarrierEngine.update_forward_refs()
EURLibor6M.update_forward_refs()
Fdm2DimSolver.update_forward_refs()
SimplePolynomialFitting.update_forward_refs()
G2.update_forward_refs()
FdBlackScholesAsianEngine.update_forward_refs()
SwaptionVolCube2.update_forward_refs()
FdmLocalVolFwdOp.update_forward_refs()
CADLiborON.update_forward_refs()
EURLibor11M.update_forward_refs()
CashOrNothingPayoff.update_forward_refs()
GaussKronrodAdaptive.update_forward_refs()
VolatilityType.update_forward_refs()
AnalyticDividendEuropeanEngine.update_forward_refs()
GaussKronrodNonAdaptive.update_forward_refs()
BlackCallableFixedRateBondEngine.update_forward_refs()
AnalyticGJRGARCHEngine.update_forward_refs()
OrnsteinUhlenbeckProcess.update_forward_refs()
CmsRateBond.update_forward_refs()
SafeLogCubicNaturalSpline.update_forward_refs()
Array.update_forward_refs()
QuantoEuropeanEngine.update_forward_refs()
FdBatesVanillaEngine.update_forward_refs()
AssetOrNothingPayoff.update_forward_refs()
JointCalendar.update_forward_refs()
FdmLogInnerValue.update_forward_refs()
BinomialJoshi4VanillaEngine.update_forward_refs()
AnalyticDigitalAmericanEngine.update_forward_refs()
AnalyticBinaryBarrierEngine.update_forward_refs()
DepositRateHelper.update_forward_refs()
HestonProcess.update_forward_refs()
AnalyticDoubleBarrierBinaryEngine.update_forward_refs()
